package com.japisoft.datasource.filesystem;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import com.japisoft.datasource.DataSource;
import com.japisoft.datasource.DataSourceContainer;
import com.japisoft.datasource.DataSourceItem;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FileSystemDataSourceContainer extends FileSystemDataSource
		implements DataSourceContainer {

	public FileSystemDataSourceContainer( File f ) {
		super( f );
	}

	public FileSystemDataSourceContainer( String f ) {
		super( f );
	}
	
	public DataSourceItem createItem( String name ) throws Exception {
		return new FileSystemDataSourceItem( new File( f, name ) );
	}
	
	public DataSourceContainer createContainer(String name) throws Exception {
		File ff = new File( f, name );
		ff.mkdir();
		return new FileSystemDataSourceContainer( ff );
	}

	public List<DataSource> list() throws Exception {
		ArrayList<DataSource> r = new ArrayList<DataSource>();
		String[] content = f.list();
		if ( content != null ) {
			for ( String c : content ) {
				File ff = new File( f, c );
				if ( ff.isDirectory() ) {
					r.add( new FileSystemDataSourceContainer( ff ) );
				} else
				if ( ff.isFile() ) {
					r.add( new FileSystemDataSourceItem( ff ) );
				}
			}
		}
		return r;
	}

}
