package com.japisoft.editix.action.dtdschema;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.SwingUtilities;

import com.japisoft.editix.editor.xsd.CustomActionListener;
import com.japisoft.editix.editor.xsd.VisualXSDEditor;
import com.japisoft.editix.editor.xsd.view2.DesignerViewImpl;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.framework.ui.toolkit.FileManager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ExportImageFromSchemaEditorAction implements CustomActionListener {

	public void action(String name, VisualXSDEditor editor) {

		File file = 
			FileManager.getSelectedFile(
					false,
					new String[] { "jpg", "png" },
					new String[] { "Image (*.jpg)", "Image (*.png)" } 
			);

		if ( file != null ) {
			
			String imageType = "JPG";
			
			if ( file.toString().endsWith( "png" ) )
				imageType = "PNG";

				
				DesignerViewImpl dvi = 
					editor.getDesignerView();

				BufferedImage img = new BufferedImage(
						dvi.getWidth(),
						dvi.getHeight(),
						BufferedImage.TYPE_INT_RGB );

				Graphics2D gc = ( Graphics2D )img.getGraphics();
				
				gc.setRenderingHint(
	                    RenderingHints.KEY_ANTIALIASING, 
	                    RenderingHints.VALUE_ANTIALIAS_ON);				
				
				gc.setColor(Color.WHITE);
				gc.fillRect(0, 0, img.getWidth(), img.getHeight());

				dvi.paint( img.getGraphics() );
				
				try {				
					ImageIO.write( 
						img, 
						imageType, 
						file );
				} catch ( IOException e ) {				
					EditixFactory.buildAndShowErrorDialog( 
							"Can't write : " + 
							e.getMessage() 
					);
				}
		}
		
	}	
	
}

