package com.japisoft.editix.action.dtdschema;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;

import com.japisoft.editix.action.xml.format.FormatAction;
import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.ui.toolkit.BrowserCaller;
import com.japisoft.framework.xml.dtd.instance.DTDInstanceGenerator;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.xsd.instance.XSDInstanceGenerator;
import com.japisoft.p3.Manager;
import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.XMLPadDocument;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class GenerateTemplateFromElementAction extends AbstractAction {

	public void actionPerformed(ActionEvent e) {

		if ( Manager.isFree() ) {
			EditixFactory.buildAndShowInformationDialog( 
				"This action is not available inside the Free Edition.\nPlease look at http://www.editix.com" );
			BrowserCaller.displayURL( 
				"http://www.editix.com" );
			return;
		}

		XMLContainer container = EditixFrame.THIS.getSelectedContainer();

		if ( container.getCurrentDocumentLocation() == null ) {
			EditixFactory.buildAndShowWarningDialog( "Please save your dtd/schema before generating a new XML document" );
			return;
		}

		String type = container.getDocumentInfo().getType();

		if ( "DTD".equals( type ) ) {

			XMLPadDocument doc = container.getXMLDocument();
			String root = doc.getDTDElementDefinitionFor( container.getCaretPosition() );
			if ( root == null ) {
				EditixFactory.buildAndShowErrorDialog( "Select a line with an element definition" );
			} else {
				
				try {

					generateXMLDocumentFromDTD( root, container.getCurrentDocumentLocation() );					

				} catch ( Throwable e1 ) {

					EditixApplicationModel.debug( e1 );
					EditixFactory.buildAndShowErrorDialog( "Can't build this XML document" );

				}

			}
		} else
			if ( "XSD".equals( type ) ) {
				
				FPNode node = container.getCurrentElementNode();
						
				try {

					generateXMLDocumentFromXSD( node, container.getCurrentDocumentLocation() );
					
				} catch ( Throwable e1 ) {

					EditixApplicationModel.debug( e1 );
					EditixFactory.buildAndShowErrorDialog( "Can't build this XML document" );

				}

			}		
	}

	static void generateXMLDocumentFromDTD( String root, String location ) throws Throwable {

		String xml = DTDInstanceGenerator.generateXMLInstance(
				root, location, null );
		IXMLPanel panel = EditixFactory.buildNewContainer();

		xml = "<?xml version='1.0' encoding='UTF-8'?>\n" +
			"\n<!DOCTYPE " + root + " SYSTEM " + "\"" + location + "\">\n" +
				xml;

		panel.getMainContainer().setText( xml );

		FormatAction.format( 
				panel.getMainContainer(), 
				null, 
				null, 
				"silence" );					

		EditixFrame.THIS.addContainer( panel );
		
	}
	
	static void generateXMLDocumentFromXSD( FPNode node, String location ) throws Throwable {

		String name = null;

		while ( node != null && ( name == null ) ) {
		
			if ( node.matchContent( "element"  ) ) {
				name = node.getAttribute( "name" );
				if ( name == null )
					name = node.getAttribute( "ref" );
			}
			
			node = node.getFPParent();
			
		}

		if ( name == null || "".equals( name ) ) {
			throw new Exception( "Can't find a name in this element definition ?" );	
		}

		String sTmp = XSDInstanceGenerator.generateXMLInstance( 
				name, 
				location
		);

		FPNode rootNode = ( FPNode )node.getDocument().getRoot();
		String target = null;

		if ( rootNode.hasAttribute( "targetNamespace" ) )
			target = rootNode.getAttribute( "targetNamespace" );
		
		int i = sTmp.indexOf( "<" + name );
		if ( i > -1 ) {
			
			sTmp = sTmp.substring( 0, i + 1 + name.length() ) + sTmp.substring( i + 1 + name.length() );

		}

		String xml = "<?xml version='1.0' encoding='UTF-8'?>\n";
		xml += sTmp;

		IXMLPanel panel = EditixFactory.buildNewContainer();
		panel.getMainContainer().setText( xml );

		FormatAction.format( 
				panel.getMainContainer(), 
				null, 
				null, 
				"silence" );					

		EditixFrame.THIS.addContainer( panel );

	}
	
}
