package com.japisoft.editix.action.dtdschema;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.StringReader;
import java.net.URL;

import javax.swing.AbstractAction;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import com.japisoft.editix.action.xml.EditixValidator;
import com.japisoft.editix.action.xsl.XSLT2Action;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.ui.toolkit.FileManager;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.xml.validator.DefaultValidator;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SchematronAction extends AbstractAction {

	public static int checkSchematron( XMLContainer container, File f, boolean messageMode ) {
		
		try {
			
			// Transform the schematron file with XSLT
			URL u = ClassLoader
					.getSystemResource( "schematron/iso_svrl_for_xslt2.xsl" );
			javax.xml.transform.TransformerFactory factory = XSLT2Action.getTransformerFactoryV2( false );
			Transformer t = factory.newTransformer( 
				new StreamSource( u.toString() )
			);
			DOMResult dr = null;
			t.transform( new StreamSource( f ), dr = new DOMResult() );
			
			// Apply the result to the source document
			DocumentBuilder db = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			Document doc = db.parse( new InputSource( new StringReader( container.getText()  ) ) );
			
			t = factory.newTransformer( new DOMSource( dr.getNode() ) );
			
			DOMResult schematronRes = null;
			
			t.transform( new DOMSource( doc ), schematronRes = new DOMResult() );
			Document docRes = ( Document )schematronRes.getNode();
			NodeList nl = docRes.getElementsByTagNameNS( "http://purl.oclc.org/dsdl/svrl", "*" );

			if ( nl.getLength() > 0 ) {
			
				container.getErrorManager().initErrorProcessing();
				
				boolean errorFound = false;
				
				for ( int i = 0; i < nl.getLength(); i++ ) {
					Element ass = ( Element )nl.item( i );
					
					if ( !( "failed-assert".equals( ass.getLocalName() ) || "successful-report".equals( ass.getLocalName() ) ) )
						continue;
					
					String test = ass.getAttribute( "test" );
					String location = ass.getAttribute( "location" );
					// Test on doc location
					
					String message = test;
					
					NodeList nl2 = ass.getChildNodes();
					for ( int j = 0; j < nl2.getLength(); j++ ) {
						if ( nl2.item( j ) instanceof Element ) {
							Element ee = ( Element )nl2.item( j );
							if ( "text".equals( ee.getLocalName() ) ) {
								message = ee.getTextContent();
							}
						}
					}

					message = ass.getLocalName() + ":" + message;
					
					int line = 0;
					
					FPNode root = container.getRootNode();
					if ( root != null ) {
						FPNode result = root.getNodeForXPathLocation( location, true );
						if ( result != null ) {
							line =  result.getStartingLine();
						}
					}
					
					container.getErrorManager().notifyError( null, true, null, line, 0, 0, message, false );
				}

				container.getErrorManager().stopErrorProcessing();

				if ( container.getErrorManager().hasLastError() )
					return EditixValidator.ERROR;
				
				return EditixValidator.OK;
				
			} else {

				container.getErrorManager().notifyNoError( false );
				if ( messageMode )
					EditixFactory.buildAndShowInformationDialog( "Your document is correct" );
				
				return EditixValidator.OK;
				
			}

			/*
			<svrl:schematron-output xmlns:xhtml="http://www.w3.org/1999/xhtml"
                    xmlns:xs="http://www.w3.org/2001/XMLSchema"
                    xmlns:schold="http://www.ascc.net/xml/schematron"
                    xmlns:iso="http://purl.oclc.org/dsdl/schematron"
                    xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                    schemaVersion=""
                    title=""><!--  --><svrl:active-pattern document="" name="CELL"/>
				   <svrl:fired-rule context="table/cell"/>
				   <svrl:failed-assert test="not(p)" location="/table/cell[1]">
				      <svrl:text>P manquant</svrl:text>
				   </svrl:failed-assert>
				   <svrl:fired-rule context="table/cell"/>
				   <svrl:failed-assert test="not(p)" location="/table/cell[2]">
				      <svrl:text>P manquant</svrl:text>
				   </svrl:failed-assert>
				   <svrl:fired-rule context="table/cell"/>
				</svrl:schematron-output>
			*/
			
		} catch( Exception exc ) {

			container.getErrorManager().notifyUniqueError( false, f.toString(), 0, 0, 0, "Invalid schematron : " + exc.getMessage(), false );
			return DefaultValidator.ERROR;
			
		}
		
		
	}
	
	@Override
	public void actionPerformed( ActionEvent e ) {

		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container != null ) {

			File f = FileManager.getSelectedFile( 
				true,
				"sch",
				"Schematron file"
			);
			if ( f != null ) {
				
				checkSchematron( container, f, true );

			}

		}

	}

}
