package com.japisoft.editix.action.dtdschema.generator.transformer;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import com.japisoft.editix.action.dtdschema.generator.MetaAttribute;
import com.japisoft.editix.action.dtdschema.generator.MetaNode;
import com.japisoft.editix.action.dtdschema.generator.Transformer;
import com.japisoft.xmlpad.action.ActionModel;
import com.japisoft.xmlpad.action.Properties;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TemplateTransformer extends AbstractTransformer {

	public String transform(MetaNode root, ArrayList nodeCollection ) {
		StringBuffer sb = new StringBuffer();
		transform( 0, root, nodeCollection, sb );
		return sb.toString();
	}

	public boolean hasVersion() { return true; }

	void transform( 
			int deep, 
			MetaNode node, 
			ArrayList nodeCollection,  
			StringBuffer sb ) {
		if ( node.getChildren().size() > 0 ) {
			addRootHeader( deep, node, sb, false );
			
			Character indentChr = (Character) ActionModel.getProperty( 
					ActionModel.FORMAT_ACTION,
					Properties.INDENT_CHAR_PROPERTY, 
					new Character( ' ' ) );
			
			List<MetaNode> l = node.getChildren();
			for ( int i = 0; i < l.size(); i++ ) {
				transform( deep + 1, ( MetaNode )l.get( i  ), nodeCollection, sb );	
			}
			addRootFooter( deep, node, sb );
		} else {
			if ( node.acceptText() ) {
				addRootHeader( deep, node, sb, false );
				addRootFooter( deep, node, sb );
			} else
			addRootHeader( deep, node, sb, true );
		}
	}

	private void indent( int number, StringBuffer sb ) {
		Character indentChr = (Character) ActionModel.getProperty( 
			ActionModel.FORMAT_ACTION,
			Properties.INDENT_CHAR_PROPERTY, 
			new Character( ' ' ) );

		Integer indentNb = ( Integer )ActionModel.getProperty(
			ActionModel.FORMAT_ACTION,
			Properties.INDENT_SIZE_PROPERTY,
			new Integer( 1 ) );
			
		for ( int i = 0; i < number * indentNb.intValue(); i++ )
			sb.append( indentChr );
	}

	void addRootHeader( int indent, MetaNode root, StringBuffer sb, boolean close ) {
		sb.append( System.getProperty( "line.separator" ) );
		indent( indent, sb );
		sb.append( "<" ).append( root.getName() );
		if ( root.hasAttributes() ) {
			sb.append( " " );

			Vector list = root.getAttributes();
			for ( int i = 0; i < list.size(); i++ ) {
				if ( i > 0 )
					sb.append( " " );
				sb.append( ( (MetaAttribute)list.get( i ) ).getName() ).append( "=").append( "\"\"" );
			}
			if ( close )
				sb.append( "/>" );
			else
				sb.append( ">" );
		} else
			if ( close )
				sb.append( "/>" );
			else
				sb.append( ">" );
	}

	void addRootFooter( int indent, MetaNode root, StringBuffer sb ) {
		sb.append( System.getProperty( "line.separator" ) );
		indent( indent, sb );
		sb.append( "</" ).append( root.getName() ).append( ">" );
	}

	public String getType() {
		return "XML";
	}

}
