package com.japisoft.editix.action.file;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JEditorPane;

import com.japisoft.editix.ui.EditixEditorFrame;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class PrintAction extends AbstractAction {

	public void actionPerformed(ActionEvent e) {
		IXMLPanel panel = EditixFrame.THIS.getSelectedPanel();
		if ( panel == null )
			return;
		
		/*
		
		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null )
			return;
		DocumentRenderer dr = new DocumentRenderer();
		dr.print( container.getEditor() );
		*/
		
		Object toPrint = panel.print();
		
		if ( toPrint instanceof XMLContainer ) {
			toPrint = ( ( XMLContainer )toPrint ).getEditor();
		}

		if ( toPrint instanceof JEditorPane ) {
			DocumentRenderer dr = new DocumentRenderer();
			dr.print( ( JEditorPane )toPrint );
		} else
			if ( toPrint instanceof JComponent ) {
				PrinterJob pj = PrinterJob.getPrinterJob();
				pj.setPrintable( new ComponentPrintable( ( JComponent )toPrint ), pj.defaultPage() );
				try {
					pj.print();
				} catch( PrinterException pe ) {
					EditixFactory.buildAndShowErrorDialog( pe.getMessage() );
				}
			}
	}

	private class ComponentPrintable implements Printable {
		private JComponent component;
		ComponentPrintable( JComponent component ) {
			this.component = component;
		}
		
		@Override
		public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException {			
			if (pageIndex > 0)
			      return NO_SUCH_PAGE;
			Graphics2D g2 = (Graphics2D) graphics;
			g2.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
			component.setDoubleBuffered( false );
			component.paint( graphics );
			component.setDoubleBuffered( true );
			return 0;
		}
		
	}
	
}
