package com.japisoft.editix.action.file;

import java.awt.event.ActionEvent;
import java.util.Iterator;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.japisoft.editix.toolkit.Toolkit;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.EditixStatusBar;
import com.japisoft.editix.ui.panels.universalbrowser.CommonUniversalBrowserPanel;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.application.descriptor.InterfaceBuilder;
import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.action.ActionModel;
import com.japisoft.xmlpad.action.XMLAction;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SaveAction extends AbstractAction {

	public void actionPerformed( ActionEvent e ) {
		if ( EditixFrame.THIS.isEmpty() )
			return;

		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null )
			return;
		save( container );
	}
	
	public boolean save( XMLContainer container ) {
		
		SaveActionDelegate sad = null;
		
		if ( container.hasProperty( "save.delegate" ) ) {
			sad = ( SaveActionDelegate )container.getProperty( "save.delegate" );
		}

		String currentEncoding = ( String )container.getProperty( "encoding" );
		if ( currentEncoding == null )
			currentEncoding = Toolkit.getCurrentFileEncoding(); 
		
		// Reset the file encoding
		ActionModel.setProperty( ActionModel.SAVE_ACTION, 
			com.japisoft.xmlpad.action.file.SaveAction.ENCODING_PROPERTY,
			currentEncoding );

		// Reset the file encoding
		ActionModel.setProperty( ActionModel.SAVEAS_ACTION, 
			com.japisoft.xmlpad.action.file.SaveAction.ENCODING_PROPERTY,
			currentEncoding );

		boolean saveOk = ( sad != null && sad.save( container ) );
		if ( !saveOk && sad == null )
			saveOk = save_action();
		
		if ( saveOk ) {
			com.japisoft.framework.application.descriptor.ActionModel.LAST_ACTION_STATE = true;
			
			EditixFrame.THIS.refreshCurrentTabName();
			EditixStatusBar.ACCESSOR.setDelayedMessage( "File " + container.getCurrentDocumentLocation() + " saved..." );
			
			ApplicationModel.fireApplicationValue( "save", container );

		} else
			com.japisoft.framework.application.descriptor.ActionModel.LAST_ACTION_STATE = false;
		return saveOk;
	}

	static boolean save_action() {
		return save_action( false );
	}

	static boolean save_action( boolean saveAs ) {
		IXMLPanel panel = EditixFrame.THIS.getSelectedPanel();
		if ( panel != null )
			panel.prepareToSave();
		
		if ( !saveAs ) {
			// Check for VFS case
			XMLContainer container = EditixFrame.THIS.getSelectedContainer();
			if ( container != null ) {
				String fileName = container.getDocumentInfo().getCurrentDocumentLocation();
				if ( fileName != null && ( fileName.indexOf( "!/" ) > -1 || fileName.startsWith( "ftp:/" ) ) ) {
					return CommonUniversalBrowserPanel.save( container );
				}
			}
		}

		boolean ok = ActionModel.activeActionByName( 
				saveAs ? ActionModel.SAVEAS_ACTION :
				ActionModel.SAVE_ACTION ) == 
			XMLAction.VALID_ACTION;
		// Save in the menu history
		if ( ok ) {
			XMLContainer container = EditixFrame.THIS.getSelectedContainer();
			if ( container != null ) {
				Action a = new OpenAction();
				String fileName = container.getDocumentInfo().getCurrentDocumentLocation();
				String type = container.getDocumentInfo().getType();
				a.putValue( Action.NAME, fileName );
				a.putValue( "param", fileName );
				if ( container.getDocumentInfo().getDocumentIconPath() != null )
					a.putValue( "iconPath", container.getDocumentInfo().getDocumentIconPath() );
				if ( type != null )
					a.putValue( "param2", type );
				String encoding = ( String )container.getProperty( "encoding" );
				if ( encoding != null )
					a.putValue( "param3", encoding );

				// Get the param / key values
				StringBuffer sb = null;
				Iterator it = container.getProperties();
				if ( it != null ) {
					while ( it.hasNext() ) {
						String property = ( String )it.next();
						Object v = container.getProperty( property );
						if ( v instanceof String ) {
							if ( sb == null )
								sb = new StringBuffer();
							if ( sb.length() > 0 )
								sb.append( ";" );
							sb.append( property ).append( "=" ).append( v );
						}
					}
					if ( sb != null )
						a.putValue( "param4", sb.toString() );
				}
				EditixFrame.THIS.getBuilder().insertMenuItemAtFirst( InterfaceBuilder.MENU_RECENT_FILE, a, 20 );
				ApplicationModel.fireApplicationValue( "document.saveAs", fileName );
			}
		}
		return ok;
	}

}
