package com.japisoft.editix.action.file;

import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;

import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import com.japisoft.editix.ui.EditixDialog;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.ui.table.ExportableTable;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SaveAllDialog extends EditixDialog {

	public SaveAllDialog() {
		super( "Save all", "Save", "Select documents to save" );
		init();
	}

	private ArrayList list;
	public ArrayList getSelections() { return list; }

	private void init() {
		JTable t = new ExportableTable() {
			public boolean editCellAt(int row, int column)  {
				return column == 0;
			}
		};

		DefaultTableModel model = new DefaultTableModel( new
			String[] { "Saved?", "Document name", "Document path" }, 0 ) {
				public boolean isCellEditable(int row, int col)
					{ return col == 0; }
			};

		list = new ArrayList();

		for ( int i = 0; i < EditixFrame.THIS.getXMLContainerCount(); i++ ) {
			XMLContainer container = EditixFrame.THIS.getXMLContainer( i );
			if ( container == null )
				continue;
			model.addRow( new Object[] {
				new Boolean( container.getEditor().isDocumentModified() ),
				container.getDocumentInfo().getDocumentName(),
				container.getDocumentInfo().getCurrentDocumentLocation()
			} );
			if ( container.getEditor().isDocumentModified() )
				list.add( new Boolean( true ) );
			else
				list.add( new Boolean( false ) );
		}
		t.setModel( model );
		t.setSelectionBackground( t.getBackground() );
		t.setSelectionForeground( t.getForeground() );
		t.getColumnModel().getColumn( 0 ).setMaxWidth( 40 );

		CustomCellEditor editor = new CustomCellEditor();
		CustomCellRenderer renderer = new CustomCellRenderer();

		t.getColumnModel().getColumn( 0 ).setCellEditor( editor );

		t.getColumnModel().getColumn( 0 ).setCellRenderer( renderer );
		t.getColumnModel().getColumn( 1 ).setCellRenderer( renderer );
		t.getColumnModel().getColumn( 2 ).setCellRenderer( renderer );

		t.getSelectionModel().setSelectionInterval( 0, 0 );

		getContentPane().add( new JScrollPane( t ) );
		setSize( 400, 250 );
	}

	class CustomCellRenderer extends DefaultTableCellRenderer {
		private JCheckBox cb = new JCheckBox();

		public CustomCellRenderer() {
			super();
			cb.setBackground( Color.white );
			cb.setHorizontalAlignment( JCheckBox.CENTER );
		}

		public Component getTableCellRendererComponent(JTable table,
			Object value,
			boolean b,
			boolean isSelected,
			int row,
			int column ) {
		
			if ( column != 0 ) {
				isSelected = false;

				JLabel lbl = ( JLabel )super.getTableCellRendererComponent(
					table, value, b, isSelected, row, column );
				if ( value != null )
					lbl.setToolTipText( value.toString() );
				return lbl;
			} else {
				cb.setSelected( ( (Boolean)list.get( row ) ).booleanValue() );
				return cb;
			}
		}
	}

	class CustomCellEditor extends DefaultCellEditor  {
		private JCheckBox cb;

		public CustomCellEditor() {
			super( new JCheckBox() );
			cb = ( JCheckBox )getComponent();
			cb.setBackground( Color.white );
			cb.setHorizontalAlignment( JCheckBox.CENTER );
		}

		private int lastRow = 0;

		public Component getTableCellEditorComponent(
			JTable table,
			Object value,
			boolean isSelected,
			int row,
			int column) {
			if ( column == 0 ) {
				lastRow = row;
				cb.setSelected( 
					( ( Boolean )list.get( row ) ).booleanValue() );
				return cb;
			} else 

			isSelected = false;

			return super.getTableCellEditorComponent(
				table,
				value,
				isSelected,
				row,
				column);			
		}

		public boolean stopCellEditing() {
			list.set( lastRow, new Boolean( cb.isSelected() ) );
			return super.stopCellEditing();
		}
		
	}

}
