package com.japisoft.editix.action.file;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;

import com.japisoft.editix.project.ProjectManager;
import com.japisoft.editix.toolkit.Toolkit;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.action.ActionModel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SaveAsAction extends AbstractAction {
	
	public void actionPerformed( ActionEvent e ) {
		if ( EditixFrame.THIS.isEmpty() || 
				EditixFrame.THIS.getSelectedContainer() == null )
			return ;

		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		String currentEncoding = ( String )container.getProperty( "encoding" );
		if ( currentEncoding == null )
			currentEncoding = Toolkit.getCurrentFileEncoding(); 
		
		// Reset the file encoding
		ActionModel.setProperty( ActionModel.SAVE_ACTION, 
			com.japisoft.xmlpad.action.file.SaveAction.ENCODING_PROPERTY,
			currentEncoding );

		// Reset the file encoding
		ActionModel.setProperty( ActionModel.SAVEAS_ACTION, 
			com.japisoft.xmlpad.action.file.SaveAction.ENCODING_PROPERTY,
			currentEncoding );

		// Restore the last file location
		String previousPath = Preferences.getPreference( 
			"file",
			"defaultPath",
			""
		);
		
		// Use the last project path
		if ( ApplicationModel.getSharedProperty( "lastProject" ) != null ) {
			ActionModel.setProperty( 
					ActionModel.SAVEAS_ACTION,
					com.japisoft.xmlpad.action.file.SaveAsAction.CURRENT_DIR_PROPERTY,
					ApplicationModel.getSharedProperty( "lastProject" ).toString() );			
		} else		
		if ( !"".equals( previousPath ) ) {
			ActionModel.setProperty( 
				ActionModel.SAVEAS_ACTION,
				com.japisoft.xmlpad.action.file.SaveAsAction.CURRENT_DIR_PROPERTY,
				previousPath );
		}

		String oldLocation = container.getCurrentDocumentLocation();

		if ( SaveAction.save_action( true ) ) {
			// Update the current tab name			
			EditixFrame.THIS.refreshCurrentTabName();
			if ( oldLocation != null )
				ProjectManager.updateFilePath( oldLocation, container.getCurrentDocumentLocation() );
			if ( Preferences.getPreference(
					"file", "restoredLastPath", true ) ) {
				Preferences.setPreference( "file", "defaultPath", ( String )ActionModel.getProperty(
						ActionModel.SAVEAS_ACTION,
						com.japisoft.xmlpad.action.file.SaveAsAction.CURRENT_DIR_PROPERTY,
						null
				) );
			}
		}
	}

}
