package com.japisoft.editix.action.file.export;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Properties;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class JavaMetaClass {
	private String name;
	
	public JavaMetaClass( String name ) {
		this.name = Tools.toClassName( name );
	}

	private Properties p;

	public void setAttribute( String name, String value ) {
		if ( p == null )
			p = new Properties();
		p.setProperty( Tools.convertIfKeyWord( name ), value );
	}

	private ArrayList al = null;

	public void addContentElement( String name ) {
		if ( al == null )
			al = new ArrayList();
		if ( !al.contains( Tools.toClassName( name ) ) )
			al.add( Tools.toClassName( name ) );
	}
		
	private boolean textContent = false;

	public void setText() {
		textContent = true;
	}

	public void write( PrintWriter pw, String pack ) {
		pw.println( "package " + pack + ";" );
		pw.println();
		pw.println( "import java.util.ArrayList;" );
		pw.println();
		pw.println( "/**" );
		pw.println(	"*\tGenerated by EditiX" );
		pw.println( "*\thttp://www.editix.com" );
		pw.println( "*/");
		pw.println( "public class " + name + " extends AbstractElement {" );

		// Attributs
		if ( p != null ) {
			pw.println();
			Enumeration e = p.propertyNames();
			while ( e.hasMoreElements() ) {
				String att = ( String )e.nextElement();
				pw.println( "\tprivate String " + att + ";" );
			}
			pw.println();
			e = p.propertyNames();
			while ( e.hasMoreElements() ) {
				String att = ( String )e.nextElement();
				// Setter
				pw.println( "\tpublic void " + Tools.toSetName( att ) + "( String _value ) {" );
				pw.println( "\t\tthis." + att + " = _value;" );
				pw.println( "\t}" );
				pw.println();
				// Getter
				pw.println( "\tpublic String " + Tools.toGetName( att ) + "() {" );
				pw.println( "\t\treturn " + att + ";" );
				pw.println( "\t}" );
				pw.println();
			}			
		}

		// Content
		if ( al != null ) {
			pw.println();
			for ( int i = 0; i < al.size(); i++ ) {
				String name = ( String )al.get( i );
				pw.println( "\tprivate ArrayList" + " " + name.toLowerCase() + ";" );
			}
			pw.println();
			for ( int i = 0; i < al.size(); i++ ) {
				String name = ( String )al.get( i );
				// Adder
				pw.println( "\tpublic void " + Tools.toAddName( name.toLowerCase() ) + "( " + name + " _value ) {" );
				pw.println( "\t\tif ( " + name.toLowerCase() + "==null ) " + name.toLowerCase() + " = new ArrayList();" );
				pw.println( "\t\tthis." + name.toLowerCase() + ".add( _value );" );
				pw.println( "\t}" );
				pw.println();
				// Getter
				pw.println( "\tpublic " + "ArrayList " + Tools.toGetName( name ) + "() {" );
				pw.println( "\t\treturn " + name.toLowerCase() + ";" );
				pw.println( "\t}" );
				pw.println();
				// Has
				pw.println( "\tpublic " + "boolean " + Tools.toAnyName( "has", name ) + "() {" );
				pw.println( "\t\treturn " + name.toLowerCase() + " != null;" );
				pw.println( "\t}" );
				pw.println();
				// First
				pw.println( "\tpublic " + name + " " + Tools.toAnyName( "getFirst", name ) + "() {" );
				pw.println( "\t\tif ( " + name.toLowerCase() + "!= null && " + name.toLowerCase() + ".size() > 0 ) {" );
				pw.println( "\t\t\treturn (" + name + ")" + name.toLowerCase() + ".get(0);" );
				pw.println( "\t\t} else return null;" );
				pw.println( "\t}" );
				pw.println();
			}
		}
		
		if ( textContent ) {
			pw.println();
			pw.println( "\tprivate String value;" );
			pw.println( "\tpublic void setValue( String value ) {" );
			pw.println( "\t\tthis.value = value;" );
			pw.println( "\t}" );
			pw.println();
			pw.println( "\tpublic String getValue() {" );
			pw.println( "\t\treturn value;" );
			pw.println( "\t}" );
		}
		
		pw.println( "}" );
	}
}
