package com.japisoft.editix.action.file.imp;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import javax.swing.AbstractAction;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.toolkit.FileToolkit;
import com.japisoft.framework.ui.toolkit.BrowserCaller;
import com.japisoft.framework.ui.toolkit.FileManager;
import com.japisoft.framework.xml.XMLToolkit;
import com.japisoft.p3.Manager;
import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CSVImport extends AbstractAction {

	public void actionPerformed(ActionEvent e) {
		
		if ( Manager.isFree() ) {

			EditixFactory.buildAndShowInformationDialog( "This action is not available inside the Free Edition.\nPlease look at http://www.editix.com" );
			BrowserCaller.displayURL( "http://www.editix.com" );
			
		} else {
			//
			
			File csvFile = FileManager.getSelectedFile( true, new String[] { "csv", "xlsx" }, new String[] { "CSV File", "Excel file" } );
			
			if ( csvFile != null ) {
				
				CSVPanel confImport = new CSVPanel();				
				confImport.setExcelMode( FileToolkit.matchExt( csvFile, "xlsx" ) );
				
				if (DialogManager.showDialog(EditixFrame.THIS, "CSV Import",
						"CSV Import", "Import a Spreadsheet (csv,xlsx...) document to XML", null,
						confImport, new Dimension(400, 550)) == DialogManager.OK_ID) {
					
					String content = impCSV(csvFile, confImport);
					
					IXMLPanel panel = EditixFactory.buildNewContainer("XML", (String)null);
					XMLContainer container = panel.getMainContainer();
					container.setText( content );
					EditixFrame.THIS.addContainer(panel);

				}
			} 
			//
		}
	}

	public static String impCSV(File dataFile, CSVPanel config) {
		StringBuffer sb = new StringBuffer();
		sb.append("<?xml version=\"1.0\"?>\n\n<document>\n");

		char sep = config.getSeparator();
		
		char chrDelim = config.getStringDelimiter();

		String row = config.getRowName();
		String col = "col";

		try {

			BufferedReader br = new BufferedReader( 
				new FileReader( dataFile ) 
			);

			try {

				String line = null;
				int startingRow = config.getStartingRow();
				int rowNumber = 1;

				while ( ( line = br.readLine() ) != null ) {
					startingRow--;
					if (startingRow <= 0) {
						// Process it
						sb.append( "\t<" + row );
						if ( config.hasRowNumber() ) {
							sb.append( " id=\"" + rowNumber + "\"" );
						}
						sb.append( ">\n");

						int i = 0;
						int c = 0;
						StringBuffer sbCol = null;
						boolean inQualifiedString = false;
						while ( i < line.length() ) {
							char ch = line.charAt( i );
							if (sbCol == null) {
								sbCol = new StringBuffer();
							}
							if (ch == sep && !inQualifiedString) {

								String columnName = config.getColName( c, "" + ( c + 1 ) );
								
								String idAttribute = "";
								
								String tmpCol = col;
								if ( config.convertColToElement() ) {
									tmpCol = columnName;
								} else
									idAttribute = " id=\"" + columnName	+ "\"";

								sb.append("\t\t<" + tmpCol + idAttribute + ">");
								sb.append( XMLToolkit.resolveCharEntities( sbCol.toString() ) );
								sb.append("</" + tmpCol + ">\n");							

								sbCol = null;
								c++;
							} else {

								if (chrDelim != 0) {

									if (ch == chrDelim) {
										inQualifiedString = !inQualifiedString;
									} else
										sbCol.append(ch);

								} else
									sbCol.append(ch);
							}
							i++;
						}

						if ( sbCol != null ) {
							String columnName = config.getColName( c, "" + ( c + 1 ) );
							
							String idAttribute = "";
							
							String tmpCol = col;
							if ( config.convertColToElement() ) {
								tmpCol = columnName;
							} else
								idAttribute = " id=\"" + columnName	+ "\"";

							sb.append("\t\t<" + tmpCol + idAttribute + ">");
							sb.append( XMLToolkit.resolveCharEntities( sbCol.toString() ) );
							sb.append("</" + tmpCol + ">\n");							
						}

						sb.append("\t</" + row + ">\n");
						
					}
					rowNumber++;
				}

			
			} finally {
				br.close();
			}

		} catch (FileNotFoundException exc) {
			EditixFactory.buildAndShowErrorDialog("File error : " + dataFile);
		} catch (IOException exc) {
			EditixFactory.buildAndShowErrorDialog("Can't parse : " + dataFile);
		}

		sb.append("\n</document>");		
		return sb.toString();
	}

}
