package com.japisoft.editix.action.file.imp;

import java.awt.event.ActionEvent;
import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import org.w3c.tidy.Tidy;
import com.japisoft.editix.toolkit.Toolkit;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.ui.toolkit.BrowserCaller;
import com.japisoft.p3.Manager;
import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class HTMLImport extends AbstractAction {

	public void actionPerformed(ActionEvent e) {
		
		if ( Manager.isFree() ) {

			EditixFactory.buildAndShowInformationDialog( "This action is not available inside the Free Edition.\nPlease look at http://www.editix.com" );
			BrowserCaller.displayURL( "http://www.editix.com" );
			
		} else {
			//
			JFileChooser chooser = EditixFactory
					.buildFileChooserForDocumentType("XHTML");
			if (chooser.showOpenDialog(EditixFrame.THIS) == JFileChooser.APPROVE_OPTION) {
				try {
					byte[] data = convertHTMLInputStream( 
							new FileInputStream( chooser.getSelectedFile() ) );
					if ( data != null ) {
						IXMLPanel panel = EditixFactory.buildNewContainer("XHTML", (String)null);
						XMLContainer container = panel.getMainContainer();
						container.setText(Toolkit.getEncodedString(data).getContent());
						EditixFrame.THIS.addContainer(panel);
					}
				} catch (Throwable exc) {
					ApplicationModel.debug( exc );
					EditixFactory.buildAndShowErrorDialog( "Can't import : " + exc.getMessage() );				
				}
			}
			//
		}
	}

	public static byte[] convertHTMLInputStream( InputStream input ) {
		Tidy tidy = new Tidy();
		tidy.setXmlOut(true);
		tidy.setNumEntities(true);
		tidy.setQuoteNbsp(true);
		tidy.setQuiet(true);
		tidy.setErrout( new PrintWriter( new StringWriter() ) );		
		try {
			InputStream in = new BufferedInputStream( input );
			ByteArrayOutputStream output = new ByteArrayOutputStream();
			try {
				tidy.parse( in, output );
			} finally {
				in.close();
				output.close();
			}
			byte[] data = output.toByteArray();
			return data;
		} catch (Exception exc) {
			ApplicationModel.debug( exc );
			EditixFactory.buildAndShowErrorDialog( "Can't import : " + exc.getMessage() );
			return null;
		}
	}

}
