package com.japisoft.editix.action.file.imp;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.DriverPropertyInfo;
import java.sql.SQLException;
import java.sql.SQLFeatureNotSupportedException;
import java.util.Properties;
import java.util.logging.Logger;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class JDBCDriver {

	private String name;
	private String driverClass;
	private String defaultUrl;
	private String[] urls;
	
	public static JDBCDriver ODBC_DRIVER = null;
	
	static {
		ODBC_DRIVER = new JDBCDriver( "ODBC" );
		// <db name="ODBC" driver="sun.jdbc.odbc.JdbcOdbcDriver" url="jdbc:odbc:<data-source-name>"/>
		ODBC_DRIVER.setDriverClass( "sun.jdbc.odbc.JdbcOdbcDriver" );
		ODBC_DRIVER.setDefaultUrl( "jdbc:odbc:<data-source-name>" );
	}

	public JDBCDriver( String name ) {
		if ( name == null ) {
			this.name = "new driver";
		} else
			this.name = name;
	}
	
	public JDBCDriver( Element e ) {
		fromXML( e );
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	public String getDriverClass() {
		return driverClass;
	}
	
	public void setDriverClass(String driverClass) {
		this.driverClass = driverClass;
	}
	
	public String getDefaultUrl() {
		return defaultUrl;
	}
	
	public void setDefaultUrl(String defaultUrl) {
		this.defaultUrl = defaultUrl;
	}
	
	public String[] getUrls() {
		return urls;
	}
	
	public void setUrls(String[] urls) {
		this.urls = urls;
	}
	
	public String getFlatUrls() {
		if ( urls != null ) {
			String tmp = "";
			for ( String u : urls ) {
				if ( !"".equals( tmp ) )
					tmp += ";";
				tmp += u;
			
			}
			return tmp;
		} else
			return null;
	}

	public void setFlatUrls( String urls ) {
		setUrls( 
			urls.split( ";" ) 
		);		
	}

	public void fromXML( Element e ) {
		setName( e.getAttribute( "name" ) );
		setDriverClass( e.getAttribute( "driverClass" ) );
		setDefaultUrl( e.getAttribute( "defaultUrl" ) );
		String urls = e.getAttribute( "libraries" );
		setFlatUrls( urls );
	}

	public Element toXML( Document doc ) {
		Element e = doc.createElement( "driver" );
		e.setAttribute( "name", name );
		e.setAttribute( "driverClass", driverClass );
		e.setAttribute( "defaultUrl", defaultUrl );
		if ( urls != null ) {
			e.setAttribute( "libraries", getFlatUrls() );
		}
		return e;
	}

	@Override
	public String toString() {
		return this.name;
	}

	public Connection getConnection( String url, String user, String password ) throws Exception {
		String drivers[] = getUrls();
		if ( drivers != null && drivers.length >= 0 ) {
			URL[] urls = new URL[ drivers.length ];
			for ( int i = 0; i < drivers.length; i++ )
				urls[ i ] = new File( drivers[ i ] ).toURL();

			URLClassLoader loader = new URLClassLoader( urls );

			Driver d = ( Driver )Class.forName( getDriverClass(), true, loader ).newInstance();
			DriverManager.registerDriver( new DriverShim( d ) );			
		} else
			Class.forName( getDriverClass() );

		return DriverManager.getConnection( url, user, password );
	}

	static class DriverShim implements Driver {
		private Driver driver;
		DriverShim(Driver d) {
			this.driver = d;
		}
		public boolean acceptsURL(String u) throws SQLException {
			return this.driver.acceptsURL(u);
		}
		public Connection connect(String u, Properties p) throws SQLException {
			return this.driver.connect(u, p);
		}
		public int getMajorVersion() {
			return this.driver.getMajorVersion();
		}
		public int getMinorVersion() {
			return this.driver.getMinorVersion();
		}
		public DriverPropertyInfo[] getPropertyInfo(String u, Properties p) throws SQLException {
			return this.driver.getPropertyInfo(u, p);
		}
		public boolean jdbcCompliant() {
			return this.driver.jdbcCompliant();
		}
		public Logger getParentLogger() throws SQLFeatureNotSupportedException {
			return null;
		}
	}	
	
	
}
