package com.japisoft.editix.action.file.project;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JMenu;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.framework.application.descriptor.InterfaceBuilder;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class RecentProjectManager {

	private static RecentProjectManager INSTANCE = null;
	
	private RecentProjectManager() {
		super();
		load();
	}
	
	public static RecentProjectManager getInstance() {
		if ( INSTANCE == null )
			INSTANCE = new RecentProjectManager();
		return INSTANCE;
	}
	
	private List<String> recentProjects = null;
	
	private void load() {
		recentProjects = new ArrayList<String>();
		File f = new File( 
			ApplicationModel.getAppUserPath(), 
			"recentProjects.dat" 
		);
		if ( f.exists() ) {
			try {
				BufferedReader br = new BufferedReader(
						new FileReader( f ) );
				try {
					String l = null;
					while ( ( l = br.readLine() ) != null ) {
						if ( new File( l ).exists() )
							recentProjects.add( l );
					}						
				} finally {
					br.close();
				}
			} catch( Exception exc ) {
				ApplicationModel.debug( exc );
			}
		}
	}

	public void addProject( File projectLocation ) {
		String str = projectLocation.toString();
		recentProjects.remove( str );
		recentProjects.add( 0, str );
		save();
		

		// Rebuild the recent menu builder
		RecentFileMenuBuilder builder = new RecentFileMenuBuilder();
		InterfaceBuilder ib = ApplicationModel.INTERFACE_BUILDER;
		builder.build( ib.getMenu( "openrp" ) );
	}

	public int getItemCount() {
		return recentProjects.size();
	}
	
	public String getItem( int index ) {
		return recentProjects.get( index );
	}

	private void save() {
		File f = new File( 
			ApplicationModel.getAppUserPath(),
			"recentProjects.dat"
		);
		try {
			BufferedWriter bf = 
				new BufferedWriter( 
					new FileWriter( f ) );
			try {
				boolean first = true;
				for ( String str : recentProjects ) {
					if ( !first ) {
						bf.newLine();
					}
					bf.write( str );
					first = false;
				}
			} finally {
				bf.close();
			}
		} catch( Exception exc ) {
			ApplicationModel.debug( exc );
		}
	}

}
