package com.japisoft.editix.action.help;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.MalformedURLException;

import javax.swing.AbstractAction;

import DTDDoc.DTDCommenter;
import DTDDoc.ExtendedDTD;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.dialog.basic.HTMLDialog;
import com.japisoft.framework.toolkit.FileToolkit;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DTDManualGenerator extends AbstractAction {

	public void actionPerformed(ActionEvent e) {

		File userHome = ApplicationModel.getAppUserPath();
		File target = new File( userHome, "html-dtd" );
		target.mkdir();
		String dtdLocation = (String)getValue( "param" );
		String dtdName = (String)getValue( "param2" );
		if (dtdLocation == null || dtdName == null) {
			EditixFactory
					.buildAndShowErrorDialog("Cannot find the DTD location !!");
			return;
		}

		File dtdLoc = new File(target, dtdName + ".dtd");
		File htmlLoc = new File(target, dtdName + ".html");

		if ( !htmlLoc.exists() ) {
			// Generate the documentation once
			if (!dtdLoc.exists()) {
				try {
					FileToolkit.copyFileFromURL(ClassLoader
							.getSystemResource(dtdLocation), dtdLoc);

					ExtendedDTD dtd = new ExtendedDTD(dtdLoc);

					DTDCommenter c = new DTDCommenter();
					c.createDocumentation( dtd, new PrintWriter(new FileWriter(
							htmlLoc)), false);

				} catch ( IOException e1 ) {
					EditixFactory
							.buildAndShowErrorDialog("Can't display the documentation : "
									+ e1.getMessage());
				}
			}
		}

		if ( htmlLoc.exists() ) {
			try {
				HTMLDialog d = new HTMLDialog( ApplicationModel.MAIN_FRAME,
						"Schema Documentation", htmlLoc.toURI().toURL()
								.toString(), false );
				d.setVisible( true );
			} catch ( MalformedURLException e1 ) {
			}

		} else {
			EditixFactory
					.buildAndShowErrorDialog( "Can't display the documentation for "
							+ dtdName );
		}

	}

}
