package com.japisoft.editix.action.help;

import java.awt.event.ActionEvent;
import java.io.ByteArrayOutputStream;

import javax.swing.AbstractAction;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import com.japisoft.editix.action.xsl.XSLTAction;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.dialog.BasicOKDialogComponent;
import com.japisoft.framework.job.BasicJob;
import com.japisoft.framework.job.Job;
import com.japisoft.framework.job.JobManager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class UIHelpAction extends AbstractAction {

	DocumentDialog dialog;

	public void actionPerformed(ActionEvent e) {
			dialog = new DocumentDialog( "Please wait while generating documentation..." );
			JobManager.addJob( new DocGeneratorJob() );
			dialog.setVisible( true );
			dialog.dispose();
			dialog = null;
	}

	class DocGeneratorJob extends BasicJob {

		public Object getSource() {
			return UIHelpAction.this;
		}

		public boolean isAlone() {
			return true;
		}
		
		public void dispose() {}
		
		public void run() {
			TransformerFactory tf = 
				XSLTAction.getTransformerFactoryV1( false );
			try {
				Transformer transformer = tf.newTransformer(
					new StreamSource( ClassLoader.getSystemResourceAsStream( "doc.xsl" ) ) );
			
				ByteArrayOutputStream output = new ByteArrayOutputStream();

				transformer.transform( 
					new StreamSource( ClassLoader.getSystemResourceAsStream( "editix.xml" ) ),
					new StreamResult( output ) );
					
				dialog.setContent( output.toString() );
			} catch( Throwable th ) {
				dialog.setContent( "Error : " + th.getMessage() );
			}
		}

		public void stopIt() {
		}
	}

	class DocumentDialog extends BasicOKDialogComponent  {
		JEditorPane ep = null;
		
		public DocumentDialog( String htmlContent ) {
			super( ApplicationModel.MAIN_FRAME, 
					"Documentation", 
					"Documentation", 
					"EditiX menu content",
					null );
			setUI( new JScrollPane( ep = new JEditorPane( "text/html", htmlContent ) ) );
			ep.setEditable( false );
			ep.setCaretPosition( 0 );
			//pack();
			setSize( 600, 600 );
		}
		
		public void setContent( String html ) {
			ep.setText( html );
			ep.setCaretPosition( 0 );
		}
	}

}
