package com.japisoft.editix.action.panels.ui;

import java.awt.event.ActionEvent;
import java.awt.print.PrinterException;
import java.io.File;
import java.io.FileOutputStream;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JTable;
import javax.swing.JToolBar;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.Resource;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.spreadsheet.Spreadsheet;
import com.japisoft.framework.spreadsheet.SpreadsheetFactory;
import com.japisoft.framework.ui.toolkit.FileManager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TableToolBar extends JToolBar {

	private JTable table;
	
	public TableToolBar( JTable table ) {
		this.table = table;
		add( new PrintAction() );
		add( new ExportAction() );
	}
	
	public class PrintAction extends AbstractAction {
	
		public PrintAction() {
			putValue( Action.SMALL_ICON, Resource.getImage( "images/printer.png") );
		}
		
		public void actionPerformed(ActionEvent arg0) {
			
			try {
				table.print();
			} catch( PrinterException exc ) {
				EditixFactory.buildAndShowErrorDialog( "Can't print : " + exc.getMessage() );
			}

		}
		
	}
	
	public class ExportAction extends AbstractAction {
		
		public ExportAction() {
			putValue( Action.SMALL_ICON, Resource.getImage( "images/tables.png") );
		}
		
		public void actionPerformed( ActionEvent evt ) {
			File selectedFile = FileManager.getSelectedFile(
					false, 
					new String[] { 
						"xlsx","xls","csv" 
					}, 
					new String[] { 
						"Microsoft Excel 2007 and later (*.xlsx)", "Microsoft Excel (*.xls)", "Comma-separated values File (*.csv)" 
					}
				);			
				
				if ( selectedFile != null ) {
					
					try {
						Spreadsheet spreadsheet = SpreadsheetFactory.getInstance().getSpreadsheet( selectedFile );
						spreadsheet.reset( table.getModel() );
						spreadsheet.write( new FileOutputStream( selectedFile ));
					} catch( Exception exc ) {
						ApplicationModel.fireApplicationValue( "error", exc.getMessage() );
					}
					
				}
			
		}
		
	}
	
}
