package com.japisoft.editix.action.search;

import java.awt.event.ActionEvent;
import java.util.StringTokenizer;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.actions.SynchronizableAction;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BookmarkAction extends AbstractAction implements SynchronizableAction {

	public void actionPerformed(ActionEvent e) {
		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null )
			return;

		FPNode node = container.getCurrentNode();
		String stored = null;
		if ( node == null ) {
			// Cursor location
			stored = "Cursor at " + container.getEditor().getCaretPosition();
		} else
			stored = node.getXPathLocation();

		addBookmarkToMenu( stored );
		storeBookmarksInContainer( container );
	}

	public void synchronizeState(Object source) {
		if ( source instanceof XMLContainer ) {
			XMLContainer container = ( XMLContainer )source;
			String bookmarks = ( String )container.getProperty( "bookmarks", "" );
			EditixFrame.THIS.getBuilder().cleanMenuItems("listOfBookmarks" );
			StringTokenizer st = new StringTokenizer( bookmarks, "!!" );
			while ( st.hasMoreTokens() ) {
				addBookmarkToMenu( st.nextToken() );
			}
		}
	}

	public static void storeBookmarksInContainer( XMLContainer container ) {
		JMenu menu = EditixFrame.THIS.getBuilder().getMenu( "listOfBookmarks" );
		if ( menu == null ) {	//??
			System.err.println( "Can't find sub menu listOfBookmarks ??" );
		} else {
			StringBuffer mustStored = null;
			for ( int i = 0; i< menu.getItemCount(); i++ ) {
				JMenuItem item = menu.getItem( i );
				String lbl = item.getText();
				if ( mustStored == null )
					mustStored = new StringBuffer();
				else
					mustStored.append( "!!" );
				mustStored.append( lbl );
			}
			if ( mustStored != null )
				container.setProperty( "bookmarks", mustStored.toString() );
		}
	}

	public static void addBookmarkToMenu( String bookmark ) {
		BookmarkGoToAction bgt = new BookmarkGoToAction();
		bgt.putValue( Action.NAME, bookmark );
		bgt.putValue( "param", bookmark );
		bgt.putValue( "iconPath", "images/bookmark.png" );
		EditixFrame.THIS.getBuilder().insertMenuItemAtFirst( "listOfBookmarks", bgt, 20 );		
	}

}
