package com.japisoft.editix.action.search;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JPanel;

import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.editix.ui.EditixContainerListener;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.findreplace.FindReplacePanel;
import com.japisoft.findreplace.Findable;
import com.japisoft.framework.dialog.BasicDialogComponent;
import com.japisoft.framework.dialog.actions.DialogActionModel;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.XMLEditor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FindAction extends AbstractAction implements EditixContainerListener {

	private BasicDialogComponent dialog = null;
	Findable panel = null;

	public FindAction() {
		super();
		EditixFrame.addEditixContainerListener( this );
	}
	
	public void actionPerformed(ActionEvent e) {
		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null )
			return;

		XMLEditor editor = container.getEditor();
		
		if ( Preferences.getPreference( "interface", "oldFindReplace", false ) ) { 
				
			if ( dialog == null ) {
				panel = new FindReplacePanel( container.getEditor(), true );
				
				String param = ( String )getValue( "param" );
	
				if ( param != null ) {
					panel.setFindValue( param );
				}
	
				DialogActionModel model = DialogActionModel.getDefaultDialogActionModel();
				
				dialog = new BasicDialogComponent( EditixFrame.THIS, "Find/Replace" );
				dialog.setModal( false );
	
				dialog.getContentPane().add( (JPanel)panel );
				dialog.setSize( 300, 350 );
				dialog.setVisible( true );
			} else {			
				
				panel.updateTextComponent( container.getEditor(), true );
	
				String param = ( String )getValue( "param" );
	
				if ( param != null ) {
					panel.setFindValue( param );
				}
				
				dialog.setVisible( true );
			}
		
		} else {
		
			panel = editor.FindAndReplaceBox();
			panel.updateTextComponent( editor, true );
			
		}
		
		SearchAgainAction action = ( SearchAgainAction )ActionModel.restoreAction( "searchAgain" );
		if ( action != null )
			action.setEnabled( true );
		else
			System.err.println( "Cannot find 'searchAgain' action ??????????????" );
		
	}

	// Freeing inner reference
	public void close( XMLContainer container ) {
		if ( container == null )	// Unknown case
			return;
		if ( panel != null && panel.getCurrentTextComponent() == container.getEditor() ) {
			panel.updateTextComponent( null, false );
			// Disabled searchAgain
			Action searchAgain = ActionModel.restoreAction( "searchAgain" );
			if ( searchAgain != null )
				searchAgain.setEnabled( false );
		}
	}

}
