package com.japisoft.editix.action.template;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.japisoft.editix.action.file.SelectTemplatePanel;
import com.japisoft.editix.document.DocumentModel;
import com.japisoft.editix.document.TemplateInfo;
import com.japisoft.editix.document.TemplateModel;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.dialog.actions.ClosableAction;
import com.japisoft.framework.dialog.actions.DialogAction;
import com.japisoft.framework.dialog.actions.DialogActionModel;
import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.XMLDocumentInfo;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class EditTemplates extends AbstractAction {

	public void actionPerformed( ActionEvent e ) {
		SelectTemplatePanel stp = new SelectTemplatePanel( false );
		DialogActionModel dam = new DialogActionModel(
			new DialogAction[] {
				DialogActionModel.DEFAULT_CANCELACTION,
				new ResetDefaultAction(),
				new EditAction()
			}
		);
		
		int resultDialog = DialogManager.showDialog( 
				EditixFrame.THIS, 
				"Edit templates", 
				"Edit current templates", 
				"User templates are located at " + TemplateModel.getUserTemplates() + "\nIf you reset a custom user template, it will delete it", 
				null, 
				stp, 
				dam, 
				null );

		TemplateInfo ti = null;
		
		if ( resultDialog != DialogManager.CANCEL_ID ) {
			
			ti = stp.getTemplateInfo();
			if ( ti == null ) {
				EditixFactory.buildAndShowWarningDialog( "No selected template ?" );
				return;
			}
			
		}
		
		if ( resultDialog == 200 ) {

			if ( EditixFactory.buildAndShowConfirmDialog( "Reset to the default content ?" ) ) {
				File f = TemplateModel.getTemplatePath( ti.location );
				f.delete();
			}	

		} else
		
		if ( resultDialog == 100 ) {
			String location = ti.location;

			XMLDocumentInfo doc = 
				DocumentModel.getDocumentForType( ti.type );
			IXMLPanel panel = 
				EditixFactory.getPanelForDocument( doc );
			XMLContainer container = 
				panel.getMainContainer();
			XMLDocumentInfo newDoc = 
				doc.cloneDocument();
			try {
				TemplateModel.resolveTemplate( null, location, newDoc );
				container.setDocumentInfo( newDoc );
				container.setText( newDoc.getTemplate() ); 
				container.setCurrentDocumentLocation( TemplateModel.getTemplatePath( location ).toString() );
				
				EditixFrame.THIS.addContainer( panel );
			} catch( Throwable th ) {
				EditixFactory.buildAndShowErrorDialog( "Can't load this template " + th.getMessage() );					
			}		
		}
		
		if ( resultDialog != DialogManager.CANCEL_ID ) {
			TemplateModel.loadModel();
		}
		
	}

	class EditAction extends AbstractAction implements DialogAction, ClosableAction {
		public EditAction() {
			putValue( Action.NAME, "Edit" );
		}
		public void actionPerformed(ActionEvent e) {
		}
		public int getActionId() {
			return 100;
		}
		public boolean isForDialogFooter() {
			return true;
		}
		public boolean isSpecial() {
			return false;
		}
	}
	
	class ResetDefaultAction extends AbstractAction implements DialogAction, ClosableAction {
		public ResetDefaultAction() {
			putValue( Action.NAME, "Reset to default" );
		}
		public void actionPerformed(ActionEvent e) {
		}
		public int getActionId() {
			return 200;
		}
		public boolean isForDialogFooter() {
			return true;
		}
		public boolean isSpecial() {
			return false;
		}	
	}
	
}
