package com.japisoft.editix.action.view;

import java.awt.event.ActionEvent;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import org.netbeans.swing.tabcontrol.TabbedContainer;

import com.japisoft.editix.ui.EditixDialog;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.dialog.actions.AbstractDialogAction;
import com.japisoft.framework.ui.table.ExportableTable;
import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SelectDialog extends EditixDialog {
	private JButton btnSelect;

	public SelectDialog() {
		super( "Select editor", "Switch to editor", "Select an editor", DialogManager.buildNewActionModel() );
		init();
		actionModel.addDialogAction( new SelectionAction() );
	}
	
	private JTable t;

	private void init() {
		t = new ExportableTable() {
			public boolean editCellAt(int row, int column)  {
				return false;
			}
		};

		DefaultTableModel model = new DefaultTableModel( new
			String[] { "Document name", "Encoding", "Document path" }, 0 ) {
				public boolean isCellEditable(int row, int col)
					{ return false; }
			};

		TabbedContainer pane = EditixFrame.THIS.getMainTabbedPane();
		for ( int i = 0; i < pane.getTabCount(); i++ ) {
			XMLContainer container = ( ( IXMLPanel )pane.getComponentAt( i ) ).getMainContainer();
			if ( container == null )
				continue;
			model.addRow( new Object[] {
				container.getDocumentInfo().getDocumentName(),
				container.getProperty( "encoding", "DEFAULT" ),
				container.getDocumentInfo().getCurrentDocumentLocation()
			} );
		}
		t.setModel( model );

		t.getSelectionModel().setSelectionInterval( 
			EditixFrame.THIS.getMainTabbedPane().getSelectedIndex(),
			EditixFrame.THIS.getMainTabbedPane().getSelectedIndex() );

		//t.setEnabled( false );
		getContentPane().add( new JScrollPane( t ) );
		setSize( 450, 250 );
	}

	////////////////////////////////////////////////////////////
	
	class SelectionAction extends AbstractDialogAction {
		public SelectionAction() {
			super( 101, false );
			putValue( Action.NAME, "Select" );
		}
		public void actionPerformed( ActionEvent e ) {
			int selection = t.getSelectedRow();
			if ( selection != -1 )
				EditixFrame.THIS.getMainTabbedPane().setSelectedIndex( selection );					
		}
	}

}
