package com.japisoft.editix.action.xml;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;

import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.EditixStatusBar;
import com.japisoft.framework.actions.SynchronizableAction;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DisabledEnabledCompletionAction extends AbstractAction implements SynchronizableAction {
	
	public void actionPerformed( ActionEvent e ) {
		if ( EditixFrame.THIS == null )
			return;
		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null )
			return;

		boolean state = container.hasSyntaxCompletion();
		if ( state ) {
			// Disabled it
			container.setSyntaxCompletion( false );
			EditixStatusBar.ACCESSOR.setXPathLocation( "Syntax completion disabled" );
		} else {
			// Enabled it
			container.setSyntaxCompletion( true );
			EditixStatusBar.ACCESSOR.setXPathLocation( "Syntax completion enabled" );
		}
		// Save it
		container.setProperty( getClass().getName(), "" + !state );
		refreshState( container );
	}

	public void synchronizeState(Object source) {
		if ( source instanceof XMLContainer ) {
			XMLContainer container = ( XMLContainer )source;
			
			boolean state = refreshState( container );
			
			// Check for state
			if ( state ) {
				if ( !container.hasSyntaxCompletion() ) {
					container.setSyntaxCompletion( true );
				}
			} else {
				if ( container.hasSyntaxCompletion() ) {
					container.setSyntaxCompletion( false );
				}
			}

		}
	}	

	private Icon okIcon = null;

	private boolean refreshState( XMLContainer container ) {
		boolean state = "true".equals( container.getProperty( getClass().getName(), "true" ) );
		if ( state ) {
			if ( okIcon == null ) {
				okIcon = ( Icon )getValue( Action.SMALL_ICON );
			}
			putValue( Action.SMALL_ICON, okIcon );
		} else {
			putValue( Action.SMALL_ICON, getValue( Action.SMALL_ICON + "2" ) );
		}
		return state;
	}

}
