package com.japisoft.editix.action.xml;

import java.io.File;
import java.net.URL;

import javax.xml.XMLConstants;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import com.japisoft.editix.action.dtdschema.AssignSchematron;
import com.japisoft.editix.action.dtdschema.SchematronAction;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.xml.validator.DefaultValidator;
import com.japisoft.xmlpad.xml.validator.RelaxNGValidator;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class EditixValidator extends DefaultValidator {

	protected boolean canValidate(
			XMLContainer container ) {
		String type = "xml";
		if ( container.getDocumentInfo() != null )
			type = container.getDocumentInfo().getType();
		FPNode root = container.getRootNode();
		if ( root != null ) {
			if ( "XSLT".equals( type ) ) {
				// Check for version = 1
				String version = 
					root.getAttribute( "version" );
				if ( "2.0".equals( version ) ) {
					EditixFactory.buildAndShowWarningDialog( "You are using an xslt 2.0 document, but your document type was for xslt 1.0, please reopen your document as an XSLT 2.0" );
				}
			} else
			if ( "XSLT2".equals( type ) ) {
				// Check for version = 2
				String version = 
					root.getAttribute( "version" );				
				if ( "1.0".equals( version ) ) {
					EditixFactory.buildAndShowWarningDialog( "You are using an xslt 1.0 document, but your document type was for xslt 2.0, please reopen your document as an XSLT 1.0" );
				}
			}
		}
		return super.canValidate( container );
	}	

	public int notifyAction(XMLContainer container, boolean silentMode ) {
		int res = 0;
		res = super.notifyAction( container, silentMode );

		if ( res == OK ) {
		
			// Check for schematron processing instruction
			
			XMLPadDocument doc = container.getXMLDocument();
			String txt = container.getText();
	
			int t = doc.nextTag( 0 );
			if ( t > -1 )
				txt = txt.substring( 0, t );
			
			int i = txt.indexOf( AssignSchematron.PI );
			if ( i > 0 ) {
				int j = txt.indexOf( "path=", i );
				if ( j > 0 ) {
					int k = txt.indexOf( "\"", j );
					if ( k > 0 ) {
						int l = txt.indexOf( "\"", k + 1 );
						if ( l > -1 ) {
							String path = txt.substring( k + 1, l );

							// Check relative schematron path
							if ( container.getCurrentDocumentLocation() != null ) {
								File tmp = new File( 
									new File( container.getCurrentDocumentLocation() ).getParentFile(),
									path
								);
								if ( tmp.exists() )
									path = tmp.toString();
							}
							
							res = SchematronAction.checkSchematron( container, new File( path ), false );
						}
					}
				}
			}
			
		}

		return res;
	}
	
	protected SAXParser buildCustomParser( XMLContainer container ) {
		// Requires jdk 5.x
		try {
			SAXParserFactory parserFactory;
			parserFactory = SAXParserFactory.newInstance();
			SchemaFactory schemaFactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
			URL u = container.getDocumentInfo().getSchemaXSDValid();
			Schema schema = schemaFactory.newSchema(u);
			parserFactory.setNamespaceAware(true);
			parserFactory.setSchema(schema);	
			return parserFactory.newSAXParser();
		} catch (Throwable e) {
			ApplicationModel.debug(	e );
		}	
		return null;
	}

	// Validate with RelaxNG
	protected boolean postAction( XMLContainer container, boolean silentMode ) {
		if ( container.getDocumentInfo().getSchemaRNGValid() != null ) {	
			URL loc = container.getDocumentInfo().getSchemaRNGValid();

			String ef = loc.toExternalForm();
			ef = ef.replace( "%20", " " );

			container.getSchemaAccessibility().setRelaxNGValidationLocation(
					ef );

			RelaxNGValidator val = new RelaxNGValidator();
			container.getErrorManager().initErrorProcessing();
			val.validate( container, silentMode );
			container.getErrorManager().stopErrorProcessing();
			return val.error;
			// Restore it
			//container.getSchemaAccessibility().setRelaxNGValidationLocation(
			//		(String)null );
		} else
			return false;
	}
}
