package com.japisoft.editix.action.xml;

import javax.swing.SwingUtilities;

import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.action.XMLAction;
import com.japisoft.xmlpad.action.xml.CheckableAction;
import com.japisoft.xmlpad.editor.XMLEditor;
import com.japisoft.xmlpad.xml.validator.DefaultValidator;
import com.japisoft.xmlpad.xml.validator.RelaxNGValidator;

import com.japisoft.xmlpad.xml.validator.Validator;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ParseAction extends XMLAction implements CheckableAction {

	public static final String ID = ParseAction.class.getName();

	public ParseAction() {
		super();
	}

	private static Validator defVal;
	
	public static void setDefaultValidator( Validator val ) {
		defVal = val;
	}
	
	@Override
	public XMLContainer getXMLContainer() {
		XMLContainer container = super.getXMLContainer();
		if ( container != null )
			return container;
		return EditixFrame.THIS.getSelectedContainer();
	}

	@Override
	public XMLEditor getXMLEditor() {
		XMLEditor editor = super.getXMLEditor();
		if ( editor != null )
			return editor;
		XMLContainer container = getXMLContainer();
		if ( container == null )
			return null;
		return container.getEditor();
	}
	
	public boolean checkDocument(XMLContainer container, boolean silentMode ) {
		setXMLContainer( container );
		try { 
			return 
				notifyAction( silentMode );
		} finally {
			dispose();
		}
	}	

	public boolean notifyAction() {
		return notifyAction( false );
	}
	
	public boolean notifyAction( boolean silentMode ) {
		getXMLContainer().getErrorManager().flushLastError();
		getXMLContainer().getErrorManager().initErrorProcessing();
		try {
			Validator v = getXMLContainer().getDocumentInfo().getCustomValidator();

			if ( v == null ) {
				if ( getXMLContainer().getSchemaAccessibility().getRelaxNGValidationLocation() != null ) {
					v = new RelaxNGValidator();
				} else {
					if ( defVal != null )
						v = defVal;
					else
						v = new DefaultValidator();
				}
			}

			int status = v.validate( getXMLContainer(), silentMode );

			if ( status == Validator.OK ) {
				getXMLContainer().getErrorManager().notifyNoError( false );
				if ( popupOk && !silentMode ) {
					EditixApplicationModel.fireApplicationValue( "information", getMessageForOk( getXMLContainer() ) );					
				}
			} else {
				if ( status == Validator.ERROR )
				EditixApplicationModel.fireApplicationValue( "error", "Error(s) found while parsing" );
			}

			if ( !silentMode ) {
				SwingUtilities.invokeLater(
					new Runnable() {
						public void run() {
							getXMLContainer().getEditor().requestFocus();			
						}
					}
				);
			}

			return status == Validator.OK;
		} finally {
			getXMLContainer().getErrorManager().stopErrorProcessing();
			getXMLContainer().getEditor().selectFirstError();
		}
	}

	private boolean popupOk = true;

	/** Enabled/Disable a dialog box confirming the parsing step is correct. By default <code>true</code> */
	public void setPopupForOk( boolean popup ) {
		this.popupOk = popup;
	}
	
	private String messageForOk = null;
	
	/** Reset the message when the parsing step is correct */
	public void setMessageForOk( String message ) {
		this.messageForOk = message;
	}

	public String getMessageForOk( XMLContainer container ) {
		if ( messageForOk == null ) {
			messageForOk = container.getLocalizedMessage( "PARSE_OK", "Your document is correct" );
		}
		return messageForOk;
	}

}

// NewAction ends here
