package com.japisoft.editix.action.xml.xinclude;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.japisoft.editix.action.xml.format.FormatAction;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.ui.toolkit.FileManager;
import com.japisoft.framework.xml.DOMToolkit;
import com.japisoft.framework.xml.XMLToolkit;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ExtractNodeAction extends AbstractAction {

	@Override
	public void actionPerformed(ActionEvent e) {
		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null ) {
			EditixFactory.buildAndShowErrorDialog( "No document" );
			return;
		}
		FPNode node = container.getCurrentElementNode();
		if ( node == null ) {
			EditixFactory.buildAndShowErrorDialog( "Please select a node before" );
			return;
		}
		if ( node.isRoot() ) {
			EditixFactory.buildAndShowErrorDialog( "Can't extract the root node, use only descendant elements" );
			return;
		}
		
		File f = FileManager.getSelectedFile( true, "xml", "XML file", container.getCurrentDocumentLocation() );
		if ( f != null ) {
			String xpath = node.getXPathLocation();
			Node n = null;
			try {
				n = XMLToolkit.extractNodeFromXPath( container.getText(), xpath );
			} catch( Exception exc ) {
			}
			if ( n == null ) {
				EditixFactory.buildAndShowErrorDialog( "Can't extract this node ?" );
				return;
			}
			try {
				String newFileContent = XMLToolkit.nodeToText( n );
				XMLToolkit.save( f, newFileContent );
			} catch( Exception exc ) {
				EditixFactory.buildAndShowErrorDialog( "Can't extract or save this node : " + exc.getMessage() );
				return;
			}
			
			Document d = n.getOwnerDocument();
			Element include = d.createElementNS( XMLToolkit.NS_XINCLUDE, "xs:include" );
			include.setAttribute( "href", com.japisoft.framework.app.toolkit.Toolkit.getRelativePath( f, new File( container.getCurrentDocumentLocation() ), true ) );
			n.getParentNode().replaceChild( include, n );
			try {
				container.setText( DOMToolkit.DOM2String( d , 1, true ) );
			} catch( Exception exc ) {
				EditixFactory.buildAndShowErrorDialog( "Can't set the new include node :" + exc.getMessage() );
			}
			
		}
		
	}

}
