package com.japisoft.editix.action.xml.xinclude;

import java.awt.event.ActionEvent;
import java.io.File;
import java.util.Iterator;

import javax.swing.AbstractAction;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.ui.toolkit.FileManager;
import com.japisoft.framework.xml.XMLToolkit;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class IncludeXMLAction extends AbstractAction {

	protected String getFileType() { return "xml"; }
	protected String getFileDescription() { return "XML File"; }
	protected String getParse() { return null; }
	
	@Override
	public void actionPerformed(ActionEvent e) {
		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null ) {
			EditixFactory.buildAndShowErrorDialog( "No document" );
			return;
		}
		File f = FileManager.getSelectedFile( true, getFileType(), getFileDescription(), container.getCurrentDocumentLocation() );
		if ( f != null ) {
			String path = null;
			if ( container.getCurrentDocumentLocation() != null )
				path = com.japisoft.framework.app.toolkit.Toolkit.getRelativePath( f, new File( container.getCurrentDocumentLocation() ) );
			else
				path = f.toString();

			path = path.replace( '\\', '/' );
			FPNode root = container.getRootNode();
			String prefix = "xi";
			boolean nsFound = false;
			
			if ( root != null ) {
				Iterator<String> prefixes = root.getNameSpaceDeclaration();
				while ( prefixes != null && prefixes.hasNext() ) {
					String p = prefixes.next();
					String ns = root.getNameSpaceDeclarationURI( p );
					if ( XMLToolkit.NS_XINCLUDE.matches( ns ) ) {
						prefix = p;
						nsFound = true;
					}
				}
			}
			
			StringBuffer sb = new StringBuffer();
			sb.append( "<" ).append( prefix ).append( ":include" );
			if ( !nsFound )
				sb.append( " xmlns:" ).append( prefix ).append( "=\"" ).append( XMLToolkit.NS_XINCLUDE ).append( "\"" );
			sb.append( " href=\"" ).append( path ).append( "\"" );
			
			String p = getParse();
			if ( p != null )
				sb.append( " parse=\"" ).append( p ).append( "\"" );
			
			sb.append( "/>" );
			container.insertText( sb.toString() );
		}
	}
	
}
