package com.japisoft.editix.action.xml.xinclude;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileInputStream;

import java.util.Iterator;

import javax.swing.AbstractAction;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.ui.toolkit.FileManager;
import com.japisoft.framework.xml.XMLFileData;
import com.japisoft.framework.xml.XMLToolkit;
import com.japisoft.framework.xml.parser.document.Document;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.tree.parser.InnerXMLParser;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class IncludeXMLXPointerAction extends AbstractAction {

	protected String getFileType() { return "xml"; }
	protected String getFileDescription() { return "XML File"; }
	protected String getParse() { return null; }

	@Override
	public void actionPerformed(ActionEvent e) {
		
		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null ) {
			EditixFactory.buildAndShowErrorDialog( "No document" );
			return;
		}
		File f = FileManager.getSelectedFile( true, getFileType(), getFileDescription(), container.getCurrentDocumentLocation() );
		if ( f != null ) {
			String path = null;
			if ( container.getCurrentDocumentLocation() != null )
				path = com.japisoft.framework.app.toolkit.Toolkit.getRelativePath( f, new File( container.getCurrentDocumentLocation() ) );
			else
				path = f.toString();

			path = path.replace( '\\', '/' );
			FPNode root = container.getRootNode();
			String prefix = "xi";
			boolean nsFound = false;
			
			if ( root != null ) {
				Iterator<String> prefixes = root.getNameSpaceDeclaration();
				while ( prefixes != null && prefixes.hasNext() ) {
					String p = prefixes.next();
					String ns = root.getNameSpaceDeclarationURI( p );
					if ( XMLToolkit.NS_XINCLUDE.matches( ns ) ) {
						prefix = p;
						nsFound = true;
					}
				}
			}
			
			StringBuffer sb = new StringBuffer();
			sb.append( "<" ).append( prefix ).append( ":include" );
			if ( !nsFound )
				sb.append( " xmlns:" ).append( prefix ).append( "=\"" ).append( XMLToolkit.NS_XINCLUDE ).append( "\"" );
			sb.append( " href=\"" ).append( path ).append( "\"" );
			
			String p = getParse();
			if ( p != null )
				sb.append( " parse=\"" ).append( p ).append( "\"" );

			try {
				SelectXPointerPanel spp = new SelectXPointerPanel( f );
				if ( 
					DialogManager.showDialog( 
							ApplicationModel.MAIN_FRAME, 
							"Select a node", 
							"XPointer selection", 
							"Select a tree node for selecting it", 
							null, 
							spp, 
							new Dimension( 200, 400 ) ) == DialogManager.OK_ID ) {

					sb.append( " xpointer=\"" + spp.getXPointer() + "\"/>" );
					container.insertText( sb.toString() );
					
				}
			} catch( Throwable th ) {
				EditixFactory.buildAndShowErrorDialog( "Can't parse : " + th.getMessage() );
				return;
			}
		}
			
	}
	
	class SelectXPointerPanel extends JPanel implements TreeSelectionListener {
		
		private JComboBox<String> cb = null;
		private JTree tree = null;
		
		public SelectXPointerPanel( File f ) throws Throwable {
			setLayout( new BorderLayout() );
			XMLFileData d = XMLToolkit.getContentFromInputStream( new FileInputStream( f ), null );
			InnerXMLParser p = new InnerXMLParser();
			Document doc = p.parseContent( d.getContent() );
			
			
			tree = new JTree( new DefaultTreeModel( (TreeNode) doc.getRoot() ) );
			cb = new JComboBox<String>();
			cb.setEditable( true );
			
			add( new JScrollPane( tree ), BorderLayout.CENTER );
			add( cb, BorderLayout.SOUTH );
		}
		
		public String getXPointer() {
			return (String)cb.getSelectedItem();
		}

		public void addNotify() {
			super.addNotify();
			tree.addTreeSelectionListener( this );
		};
		
		public void removeNotify() {
			super.removeNotify();
			tree.removeTreeSelectionListener( this );
		};
		
		@Override
		public void valueChanged( TreeSelectionEvent e ) {
			FPNode node = ( FPNode )tree.getSelectionPath().getLastPathComponent();
			cb.setSelectedItem( node.getXPointerElement() );
		}
		
	}
	
}
