package com.japisoft.editix.action.xquery;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.xml.transform.SourceLocator;

import net.sf.saxon.Configuration;
import net.sf.saxon.query.StaticQueryContext;
import net.sf.saxon.query.XQueryExpression;
import net.sf.saxon.trans.XPathException;

import com.japisoft.editix.editor.xquery.XQueryEditor;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.ui.toolkit.BrowserCaller;
import com.japisoft.p3.Manager;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.action.xml.CheckableAction;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CheckAction extends AbstractAction implements CheckableAction {

	public boolean checkDocument(XMLContainer editor, boolean silentMode ) {
		editor.getErrorManager().initErrorProcessing();			

		Configuration config = new Configuration();
		StaticQueryContext staticContext = 
		        new StaticQueryContext( config );

		// Apply parameters
		for (int i = 0; i < 100; i++) {
			String param = (String) editor
					.getProperty( "xquery.param.name." + i);
			String value = (String) editor
					.getProperty( "xquery.param.value." + i);
			if (param != null && value != null && !"".equals(param)
					&& !"".equals(value)) {

				staticContext.declareNamespace(
						param, 
						value 
				);
			}
		}		
		
		try {
			
			XQueryExpression exp = 
			        staticContext.compileQuery( editor.getText() );
			
			if ( !editor.getErrorManager().hasLastError() ) {
				if ( !silentMode )
					EditixFactory.buildAndShowInformationDialog( "Your XQuery document is correct" );
				editor.getErrorManager().notifyNoError(false);
			}

		} catch( XPathException evt ) {
			
			SourceLocator locator = evt.getLocator();
			editor.getErrorManager().notifyError( 
					null, true, null, locator.getLineNumber(), locator.getColumnNumber(), -1, evt.getMessage(), false );
						
		}

		editor.getErrorManager().stopErrorProcessing();

		return !editor.getErrorManager().hasLastError();
	}

	public void actionPerformed(ActionEvent e) {

		if ( Manager.isFree() ) {		

			EditixFactory.buildAndShowInformationDialog( "This action is not available inside the Free Edition.\nPlease look at http://www.editix.com" );
			BrowserCaller.displayURL( "http://www.editix.com" );
			
		} else {
			//

			XQueryEditor editor = ( XQueryEditor )EditixFrame.THIS.getSelectedContainer();
			if ( editor == null ) {
				EditixFactory.buildAndShowErrorDialog( "Can't Check your XQuery document" );
				return;
			}

			checkDocument( editor, false );
			//
		}
		
	}

}
