package com.japisoft.editix.action.xsl;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;

import com.japisoft.editix.document.DocumentModel;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.xflows.XFlowsEditor;
import com.japisoft.framework.toolkit.FileToolkit;
import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.XMLDocumentInfo;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CreateScenarioAction extends AbstractAction {

	@Override
	public void actionPerformed( ActionEvent ae ) {
		// XSLT source, file,
		/*
		DATAFILE_PROPERTY = XSLT + ".data.file";
		XSLTFILE_PROPERTY = XSLT + ".xslt.file";
		XSLTRESULT_PROPERTY = XSLT + ".result.file"; 
		*/
		
		String key = "xslt";
		String stylesheet = "stylesheet";
		
		IXMLPanel panel = EditixFrame.THIS.getSelectedPanel();

		if ( "XQR".equals( panel.getMainContainer().getDocumentInfo().getType() ) ) {
			key = "xquery";
			stylesheet = "xquery";
		}
		
		
		String source = ( String )panel.getProperty( key + ".data.file" );
		String xslt = ( String )panel.getProperty( key + ".xslt.file" );
		String result = ( String )panel.getProperty( key + ".result.file" );
		
		if ( source == null || xslt == null || result == null ) {
			EditixFactory.buildAndShowErrorDialog( "Invalid parameters for building the scenario" );
			return;
		}
		
		String sourceExt = FileToolkit.fileExt( source );
		String resultExt = FileToolkit.fileExt( result );
		
		StringBuffer sb = new StringBuffer( "<?xml version='1.0'?>\n" );
		sb.append( "<scenario>\n" );
		sb.append( "\t<task name=\"myTask\" type=\"" + key.toUpperCase() + "\">\n" );
		sb.append( "\t\t<params>\n" );
		sb.append( "\t\t\t<param key=\"sourcefilter\" value=\"(.*)\\." ).append( sourceExt ).append( "\" type=\"0\"/>\n" );		
		sb.append( "\t\t\t<param key=\"" + stylesheet + "\" value=\"" ).append( xslt ).append( "\" type=\"0\"/>\n" );
		sb.append( "\t\t\t<param key=\"targetname\" value=\"$1." ).append( resultExt ).append( "\" type=\"0\"/>\n" );
		sb.append( "\t\t\t<param key=\"targetpath\" value=\"" ).append( FileToolkit.parentPath( result ) ).append( "\" type=\"0\"/>\n" );
		sb.append( "\t\t\t<param key=\"sourcepath\" value=\"" ).append( FileToolkit.parentPath( source ) ).append( "\" type=\"0\"/>\n" );
		sb.append( "\t\t</params>\n" );
		sb.append( "\t\t</task>\n" );
		sb.append( "</scenario>" );

		XMLDocumentInfo doc = 
				DocumentModel.getDocumentForType( "XSC" );
		IXMLPanel editor = 
			EditixFactory.getPanelForDocument( doc );
		
		editor.getMainContainer().setText( sb.toString() );
		
		XMLContainer container = 
			panel.getMainContainer();
		XMLDocumentInfo newDoc = 
			doc.cloneDocument();
		editor.setDocumentInfo( newDoc );
		
		EditixFrame.THIS.addContainer( editor );
		EditixFrame.THIS.updateCurrentXMLContainer( editor );

	}

}
