package com.japisoft.editix.action.xsl;

import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.dom.DOMSource;

import com.japisoft.editix.wizard.document.JSON2XMLDocument;
import com.japisoft.framework.toolkit.FileToolkit;
import com.japisoft.xmlpad.SharedProperties;
import com.japisoft.xmlpad.xml.validator.XMLPadSAXParserFactory;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DOMSourceFactory {

	private DOMSourceFactory() {
	}

	private static DOMSourceFactory INSTANCE = null;

	public static DOMSourceFactory Instance() {
		if ( INSTANCE == null )
			INSTANCE = new DOMSourceFactory();
		return INSTANCE;
	}
	
	public DOMSource getDOMSource( String filePath ) throws Throwable {
		DocumentBuilder builder = XMLPadSAXParserFactory.getNewDocumentBuilder( false, false );
		if ( SharedProperties.DEFAULT_ENTITY_RESOLVER != null )
			builder.setEntityResolver( SharedProperties.DEFAULT_ENTITY_RESOLVER );
		
		File f = new File( filePath );

		String type = FileToolkit.fileExt( filePath );
		if ( "json".equals( type ) || "jso".equals( type ) ) {
			return new DOMSource( JSON2XMLDocument.JSONTODOM( f ) );
		}

		return new DOMSource( builder.parse( f ) );				
	}
	
}
