package com.japisoft.editix.db.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;

import org.xmldb.api.base.XMLDBException;

import com.japisoft.editix.db.Driver;
import com.japisoft.editix.db.DriverDbManager;
import com.japisoft.editix.ui.EditixFactory;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DbConnectionPanel extends javax.swing.JPanel implements ActionListener {
    
    /** Creates new form DbConnectionPanel */
    public DbConnectionPanel() {
        initComponents();
        
        for ( int i = 0; i < DriverDbManager.getDriverCount(); i++ ) {
        	cbDriver.addItem( 
        			DriverDbManager.getDriver( i ) );
        }

        btnTest.setIcon( 
        	new ImageIcon( getClass().getResource( "data_ok.png" ) ) 
        );
    }

    public void addNotify() {
    	super.addNotify();
    	cbDriver.addActionListener( this );
    	if ( cbDriver.getItemCount() > 0 )
    		cbDriver.setSelectedIndex( 0 );
    	else
    		System.out.println( "No driver found ??" );
    	btnTest.addActionListener( this );
    }

    public void removeNotify() {
    	super.removeNotify();
    	cbDriver.removeActionListener( this );
    	btnTest.removeActionListener( this );
    }

    public void actionPerformed(ActionEvent e) {
    	if ( e.getSource() == cbDriver ) {
    		Driver d = ( Driver )cbDriver.getSelectedItem();
    		txtUrl.setText( d.url );

    		txtPassword.setEnabled( !d.embedded );
    		txtUser.setEnabled( !d.embedded );
    		
    	} else
    	if ( e.getSource() == btnTest ) {
    		Driver d = ( Driver )cbDriver.getSelectedItem();
    		
    		try {
    		
	    		if ( d.test( txtUrl.getText(), txtUser.getText(), txtPassword.getText() ) ) {
	    			EditixFactory.buildAndShowInformationDialog( "Success Connection" );
	    		} else
	    			EditixFactory.buildAndShowErrorDialog( "Can't connect" );
	    		
    		} catch( Exception exc ) {

    			String message = exc.getMessage();
    			
    			if ( exc instanceof XMLDBException ) {
    				
    				XMLDBException xde = ( XMLDBException )exc;
    				if ( xde.getCause() != null ) {
    					message = xde.getCause().getMessage();
    				}
    				
    			}

    			EditixFactory.buildAndShowErrorDialog( "Can't connect : [ " + message + " ]" );
    			
    		}
    	}
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">
    private void initComponents() {
        jLabel1 = new javax.swing.JLabel();
        cbDriver = new javax.swing.JComboBox();
        jSeparator1 = new javax.swing.JSeparator();
        jLabel2 = new javax.swing.JLabel();
        txtUrl = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        txtUser = new javax.swing.JTextField();
        jLabel4 = new javax.swing.JLabel();
        txtPassword = new javax.swing.JPasswordField();
        btnTest = new javax.swing.JButton();

        jLabel1.setText("Driver");
        jLabel2.setText("URL/Path");
        jLabel3.setText("User");
        jLabel4.setText("Password");
        btnTest.setText("Test");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(cbDriver, 0, 224, Short.MAX_VALUE)
                    .add(jLabel1)
                    .add(jSeparator1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 224, Short.MAX_VALUE)
                    .add(jLabel2)
                    .add(txtUrl, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 224, Short.MAX_VALUE)
                    .add(jLabel3)
                    .add(txtUser, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 224, Short.MAX_VALUE)
                    .add(jLabel4)
                    .add(txtPassword, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 224, Short.MAX_VALUE)
                    .add(btnTest))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel1)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(cbDriver, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jSeparator1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 10, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel2)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(txtUrl, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel3)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(txtUser, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel4)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(txtPassword, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 18, Short.MAX_VALUE)
                .add(btnTest)
                .addContainerGap())
        );
    }// </editor-fold>

    // Variables declaration - do not modify
    private javax.swing.JComboBox cbDriver;
    private javax.swing.JButton btnTest;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JPasswordField txtPassword;
    private javax.swing.JTextField txtUrl;
    private javax.swing.JTextField txtUser;
    // End of variables declaration
    
    // ---------------------------------------------------------------------------------

    public String getUrl() { return txtUrl.getText(); }
    public String getUser() { return txtUser.getText(); }
    public String getPassword() { return txtPassword.getText(); }
    public Driver getDriver() { return (Driver)cbDriver.getSelectedItem(); }

}
