package com.japisoft.editix.db.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;

import com.japisoft.editix.db.NodeDb;
import com.japisoft.editix.db.RootNodeDb;
import com.japisoft.editix.editor.xquery.XQueryEditor;
import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DbRequestPanel extends JPanel implements ActionListener {

	private DbBrowser browser;

	/** Creates new form DbRequestPanel */
    public DbRequestPanel( DbBrowser browser ) {
        initComponents();
        this.browser = browser;
        txtRequest.setDisplayRowHeader( false );
//        Icon i = com.japisoft.framework.ui.Toolkit.getIconFromClasspath("images/gear_run.png");
//btRun.setIcon( i );
        btRun.setToolTipText( "Run an XPath request on the current database connection" );
//btRunCurrentDoc.setIcon( i );
        btRunCurrentDoc.setToolTipText( "Run an XPath request using the edited one (from the main editor)" );
    }

    public void addNotify() {
    	super.addNotify();
    	btRun.addActionListener( this );
    	btRunCurrentDoc.addActionListener( this );
    }

    public void removeNotify() {
    	super.removeNotify();
    	btRun.removeActionListener( this );
    	btRunCurrentDoc.removeActionListener( this );
    }
    
    public void actionPerformed(ActionEvent e) {
    	if ( e.getSource() == btRun ) {
    		runRequest( txtRequest.getText()  );
    	} else {
    		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
    		if ( container == null ) {
    			EditixFactory.buildAndShowWarningDialog( "You must open an XQuery document" );
    		} else {
    			String type = container.getDocumentInfo().getType();
    			if ( !"XQR".equals( type ) ) {
    				EditixFactory.buildAndShowWarningDialog( "The current document is not an xquery document" );
    			} else {
    				runRequest( container.getText() );
    			}
    		}
    	}
    }

    private void runRequest( String request ) {
    	
    	RootNodeDb node = browser.getCurrentConnection();
    	if ( node == null )
    		EditixFactory.buildAndShowWarningDialog( "Please select a database" );
    	else {
    		try {

				DefaultMutableTreeNode root = spResult.getRoot();
				root.removeAllChildren();
				
				NodeDb[] res = 
					node.request( request );				

				// Clean the root

				if ( res != null ) {		

					TreeNode parent = null;

					// Add the common container
					
					for ( int i = 0; i < res.length; i++ ) {

						if ( res[ i ].getParent() == null ) {
							
							root.add( res[ i ] );

						} else
						
						if ( res[ i ].getParent() != parent ) {
							// They share the same container
							root.add( ( MutableTreeNode )res[ i ].getParent() );	
						}
						
						parent = res[ i ].getParent();
						
					}

				}

				spResult.refreshRoot();
				if ( res != null && res.length > 0 )
					tp.setSelectedIndex( 1 );

			} catch ( Exception e1 ) {

				EditixApplicationModel.debug( e1 );
				
				EditixFactory.buildAndShowErrorDialog( "Can't execute : " + e1.getMessage() );

			}
    	}
    }
    
    public void setEnabledRequest( boolean enabled ) {
    	this.btRun.setEnabled( enabled );
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">                          
    private void initComponents() {
        tp = new javax.swing.JTabbedPane();
        pnlXPath = new javax.swing.JPanel();
        btRun = new javax.swing.JButton();
        btRunCurrentDoc = new javax.swing.JButton();
        txtRequest = new XQueryEditor();
        jSeparator1 = new javax.swing.JSeparator();
        jPanel1 = new javax.swing.JPanel();
        spResult = new DbRequestResultPanel();
        
        setBorder(javax.swing.BorderFactory.createTitledBorder("Request"));
        tp.setTabPlacement(javax.swing.JTabbedPane.BOTTOM);
        
        btRun.setText("Run");
        btRunCurrentDoc.setText("Run XQuery document");

        org.jdesktop.layout.GroupLayout pnlXPathLayout = new org.jdesktop.layout.GroupLayout(pnlXPath);
        pnlXPath.setLayout(pnlXPathLayout);
        pnlXPathLayout.setHorizontalGroup(
            pnlXPathLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(pnlXPathLayout.createSequentialGroup()
                .addContainerGap()
                .add(btRun)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(btRunCurrentDoc)
                .add(75, 75, 75))
            .add(txtRequest.getView(), org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 223, Short.MAX_VALUE)
            .add(jSeparator1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 223, Short.MAX_VALUE)
        );
        pnlXPathLayout.setVerticalGroup(
            pnlXPathLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, pnlXPathLayout.createSequentialGroup()
                .add(txtRequest.getView(), org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 221, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jSeparator1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(pnlXPathLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btRunCurrentDoc)
                    .add(btRun))
                .addContainerGap())
        );
        tp.addTab("XPath", pnlXPath);

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(spResult, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 223, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(spResult, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 266, Short.MAX_VALUE)
        );
        tp.addTab("Result", jPanel1);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(tp, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 228, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(tp, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 294, Short.MAX_VALUE)
        );
    }// </editor-fold>                        

    // Variables declaration - do not modify                     
    private javax.swing.JButton btRun;
    private javax.swing.JButton btRunCurrentDoc;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JPanel pnlXPath;
    private DbRequestResultPanel spResult;
    private javax.swing.JTabbedPane tp;
    private XQueryEditor txtRequest;

    // End of variables declaration}
}
