package com.japisoft.editix.db.xmldb;

import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Database;

import com.japisoft.editix.db.Driver;
import com.japisoft.editix.db.RootNodeDb;
import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.framework.xml.parser.node.FPNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XmlDbDriver extends Driver {

	// Specific to XML DB driver
	public String dbDriver;

	public XmlDbDriver( String name ) {
		super( name );
	}

	public XmlDbDriver( 
			FPNode driverNode ) {
		super( driverNode );
		this.dbDriver= driverNode.getAttribute( "xmldbDriver" );
	}

	public boolean test(String url, String user, String password) throws Exception {
		install();
		RootNodeDb root = getRoot( url, user, password );
		boolean ok = root.canConnect();
		if ( root != null ) {
			try {
				root.close();
			} catch( Exception e ) {
			}
		}
		return ok;
	}

	protected void install(ClassLoader loader) {
		try {
		
			if ( dbDriver != null )

				installXmlDbDriver( dbDriver, loader );

		} catch (Exception e) {

			EditixApplicationModel.debug( e );

		}
	}


	public static void installXmlDbDriver( String driver, ClassLoader loader ) throws Exception {
		
		Class cl = loader.loadClass( driver );  
        Database database = (Database) cl.newInstance();  
        DatabaseManager.registerDatabase(database);

	}	
	
	public RootNodeDb getRoot(String url, String user, String password) throws Exception {
		install();
		return new XmlDbRootDbImpl( toString(), url, user, password );
	}	

}
