package com.japisoft.editix.document;

import java.util.ArrayList;
import java.util.List;
import javax.swing.Icon;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.xml.parser.node.FPNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class GroupTemplate {

	private String name;
	private String icon;
	private String docIcon;
	
	public GroupTemplate( String name, String icon, String docIcon ) {
		this.name = name;
		this.icon = icon;
		this.docIcon = docIcon;
	}
	
	private List<TemplateInfo> templates = null;
	
	public void addTemplate( TemplateInfo ti ) {
		if ( templates == null )
			templates = new ArrayList<TemplateInfo>();
		templates.add( ti );
	}

	public int getTemplateInfoCount() {
		if ( templates == null )
			return 0;
		return templates.size();
	}
	
	public TemplateInfo getTemplateInfo( int index ) {
		return templates.get( index );
	}
	
	public String getName() {
		return name;
	}

	public Icon getIcon() {
		try {
			return com.japisoft.framework.app.toolkit.Toolkit.getImageIcon( icon );
		} catch( Exception exc ) {
			ApplicationModel.debug( "Can't read " + icon );
			return null;
		}
	}

	public Icon getDocIcon() {
		try {
			return com.japisoft.framework.app.toolkit.Toolkit.getImageIcon( docIcon );
		} catch( Exception exc ) {
			ApplicationModel.debug( "Can't read " + docIcon );
			return null;
		}
	}
		
	public FPNode toXML() {
		FPNode n = new FPNode( FPNode.TAG_NODE, "group" );
		n.setAttribute( 
			"label", 
			name.replace( "<", "&lt;" ).replace( ">", "&gt;" ) 
		);
		
		if ( templates != null ) {
			for ( TemplateInfo ti : templates ) {
				n.appendChild( ti.toXML() );
			}
		}

		return n;
	}

}
