package com.japisoft.editix.document.json;

import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONKey;
import org.json.JSONObject;
import org.json.JSONTokener;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SchemaResolver {

	private JSONObject root;
	
	public SchemaResolver( String path ) throws IOException {

		root = new JSONObject( new JSONTokener( new FileReader( path ) ) );
		
	}
	
	public List<SchemaProperty> resolve( Object subject, String pointer ) {
		
		List<SchemaProperty> res = new ArrayList<SchemaProperty>();
		
		try {
		
			// Update the pointer
			String[] parts = pointer.split( "/" );
			
			JSONObject parent = root;
			
			for( String part : parts ) {
				if ( "#".equals( part ) ) {
					parent = root;
				} else {
					// Array index
					if ( part.matches( "/d+" ) ) {
						
					} else {
						// Property
	
						if ( parent.has( "properties" ) ) {
							
							parent = ( JSONObject )parent.get( "properties" );
							if ( parent.has( part ) ) {
								parent = ( JSONObject )parent.get( part );
								
							} else
								break;
	
						} else
							break;
											
					}
				}			
			}
			
			if ( parent.has( "type" ) ) {
				String type = ( String )parent.get( "type" );
				if ( "object".equals( type ) ) {
					// Extract all properties
					
					if ( parent.has( "properties" ) ) {
	
						JSONObject properties = ( JSONObject )parent.get( "properties" );
						for ( int i = 0; i < properties.getKeyCount(); i++ ) {
							JSONKey key = properties.getKey( i );
							String property = key.getKey();
							JSONObject obj = properties.getJSONObject( property );							
							SchemaProperty sp = new SchemaProperty( property, obj.getString( "type" ), obj.getString( "description", "" ) );
							res.add( sp );
							
						}
						
					}
					
					if ( parent.has( "required" ) ) {
						
						JSONArray array = ( JSONArray )parent.getJSONArray( "required" );
						for ( int i = 0; i < array.length(); i++ ) {
							String name = array.getString( i );
							for ( SchemaProperty sp : res ) {
								if ( sp.name.equals( name ) )
									sp.required = true;
							}
						}
						
					}
					
				}
			}
			
			// Remove already present properties inside the subject
			
			if ( subject instanceof JSONObject ) {
				JSONObject obj = ( JSONObject )subject;
				for ( int i = 0; i < obj.getKeyCount(); i++ ) {
					JSONKey key = obj.getKey( i );
					for ( int j = 0; j < res.size(); j++ )
						if ( res.get( j ).name.equals( key.getKey() ) ) {
							res.remove( j );
							break;
						}
				}
			}
			
		} catch( ClassCastException exc ) {
			exc.printStackTrace();
		}

		return res;
		
	}
	
	public static void main( String[] args ) throws Exception {
		
		SchemaResolver sr = new SchemaResolver( "C:/Users/alexandre/Documents/testEditiX/schema1.json" );
		JSONObject root = new JSONObject( new JSONTokener( new FileReader( "C:/Users/alexandre/Documents/testEditiX/test-schema.json"  ) ) );
		List<SchemaProperty> l = sr.resolve( root, "#" );
		System.out.println( l.size() + " properties" );
		for( SchemaProperty sp : l ) {
			System.out.println( "- " + sp.name + " [" + sp.type + "] - " + sp.description + " " + ( sp.required ? " required " : "" ) );
		}
		
	}
	
}
