package com.japisoft.editix.document.schema;

import java.io.IOException;

import org.apache.xerces.impl.xs.XMLSchemaLoader;
import org.apache.xerces.xni.parser.XMLErrorHandler;
import org.apache.xerces.xni.parser.XMLInputSource;
import org.apache.xerces.xni.parser.XMLParseException;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.error.ErrorManager;
import com.japisoft.xmlpad.xml.validator.Validator;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SchemaValidator implements Validator, XMLErrorHandler {

	public void error(String arg0, String arg1, XMLParseException arg2)
			throws org.apache.xerces.xni.XNIException {
		currentErrorManager.notifyError(this, true, arg2.getBaseSystemId(),
				arg2.getLineNumber(), arg2.getColumnNumber(), -1, arg2
						.getMessage(), false);
	}

	public void fatalError(String arg0, String arg1, XMLParseException arg2)
			throws org.apache.xerces.xni.XNIException {
		currentErrorManager.notifyError(this, true, arg2.getBaseSystemId(),
				arg2.getLineNumber(), arg2.getColumnNumber(), -1, arg2
						.getMessage(), false);
	}

	public void warning(String arg0, String arg1, XMLParseException arg2)
			throws org.apache.xerces.xni.XNIException {
	}

	ErrorManager currentErrorManager = null;

	public int validate(XMLContainer container, boolean silentMode ) {

		if (container.getCurrentDocumentLocation() == null) {
			if ( !silentMode )
				EditixFactory
					.buildAndShowInformationDialog("Please save your document before checking");
			return WARNING;
		}
		
		if ( Preferences.getPreference( "xsdEditor", "saveBeforeChecking", true ) ) {
			// Save it
			ActionModel.activeActionById("save", null);
		}

		currentErrorManager = container.getErrorManager();
		currentErrorManager.initErrorProcessing();

		try {

			XMLSchemaLoader schemaLoader = new XMLSchemaLoader();
			schemaLoader.setErrorHandler(this);
//			SharedProperties.DEFAULT_ENTITY_RESOLVER );
			try {
				schemaLoader.loadGrammar(new XMLInputSource(null, container
						.getCurrentDocumentLocation(), null));
			} catch (org.apache.xerces.xni.XNIException e) {

				if (e.getCause() instanceof XMLParseException) {
					XMLParseException p = (XMLParseException) e.getCause();
					currentErrorManager.notifyError(this, true, p
							.getLiteralSystemId(), p.getLineNumber(), p
							.getColumnNumber(), -1, p.getMessage(), false);
				}

			} catch (IOException e) {
			}

		} finally {
			boolean error = currentErrorManager.hasLastError();
			currentErrorManager.stopErrorProcessing();
			currentErrorManager = null;

			if (error)
				return ERROR;
		}

		return OK;
	}

}
