package com.japisoft.editix.document.xslt;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.handler.AbstractHelperHandler;

import com.japisoft.xmlpad.helper.model.AttDescriptor;
import com.japisoft.xmlpad.helper.model.BasicDescriptor;
import com.japisoft.xmlpad.helper.model.TagDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SaxonExtHandler extends AbstractHelperHandler {

	static Map<String,List<String>> grammar = new HashMap<String,List<String>>();

	static {		
		InputStream input = SaxonExtHandler.class.getResourceAsStream( "saxon.grammar" );
		InputStreamReader r = new InputStreamReader( input );
		BufferedReader br = new BufferedReader( r );
		try {
			String l = null;
			List<String> lst = null;
			while ( ( l = br.readLine() ) != null ) {
				if ( l.startsWith( "*" ) ) {
					grammar.put( l.substring( 1 ), lst = new ArrayList<String>() );
				} else
					lst.add( l );
			}
		} catch( IOException exc ) {
			
		} finally {
			try {
				br.close();
			} catch( Throwable th ) {
			}
		}
	}

	protected void installDescriptors( 
			FPNode currentNode,
			XMLPadDocument document, 
			int offset,
			String addedString ) {

		boolean inTag = document.isInsideTag(offset);
		
		if ( "<".equals( addedString ) || ( addedString == null && !inTag ) ) {
			// elements
			List<String> lst = grammar.get( "elements" );
			for ( String l : lst ) {
				TagDescriptor td = new TagDescriptor( l, false );
				td.setAddedPart( addedString );
				addDescriptor( td );
			}
		} else {
			
			if ( document.isInsideAttributeValue( offset ) ) {
				List<String> lst = grammar.get( "functions" );
				for ( String l : lst ) {
					addDescriptor( new BasicDescriptor( l ) );
				}
			} else
				if ( inTag ) {
					List<String> lst = grammar.get( "attributes" );
					for ( String l : lst ) {
						AttDescriptor ad = new AttDescriptor( l, null );
						ad.setAddedPart( " " );
						addDescriptor( ad );
					}
				}			
		}
		
		
		
	}

	protected String getActivatorSequence() {
		return null;
	}

	private boolean isForSaxon( FPNode currentNode ) {
		if ( "http://icl.com/saxon".equals( currentNode.getNameSpaceDeclarationURI( "saxon" ) ) )
			return true;
		if ( currentNode.getFPParent() != null ) {
			return isForSaxon( currentNode.getFPParent() );
		} else
			return false;
	}
	
	public boolean haveDescriptors(
			FPNode currentNode,
			XMLPadDocument document, 
			boolean insertBefore, 
			int offset,
			String activatorString ) {

		if ( currentNode == null )
			return false;

		return ( null == activatorString ) && isForSaxon( currentNode );
	}

	public String getTitle() {
		return "Saxon extensions";
	}

	public int getPriority() {
		return -1;
	}

}
