package com.japisoft.editix.document.xslt;

import javax.xml.transform.ErrorListener;
import javax.xml.transform.SourceLocator;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamSource;

import org.xml.sax.SAXParseException;

import net.sf.saxon.s9api.Processor;
import net.sf.saxon.s9api.SaxonApiException;
import net.sf.saxon.s9api.XsltCompiler;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.error.ErrorManager;
import com.japisoft.xmlpad.xml.validator.Validator;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XSLT2Validator implements Validator, ErrorListener {
	
	ErrorManager currentErrorManager = null;

	public int validate(XMLContainer container, boolean silentMode ) {
		
		if ( EditixFactory.mustSaveDialog( container ) ) {
			return WARNING;
		}

		currentErrorManager = container.getErrorManager();
		currentErrorManager.initErrorProcessing();

		try {
		
			// Save it
			ActionModel.activeActionById("save", null);
	
			Processor p = new Processor( false );
			XsltCompiler compiler = p.newXsltCompiler();
			compiler.setErrorListener( this );
			try {
				compiler.compile( new StreamSource( 
						container.getCurrentDocumentLocation() ) );
			} catch (SaxonApiException e) {
				return ERROR;
			}
		} finally {
			boolean error = currentErrorManager.hasLastError();
			currentErrorManager.stopErrorProcessing();
			currentErrorManager = null;
			if (error)
				return ERROR;
		}
		return OK;
	}

	public void error(TransformerException te) throws TransformerException {
		SourceLocator locator = te.getLocator();
		if ( locator == null )
			locator = new SourceLocator() {
			public int getColumnNumber() {
				return -1;
			}
			public int getLineNumber() {
					return -1;
			}
			public String getPublicId() {
				return null;
			}
			public String getSystemId() {
				return null;
			}
		};

		int lineError = locator.getLineNumber();
		int colError = locator.getColumnNumber();
		
		Throwable th  = te.getCause();
		int i = 0;
		while ( th != null ) {
			if ( th instanceof SAXParseException ) {
				SAXParseException spe = ( SAXParseException )th;
				colError = spe.getColumnNumber();
				lineError = spe.getLineNumber();
				break;
			}
			if ( th.getCause() == th )
				break;			
			th = th.getCause();
			i++;
			if ( i > 10 )	// Non loop add-on
				break;
		}

		currentErrorManager.notifyError(
				this, 
				true, 
				null,
				lineError, 
				colError, 
				-1, 
				te.getMessage(), 
				false );

	}

	public void fatalError(TransformerException te)
			throws TransformerException {
		error( te );
	}

	public void warning(TransformerException arg0) throws TransformerException {
		
	}
	
}
