package com.japisoft.editix.editor.css.helper;

import java.awt.Color;

import javax.swing.text.BadLocationException;
import javax.swing.text.Element;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.handler.AbstractHelperHandler;
import com.japisoft.xmlpad.helper.model.BasicDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class PropertiesHandler extends AbstractHelperHandler {

	protected String getActivatorSequence() {
		return null;
	}

	public String getTitle() {
		return "CSS properties";
	}

	protected void installDescriptors(
			FPNode currentNode,
			XMLPadDocument document, 
			int offset,
			String activatorString ) {

		for ( int i = 0; i< Keywords.properties.length; i++ ) {
			BasicDescriptor rd = new BasicDescriptor( Keywords.properties[ i ] );
			if ( "{".equals( activatorString ) )
				rd.setAddedPart( "{\n" );
			else
				rd.setAddedPart( activatorString );
			addOrderedDescriptor( rd );
		}
		
		Color c = new Color( 100, 100, 0 );
		
		for ( int i = 0; i< Keywords.CSS3Properties.length; i++ ) {
			BasicDescriptor rd = new BasicDescriptor( Keywords.CSS3Properties[ i ] );
			rd.setComment( "CSS3 Property" );
			rd.setColor( c );
			if ( "{".equals( activatorString ) )
				rd.setAddedPart( "{\n" );
			else
				rd.setAddedPart( activatorString );
			addOrderedDescriptor( rd );
		}
				
	}

	public boolean haveDescriptors(FPNode currentNode,
			XMLPadDocument document, boolean insertBefore, int offset,
			String activatorString) {		
		if ( activatorString == null && 
				!checkForLeftPoint(document, offset - 1) && inSelector(document, offset))
			return true;
		return false;
	}

	static boolean inSelector( XMLPadDocument document, int offset ) {
		try {
			int index = document.getDefaultRootElement().getElementIndex( offset );
			int line = index - 1;	
			// Search for {
			before:for ( ; line >=0; line-- ) {
				Element e = document.getDefaultRootElement().getElement( line );
				for ( int i = e.getStartOffset(); i <= e.getEndOffset(); i++ ) {
					if ( "}".equals( document.getText( i, 1 ) ) )
						return false;
					if ( "{".equals( document.getText( i, 1 ) ) )
						break before;
				}
			}
			line = index;
			for ( ; line < document.getDefaultRootElement().getElementCount(); line++ ) {
				Element e = document.getDefaultRootElement().getElement( line );
				if ( e == null )
					return false;

				int start = e.getStartOffset();
				if ( line == index ) {
					start = offset;
				}
				for ( int i = start; i <= e.getEndOffset(); i++ ) {
					if ( "}".equals( document.getText( i, 1 ) ) )
						return true;
					if ( "{".equals( document.getText( i, 1 ) ) )
						return false;
				}
			}			
			return false;
		} catch( BadLocationException ble ) {
			return false;
		}
	}

	private boolean checkForLeftPoint( XMLPadDocument document, int offset ) {
		try {
			int index = document.getDefaultRootElement().getElementIndex( offset );
			Element e = document.getDefaultRootElement().getElement( index );
			
			for ( int i = offset; i >=e.getStartOffset(); i-- ) {
				char c = document.getText( i, 1 ).charAt( 0 );
				if ( Character.isWhitespace( c ) )
					continue;
				else
					if ( c == ':' )
						return true;
			}
		} catch (BadLocationException e) {
			
		}
		return false;
	}
	
}
