package com.japisoft.editix.editor.js;

import java.util.ArrayList;

import javax.swing.Action;
import javax.swing.JFrame;
import javax.swing.JToolBar;
import javax.swing.text.EditorKit;
import javax.swing.text.PlainDocument;

import com.japisoft.editix.editor.js.helper.CommentHandler;
import com.japisoft.editix.editor.js.helper.ObjectInScopeHandler;
import com.japisoft.editix.editor.js.helper.ParametersHandler;
import com.japisoft.editix.editor.js.kit.JSEditorKit;
import com.japisoft.editix.ui.EditixErrorPanel;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.xmlpad.ComponentFactory;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.EditorContext;
import com.japisoft.xmlpad.editor.XMLEditor;
import com.japisoft.xmlpad.elementview.ElementView;
import com.japisoft.xmlpad.tree.parser.Parser;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class JSContainer extends XMLContainer {

	static boolean STANDALONE = false;
	
	private Action parseAction = null;
	
	public JSContainer( Action parseAction ) {
		super( new ComponentFactoryForJSEditor() );
		this.parseAction = parseAction;
		getUIAccessibility().setToolBarAvailable( false );
		setStatusBarAvailable( false );
		getUIAccessibility().setPopupAvailable( false );
		getUIAccessibility().setTreePopupAvailable( false );
		setAutoNewDocument( false );
		setErrorPanelAvailable( true );
		getDocumentInfo().setRealTimeTree( true );		

		setEnabledTreeLocationForCaret( true );
		
		setDisposeAction(false);
		setTreeAvailable( true );
		setTreePopupAvailable( false );
		setTreeElementViewAvailable( false );

		getDocument().putProperty(PlainDocument.tabSizeAttribute,
				new Integer(Preferences.getPreference("file", "tab-size", 2)));

		ArrayList assistant = new ArrayList();
		assistant.add( new CommentHandler() );
		assistant.add( new ObjectInScopeHandler() );
		assistant.add( new ParametersHandler() );
		getHelperManager().resetHandlers( assistant, false );

		getUIAccessibility().setErrorView(new EditixErrorPanel());
		setDisposeAction(false);		
	}

	@Override
	public Parser createNewParser( boolean lightweightMode ) {
		return new JSParser();
	}	
	
	public String getPreferenceGroupe() {
		return "JSEditor";
	}

	public Action getAction(String actionId) {
		if ( "parse".equals( actionId ) ) {
			return parseAction;
		} else
			return super.getAction( actionId );
	}

	static class ComponentFactoryForJSEditor extends ComponentFactory {
		public XMLEditor getNewXMLEditor(EditorContext context) {
			return new CustomXMLEditorForJS( context );
		}
		@Override
		public ElementView getNewElementView(XMLContainer container) {
			return null;
		}
		@Override
		public JToolBar getNewTreeToolBar() {
			return null;
		}		
	}

	static class CustomXMLEditorForJS extends XMLEditor {
		CustomXMLEditorForJS( EditorContext context ) {
			super( context );
		}
		public EditorKit getEditorKit() {
			return new JSEditorKit( "JSEditor" );
		}
	}
	
	public static void main( String[] args ) {
		ApplicationModel.SHORT_APPNAME = "test";
		JFrame f = new JFrame();
		f.add( new JSContainer( null ).getView() );
		f.setSize( 300, 300 );
		f.setVisible( true );
	}
	
}
