package com.japisoft.editix.editor.js;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.io.IOUtils;

import com.japisoft.framework.collection.FastVector;
import com.japisoft.framework.xml.parser.ErrorParsingListener;
import com.japisoft.framework.xml.parser.ParseException;
import com.japisoft.framework.xml.parser.document.Document;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.parser.node.NodeFactory;
import com.japisoft.xmlpad.tree.parser.Parser;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class JSParser implements Parser {

	public boolean hasError() {
		return false;
	}

	public void interruptParsing() {
	}
	
	public boolean isInterrupted() {
		return false;
	}
	
	@Override
	public void setLightweightMode(boolean b) {
	}
	@Override
	public boolean isLightweightMode() {
		return false;
	}
	
	
	public Document parse(Reader reader, Object context ) throws ParseException {
		try {

			String js = IOUtils.toString( reader );
			String[] lines = js.split( "\n" );

			Document doc = new Document();
			ArrayList<FPNode> flatNodes = new ArrayList<FPNode>(); 
			doc.setFlatNode( flatNodes );

			FPNode root = new FPNode( FPNode.TAG_NODE, "javascript" );
			root.setStartingOffset( 0 );
			root.setStoppingOffset( js.length() );
			doc.setRoot( root );
			root.setDocument( doc );
			
			Pattern pat = Pattern.compile( "function\\s+(\\w+)\\s*\\(" );
			int i = 1;
			int offset = 0;
			
			FPNode nn = null;
			int openCpt = 0;
			
			for ( String line : lines ) {
				Matcher matcher = pat.matcher( line );
				if ( matcher.find() ) {
					String function = matcher.group( 1 );
					int start = matcher.regionStart() + offset;
					int end = matcher.regionEnd() + offset;
					
					nn = new FPNode( FPNode.TAG_NODE, function );
					nn.startingOffset = start;
					nn.stoppingOffset = end;
					nn.setStartingLine( i );
					nn.setStoppingLine( i );
					
					flatNodes.add( nn );
					
					root.appendChild( nn );
				} else {
					
					if ( nn != null ) {

						openCpt += line.length() - line.replace("{", "").length();
						int endP = line.length() - line.replace("}", "").length();
						openCpt -= endP;
						if ( openCpt <= 0 && endP > 0 ) {
							nn.setStoppingOffset( offset );
							nn.setStoppingLine( i );
						}
						
					}
					
				}
				offset += line.length() + 1;
				i++;
			}
			
			return doc;
			
		} catch( IOException exc ) {
			throw new ParseException( exc.getMessage() );
		}
	}
	
	public void setBackgroundMode(boolean b) {
	}

	public void setErrorSignal(ErrorParsingListener parsingErrorListener) {
	}

	public void setFlatView(boolean b) {
	}

	public void setNodeFactory(NodeFactory factory) {
	}

	public void setParsingMode(int continueParsingMode) {
	}
	
	public static void main( String[] args ) throws Exception {
		JSParser p = new JSParser();
		StringReader sr = new StringReader( "function a() {\n\naa" +
				"}\n\n\n\n" +
				"function b() {}" );
		p.parse( sr, null );
	}
	
}
