package com.japisoft.editix.editor.json.helper;

import java.util.List;

import javax.swing.text.BadLocationException;
import javax.swing.text.Element;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.handler.AbstractHelperHandler;
import com.japisoft.xmlpad.helper.model.BasicDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class KeyValueHandler extends AbstractHelperHandler {
	
	private boolean caseSensitive = true;
	private PatternsGroup pg = null;
	
	public KeyValueHandler() {
		pg = new PatternsGroup( caseSensitive );
		pg.addPattern( "\\\"([^\\\"]+)\\\"\\s*:", "value" );
	}
	
	@Override
	protected void installDescriptors(
			FPNode currentNode,
			XMLPadDocument document, 
			int offset, 
			String activatorString ) {
		String[] groups = {
			"value"
		};
		for ( String group : groups ) {
			List<String> r = pg.getResult( group );
			if ( r != null ) {
				for ( String f : r ) {
					if ( "function".equals( group ) ) {
						f = f + "()";
					}
					addDescriptor( new BasicDescriptor( f ) );
				}
			}
		}
		pg.reset();
	}

	public boolean haveDescriptors(
			FPNode currentNode,
			XMLPadDocument document, 
			boolean insertBefore, 
			int offset, 
			String activatorString ) {
		if ( activatorString == null ) {
			int line = document.getDefaultRootElement().getElementIndex( offset );
			for ( int i = line - 1; i >= 0; i-- ) {
				try {
					Element le = document.getDefaultRootElement().getElement(i);
					String content = document.getText( le.getStartOffset(), le.getEndOffset() - le.getStartOffset() );
					pg.match( content );
				} catch( BadLocationException exc ) {}
			}
			return pg.hasResult();
		} else
			return false;
	}

}
