package com.japisoft.editix.editor.json.helper;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class PatternsGroup {

	private List<Object> patterns;
	private boolean caseSensitive;
	
	public PatternsGroup( boolean caseSensitive ) {
		this.caseSensitive = caseSensitive;
	}

	public void addPattern( String regexp, String name ) {
		if ( patterns == null ) {
			patterns = new ArrayList<Object>();
		}
		patterns.add( Pattern.compile( regexp, !caseSensitive ? Pattern.CASE_INSENSITIVE : 0 ) );
		patterns.add( name );
	}

	private Map<String,List<String>> res = null;

	public void match( String sequence ) {
		if ( patterns == null ) {
			return;
		}
		for ( int i = 0; i < patterns.size(); i += 2 ) {
			Pattern p = ( Pattern )patterns.get( i );
			String name = ( String )patterns.get( i + 1 );
			Matcher m = p.matcher( sequence );
			while ( m.find() ) {
				if ( res == null ) {
					res = new HashMap<String, List<String>>();
				}
				List<String> l = res.get( name );
				if ( l == null ) {
					res.put( name, l = new ArrayList<String>() );
				}
				for ( int j = 1; j <= m.groupCount(); j++ ) {
					l.add( m.group( j ) );
				}
			}
		}
	}
	
	public boolean hasResult() { return res != null; }
	
	public List<String> getResult( String name ) {
		if ( res == null ) {
			return null;
		}
		List<String> l = res.get( name );
		if ( l != null )
			Collections.sort( l );
		return l;
	}

	public void reset() { res = null; }
	
}
