package com.japisoft.editix.editor.xsd.view.designer.container;

import java.awt.Dimension;
import java.awt.Graphics;

import org.w3c.dom.Element;

import com.japisoft.editix.editor.xsd.view.designer.LeftToRightLayout;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XSDGroupComponentImpl extends XSDContainerComponentImpl {

	public XSDGroupComponentImpl() {
		setLayout( new LeftToRightLayout() );
	}

	private boolean foundRef = false;
	private Dimension refPreferredSize = null;
	private String lastRef;

	public void setElement(Element e) {
		super.setElement(e);
		resetSize( false );
	}	
	
	private void resetSize( boolean invalidateMode ) {
		if ( !foundRef ) {
			if ( e.hasAttribute( "ref" ) ) {
				foundRef = true;
				paintName = true;
				paintElementName = false;
				lastRef = e.getAttribute( "ref" );
				// Reduce the size
				refPreferredSize = getNameSize();
				if ( invalidateMode ) {
					invalidate();
					getParent().validate();
				}
			}
		} else {
			// The ref attribute has been remove
			if ( !e.hasAttribute( "ref" ) ) {
				foundRef = false;
				paintName = false;
				paintElementName = true;
				refPreferredSize = null;
				if ( invalidateMode ) {
					invalidate();
					getParent().invalidate();
					getParent().getParent().validate();
				}
			} else 
				if ( lastRef != null ) {
					if ( !lastRef.equals( e.getAttribute( "ref" ) ) ) {
						lastRef = e.getAttribute( "href" );
						refPreferredSize = getNameSize();
						if ( invalidateMode ) {
							invalidate();
							getParent().validate();
						}
					}
				}
		}
	}
	
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		resetSize( true );
	}

	public Dimension getPreferredSize() {
		if ( refPreferredSize != null )
			return refPreferredSize;
		return super.getPreferredSize();
	}	

}
