package com.japisoft.editix.filedragging;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.FileDragging;
import com.japisoft.xmlpad.editor.XMLEditor;
import com.japisoft.xmlpad.editor.XMLPadDocument;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DefaultFileDragging implements FileDragging {

	protected String getFileExt( File f ) {
		String path = f.getPath();
		int i = path.lastIndexOf( "." );
		if ( i > -1 ) {
			return path.substring( i + 1 ).toLowerCase();
		}
		return null;
	}
	
	private void dragXSD( XMLEditor editor, File f, boolean sameDocumentPath ) {
		// Bind the schema to the document
		FPNode rootNode = editor.getXMLContainer().getRootNode();
		XMLPadDocument doc = editor.getXMLDocument();
		if ( rootNode != null ) {
			rootNode.removeNameSpaceDeclaration( "xsi" );
			rootNode.addNameSpaceDeclaration( "xsi", "http://www.w3.org/2001/XMLSchema-instance" );  

			rootNode.setAttribute( "xsi:noNamespaceSchemaLocation", null );
			rootNode.setAttribute( "xsi:schemaLocation", null );			

			String xsdPath = f.toString();
			if ( sameDocumentPath ) {
				xsdPath = f.getName();
			}

			if ( rootNode.getNameSpaceURI() == null )
				rootNode.setAttribute( "xsi:noNamespaceSchemaLocation", xsdPath );
			else 
				rootNode.setAttribute( "xsi:schemaLocation", rootNode.getNameSpaceURI() + " " + xsdPath );
			
			doc.updateNodeOpeningClosing( rootNode );
			doc.parseSchema();					

		} else {
			EditixFactory.buildAndShowWarningDialog( "No document root for assigning your XSD Schema ?" );
		}		
	}

	protected boolean sameParent( File f, XMLEditor editor ) {
		String docPath = editor.getXMLContainer().getCurrentDocumentLocation();
		if ( docPath == null ) {
			return false;
		} else
			return f.getParentFile().equals( new File( docPath ).getParentFile() );
	}
	
	@Override
	public void drag( XMLEditor editor, List<File> files ) {
		boolean first = true;
		for ( File f : files ) {
			if ( !first ) {
				editor.insertText( " " );
			}

			if ( "xsd".equals( getFileExt( f ) ) ) {
				dragXSD( editor, f, sameParent( f, editor ) );
			} else {
				editor.insertText( f.toString() );
				first = false;
			}
			
		}
	}
	
	protected void drag( XMLEditor editor, File f ) {
		List<File> l = new ArrayList<File>();
		l.add( f );
		drag( editor, l );
	}
	
}
