package com.japisoft.editix.main;

import java.io.File;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.List;

import javax.swing.Action;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import com.apple.eawt.AboutHandler;
import com.apple.eawt.Application;
import com.apple.eawt.OpenFilesHandler;
import com.apple.eawt.PreferencesHandler;
import com.apple.eawt.QuitHandler;
import com.apple.eawt.QuitResponse;
import com.apple.eawt.AppEvent.AboutEvent;
import com.apple.eawt.AppEvent.OpenFilesEvent;
import com.apple.eawt.AppEvent.PreferencesEvent;
import com.apple.eawt.AppEvent.QuitEvent;
import com.japisoft.editix.action.file.OpenAction;
import com.japisoft.editix.action.file.project.RecentProjectManager;

import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.panels.PanelManager;
import com.japisoft.framework.ApplicationMain;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.ApplicationModel.ApplicationModelListener;
import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.framework.preferences.Preferences;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class Main extends ApplicationMain implements ApplicationModelListener {

	public void fireApplicationData(String key, Object... values) {
		if ( "lastProject".equals( key ) ) {
			String project = ( String )values[ 0 ];
			// Store the last opened project
			Preferences.setPreference( 
				Preferences.SYSTEM_GP, 
				"lastProject",
				project 
			);
			if ( project != null ) {
				RecentProjectManager.getInstance().addProject( new File( project ) );
			}
		} else
		if ( "windowOpened".equals( key ) ) {
			String lastProject = Preferences.getPreference(
				Preferences.SYSTEM_GP, 
				"lastProject",
				( String )null 
			);
			if ( lastProject != null ) {				
				ApplicationModel.fireApplicationValue(
					"lastProject",
					lastProject
				);				
			}
		} else
		if ( "windowClosing".equals( key ) ) {
			Action a = ( Action )com.japisoft.framework.application.descriptor.ActionModel.restoreAction( "quit" );
			a.actionPerformed( null );			
		} else
		if ( "quit".equals( key ) ) {
			try {
				EditixFrame.THIS.getBuilder().save();
			} catch ( Throwable e1 ) {
				System.err.println( "Error, can't save recent files menu ? : " + e1.getMessage() );
				ApplicationModel.debug( e1 );
			}
			Preferences.setRawPreference(
				"dialog",
				"editix",
				EditixFrame.THIS.getBounds() 
			);
			PanelManager.saveState( true );
			Preferences.savePreferences();
			System.exit( 0 );
		}
	}
	

	
	public static void main( String[] args ) throws Throwable {
		
		try {

			new EditixApplicationModel();

			if ( EditixApplicationModel.isMacOSXPlatform() ) {
				try {
					Class.forName( "MacOSXMain" ).newInstance();
				} catch( Throwable th ) {
					th.printStackTrace();
				}				
			}

			start( args );

		} catch ( Throwable th ) {
			th.printStackTrace();
			StringWriter sw = new StringWriter();
			th.printStackTrace(new PrintWriter(sw));

			JFrame fr = new JFrame( "Unknown error !" );
			StringBuffer sb = new StringBuffer();
			sb.append("EditiX " + EditixApplicationModel.getAppVersion()
					+ " - build " + EditixApplicationModel.BUILD);
			sb.append("\nHas met an unknown error while starting, please send it to : "
					+ EditixApplicationModel.MAIN_SUPPORT_EMAIL + "\n");
			sb
					.append("specifying your operating system version and java version\n");
			sb
					.append("\n----------------------------------------------------------------\n\n");
			sb.append(sw.toString());
			JTextArea ar = new JTextArea(sb.toString());
			fr.getContentPane().add(new JScrollPane(ar));
			fr.pack();
			fr.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			fr.setVisible(true);
		}

	}
}
