package com.japisoft.editix.main.steps;

import java.io.File;

import org.apache.xerces.impl.Version;

import com.japisoft.editix.document.DocumentModel;
import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.editix.main.Main;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.update.UpdateManager;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.ApplicationStep;
import com.japisoft.framework.dialog.about.AboutPanel;
import com.japisoft.framework.dialog.console.ConsolePanel;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.p3.Manager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ConfigurationApplicationStep implements ApplicationStep {

	static {
		EditixApplicationModel.SHORT_APPNAME = "editix";
		EditixApplicationModel.LONG_APPNAME = "Editix XML Editor";
		EditixApplicationModel.BUILD = "010123";
		EditixApplicationModel.INNER_BUILD = "010123";
		EditixApplicationModel.BETA_VERSION = 0;
		EditixApplicationModel.MAJOR_VERSION = 20;
		
		Preferences.PREF_FILENAME = "pref" + EditixApplicationModel.MAJOR_VERSION + ".prop";		
		
		EditixApplicationModel.MAJOR_YEAR = 2023;
		EditixApplicationModel.SERVICE_PACK = 0;
		EditixApplicationModel.MINOR_VERSION = 0;
		EditixApplicationModel.SUBMINOR_VERSION = 0;
		EditixApplicationModel.MAIN_SUPPORT_EMAIL = "editixsupport@japisoft.com";
		EditixApplicationModel.REGISTERED_FILE = "editix" + EditixApplicationModel.MAJOR_VERSION + ".reg";
		EditixApplicationModel.REGISTERED_FILE2 = "editix" + ( EditixApplicationModel.MAJOR_VERSION - 1 ) + ".reg";
		
		EditixApplicationModel.DEF_MANUAL_PATH = "http://www.editix.com/doc/manual" + ( EditixApplicationModel.MAJOR_VERSION + 1 ) + "/index.html";
		EditixApplicationModel.PURCHASING_URL = "http://www.editix.com/buy.html";
		
		EditixApplicationModel.PREFERENCES_SUBMENU =
			Main.class.getResource( 
				"prefnode.txt" 
		);
		
		EditixApplicationModel.setSharedProperty( "defaultFont", "consolas" );

		Manager.PERS_SIGNATURE = "pers" + EditixApplicationModel.MAJOR_VERSION;
		Manager.PROF_SIGNATURE = "prof" + EditixApplicationModel.MAJOR_VERSION;
		Manager.STUD_SIGNATURE = "stud" + EditixApplicationModel.MAJOR_VERSION;
		Manager.ENT_SIGNATURE = "ent" + EditixApplicationModel.MAJOR_VERSION;
		Manager.FLOAT_SIGNATURE = "float" + EditixApplicationModel.MAJOR_VERSION;
		Manager.NONCOMMERCIAL_SIGNATURE = "noncom" + EditixApplicationModel.MAJOR_VERSION;
		

		Manager.PERS_SIGNATURE2 = "pers" + ( EditixApplicationModel.MAJOR_VERSION - 1 );
		Manager.PROF_SIGNATURE2 = "prof" + ( EditixApplicationModel.MAJOR_VERSION - 1 );
		Manager.STUD_SIGNATURE2 = "stud" + ( EditixApplicationModel.MAJOR_VERSION - 1 );
		Manager.ENT_SIGNATURE2 = "ent" + ( EditixApplicationModel.MAJOR_VERSION - 1 );
		Manager.ENT_SIGNATURE2 = "float" + ( EditixApplicationModel.MAJOR_VERSION - 1 );
		
		//%%%%%
		//Manager.setFree();

		Manager.MAGIC_NUMBER_1 = 19;
		Manager.MAGIC_NUMBER_2 = 11;
		
		Manager.CURRENT_PRO_FILE1 = ".sysedb53";
		Manager.CURRENT_PRO_FILE2 = ".sysedb54";
		
		Manager.PREVIOUS_FILES = new String[] {
				".sysedb51", ".sysedb52",				
				".sysedb49", ".sysedb50",
				".sysedb47", ".sysedb48",
				".sysedb45", ".sysedb46",
				".sysedb44", ".sysedb43",
				".sysedb42", ".sysedb41",
				".sysedb40", ".sysedb39",
				".sysedb37", ".sysedb38",
				".sysedb35", ".sysedb36",
				".sysedb33", ".sysedb34",
				".sysedb31", ".sysedb32",
				".sysedb29", ".sysedb30",
				".sysedb27", ".sysedb28",
				".sysedb21", ".sysedb22",
				".sysedb19", ".sysedb20"
		};
		
		Manager.PREVIOUS_INC = 25;

		try {
			Preferences.loadPreferences();
		} catch( Throwable th ) {
			System.err.println( "Can't load preferences :" + th.getMessage() );
		}

		if ( "true".equals( System.getProperty( "editix.debug" ) ) )
			System.out.println( "DEBUG MODE" );

	}

	// For line command

	public static File startingFile = null;
	public static File startingProject = null;

	public static File getRegisteredPath() {
		File home = new File(System.getProperty("user.home"));
		home = new File(home, ".editix");
		if (!home.exists()) {
			boolean rs = home.mkdirs();
			if (!rs) {
				return null;
			}
		}
		return home;
	}
	
	public boolean isFinal() {
		return false;
	}

	@Override
	public void quit() {
	}
	
	public void start(String[] args) {
		
		ApplicationModel.addApplicationModelListener( new Main() );
		
		if (args.length > 0) {
			if (args[0].endsWith(".pre"))
				startingProject = new File(args[0]);
			else
				startingFile = new File(args[0]);
		}

		new UpdateManager();
		
		// Anti aliasing
		if ( Preferences.getPreference( "interface", "antialiasing", true ) ) {
			System.setProperty("awt.useSystemAAFontSettings","on");
			System.setProperty( "swing.aatext", "true" );
			ApplicationModel.debug( "Setting anti aliasing on" );
		}

		EditixApplicationModel.init( args );
		EditixApplicationModel.starting();

		// Disable logs for FOP
		System.setProperty( 
				"org.apache.commons.logging.Log", 
				EmptyApacheLogs.class.getName() 
		);
		
		EditixFrame frame = null;

		try {
			AboutPanel.addAboutProperty( "XERCES VERSION", Version.getVersion() );
			AboutPanel.addAboutProperty( "FOP VERSION", org.apache.fop.Version.getVersion() );
			AboutPanel.addAboutProperty( "SAXON VERSION", net.sf.saxon.Version.getProductVersion() );
			AboutPanel.addAboutProperty( "SYSTEM FILE ENCODING", System.getProperty("file.encoding") );
			if ( EditixApplicationModel.INNER_BUILD != null )
					AboutPanel.addAboutProperty( "inner build", EditixApplicationModel.INNER_BUILD );	
		} catch (RuntimeException e1) {
			e1.printStackTrace();
		}

		if ( "on".equals( System.getProperty( "uiconsole", "on" ) ) )
			ConsolePanel.initConsoleState();
		
		// EditixApplicationModel.ACCESSOR.checkHck();

		String proxyHost = Preferences.getPreference("proxy", "host", "");
		int proxyPort = Preferences.getPreference("proxy", "port", 0);

		if (proxyHost.length() > 0 && proxyPort != 0) {
			System.setProperty("proxyHost", proxyHost);
			System.setProperty("proxyPort", "" + proxyPort);
			System.setProperty("proxySet", "true");
			System.setProperty("http.proxyHost", proxyHost);
			System.setProperty("http.proxyPort", "" + proxyPort);
			System.setProperty("http.proxySet", "true");
		}
		
		DocumentModel.class.getName();

		
	}

	public void stop() {
	}

}
