package com.japisoft.editix.main.steps;

import java.io.File;
import java.net.URL;

import com.japisoft.editix.document.DocumentModel;
import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.editix.ui.EditixActionBuilder;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.ApplicationStep;
import com.japisoft.framework.ApplicationModel.ApplicationModelListener;
import com.japisoft.framework.application.descriptor.InterfaceBuilder;
import com.japisoft.framework.application.descriptor.helpers.InterfaceBuilderFactory;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.dialog.actions.StoringLocationAction;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class InterfaceBuilderApplicationStep implements ApplicationStep {
	
	public boolean isFinal() {
		// TODO Auto-generated method stub
		return false;
	}

	public void start(String[] args) throws Exception {

		// Interface definition
		URL url = ClassLoader.getSystemResource("editix.xml");
		if ( url == null )
			url = ClassLoader.getSystemResource("com/japisoft/editix/descriptor/editix.xml");
		if (url == null) {
			// Search from the working directory
			File f = new File("res/editix.xml");
			if (f.exists())
				url = f.toURL();
			else
				throw new RuntimeException( "Can't find res/editix.xml" );
		}
		
		if ( url == null )
			throw new RuntimeException(
					"Can't find editix.xml or res/editix.xml" );
		
		// Menu building
	
		InterfaceBuilderFactory.setActionBuilder( new EditixActionBuilder() );
		
		InterfaceBuilder builder = null;
	
		URL tmpDescriptorUrl = url;
	
		if ( EditixApplicationModel.getCustomEditiXDescriptor().exists() ) {
			tmpDescriptorUrl = EditixApplicationModel.getCustomEditiXDescriptor().toURL();
		}
	
		try {
			builder = new InterfaceBuilder( tmpDescriptorUrl, "com/japisoft/editix/descriptor/editix.xml" );
			builder.setEnabledActionForAllGroup( false );
			ApplicationModel.addApplicationModelListener( ( ApplicationModelListener )builder.getActionById( "open" ) );

		} catch( Exception exc ) {
			ApplicationModel.debug( exc );
			EditixFactory.buildAndShowErrorDialog( "Can't use the main descriptor [" + tmpDescriptorUrl + ", " + exc.getMessage() + "],\nTry the load the default one..." );
			builder = new InterfaceBuilder( url, "com/japisoft/editix/descriptor/editix.xml" );
		}
		
		EditixApplicationModel.INTERFACE_BUILDER = builder;
		
	}
	
	public void stop() {
		// TODO Auto-generated method stub	
	}
	
	@Override
	public void quit() {
	}

}
