package com.japisoft.editix.main.steps;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Toolkit;

import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.EditixInitPanel;
import com.japisoft.framework.ApplicationStep;
import com.japisoft.framework.preferences.Preferences;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class MainFrameApplicationStep implements ApplicationStep {

	public boolean isFinal() {
		return false;
	}

	public void start(String[] args) {
		EditixFrame frame = new EditixFrame( EditixApplicationModel.INTERFACE_BUILDER );

		//////////////

		Rectangle r = Preferences.getPreference("dialog", "editix",
				new Rectangle(0, 0, 0, 0));
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();				
		
		if ( !((r.x == r.y) && (r.width == r.height) && (r.x == 0)) ) {

			if ( Preferences.getPreference( 
					"interface", 
						"windowLocationChecked", true ) ) {					
				if ( r.x < 0 )
					r.x = 0;
				if ( r.y < 0 )
					r.y = 0;
				if ( r.x >= dim.width )
					r.x = 0;
				if ( r.y >= dim.height )
					r.y = 0;
				if ( r.width < 1 )
					r.width = dim.width;
				if ( r.height < 1 )
					r.height = dim.height;
			}

			frame.setBounds( r );
		}
		else {
			frame.setLocation(0, 0);

			frame.setSize(dim.width, dim.height - 30);
		}

		EditixApplicationModel.MAIN_FRAME = frame;

		if ( Preferences.getPreference( 
				"system", 
				"browserenabled", 
				false ) ) {
			String lastBrowse = Preferences.getPreference( 
					"system", 
					"browserlastfile", 
					( String )null );				
		}

		if ( Preferences.getPreference("interface", "initialDocument", true ) ) {
			if ( !frame.hasContainer() ) {
				if ( !EditixApplicationModel.isMacOSXPlatform() ) {
					EditixFrame.THIS.addContainer( new EditixInitPanel() );
					EditixFrame.THIS.invalidate();
					EditixFrame.THIS.validate();
					EditixFrame.THIS.repaint();					
					EditixFrame.THIS.selectDefaultContainer();
				}
			}
		}

		if (Preferences.getPreference("interface", "tipOfTheDay", false)) {
			com.japisoft.framework.application.descriptor.ActionModel.activeActionById(
					"tipOfTheDay", null);
		}

		frame.setVisible( true );
		// Splashscreen.restart();
		
	}

	public void stop() {
	}
	
	@Override
	public void quit() {
	}

}
