package com.japisoft.editix.main.steps;

import javax.swing.JOptionPane;

import com.japisoft.dtdparser.DTDMapperFactory;
import com.japisoft.editix.action.xml.ParseAction;
import com.japisoft.editix.toolkit.EditiXSAXParserFactory;
import com.japisoft.framework.ApplicationStep;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.framework.xml.XMLParser;
import com.japisoft.xmlpad.SharedProperties;
import com.japisoft.xmlpad.action.ActionModel;
import com.japisoft.xmlpad.action.xml.FormatAction;
import com.japisoft.xmlpad.xml.validator.DefaultValidator;
import com.japisoft.xmlpad.xml.validator.XMLPadSAXParserFactory;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XMLApplicationStep implements ApplicationStep {

	public boolean isFinal() {
		return false;
	}

	public void start( String[] args ) {
		// Format preference

		ActionModel.setProperty(FormatAction.class.getName(),
				FormatAction.INDENT_CHAR_PROPERTY, "\t");

		ActionModel.setProperty(FormatAction.class.getName(),
				FormatAction.INDENT_SIZE_PROPERTY, new Integer(
						Preferences.getPreference("xmlconfig",
								"format-space", 1)));

		SharedProperties.EDITOR_LINE_NUMBER =
			Preferences.getPreference( "editor", "line-number", true );

		FormatAction.CANONICAL = Preferences.getPreference("xmlconfig",
				"format-canonical", false);

		FormatAction fa = 
			( FormatAction )ActionModel.getActionByName( ActionModel.FORMAT_ACTION );

		fa.setReplaceAmp( Preferences.getPreference( "xmlconfig", "format-replaceAmp", true ) );
		fa.setReplaceGt( Preferences.getPreference( "xmlconfig", "format-replaceGt", true ) );
		fa.setReplaceLt( Preferences.getPreference( "xmlconfig", "format-replaceLt", true ) );
		fa.setReplaceAPos( Preferences.getPreference( "xmlconfig", "format-replaceAPos", true ) );
		fa.setReplaceQuote( Preferences.getPreference( "xmlconfig", "format-replaceQuote", true ) );

		//////////// XML

		XMLParser.resetDefaultParser();

		XMLPadSAXParserFactory
				.setDelegate( new EditiXSAXParserFactory() );

		try {
			ParseAction.setDefaultValidator( 
				( DefaultValidator )Class.forName( "com.japisoft.editix.action.xml.EditixValidator" ).newInstance() 
			);
		} catch (InstantiationException e2) {
		} catch (IllegalAccessException e2) {
		} catch (ClassNotFoundException e2) {
		}

		DTDMapperFactory.setDTDMapper( EditixDTDMapper.getInstance() );
		SharedProperties.SCHEMA_CACHING = true; // For sharing multiple content
		
		try {
			SharedProperties.DEFAULT_ENTITY_RESOLVER = EditixEntityResolver
					.getInstance();
		} catch ( Throwable th ) {
			JOptionPane
					.showMessageDialog(
							null,
							"Error found inside the XML catalog files,\nPlease fix it using the XML Menu"
			);
		}
	}

	public void stop() {
	}
	
	@Override
	public void quit() {
	}

}
