package com.japisoft.editix.mapper.xsd;

import com.japisoft.editix.mapper.AbstractMapper;
import com.japisoft.framework.xml.parser.node.FPNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TypeMapper extends AbstractMapper {

	public boolean canMap( FPNode node ) {
		return "element".equals( node.getContent() ) && 
					node.hasAttribute( getMapAttribute() );
	}

	@Override
	protected boolean isMatchingNode(
			FPNode sourceNode,
			FPNode walkingNode ) {
		boolean typeDef = 
			( "complexType".equals( walkingNode.getContent() ) || 
				"simpleType".equals( walkingNode.getContent() ) ) && walkingNode.hasAttribute( "name" );
		if ( !typeDef ) {
			return false;
		} else {
			String type = sourceNode.getAttribute( "type" );
			String name = walkingNode.getAttribute( "name" );
			int i = type.lastIndexOf( ":" );
			if ( i > -1 ) {
				type = type.substring( i );
			}
			return name.equals( type );
		}
	}

	public String getMapAttribute() {
		return "type";
	}	
	
	@Override
	public String toString() {
		return "Find type definition";
	}
}
