package com.japisoft.editix.plugin;

import java.awt.event.ActionEvent;
import java.util.Iterator;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.application.descriptor.ActionModel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class EditiXManager {

	private EditiXManager() {	
	}

	private static EditiXManager INSTANCE = null;
	
	/** @return the current singleton for accessing to the editix manager */
	public static EditiXManager getInstance() {
		if ( INSTANCE == null ) {
			INSTANCE = new EditiXManager();
		}
		return INSTANCE;
	}

	/**
	 * Active an item action from the editix descriptor
	 * @param actionId A unique action id */
	public void activeAction( String actionId ) {
		ActionModel.activeActionById( 
			actionId, 
			new ActionEvent( actionId, 0, "" ) 
		);
	}

	/** @return available actions id */
	public Iterator<String> getActions() {
		return ActionModel.getActionsName();
	}

	/**
	 * Display a dialog
	 * @param message Simple dialog message */
	public void info( String message ) {
		EditixFactory.buildAndShowInformationDialog( message );
	}

	/** Show a dialog and return the user input */
	public String prompt( String title, String defaultMessage ) {
		return EditixFactory.buildAndShowInputDialog(title, defaultMessage );
	}
	
	private EditixDocumentModel model = null;

	/**
	 * @return An access to all the opened document */
	public EditixDocumentModel getDocumentModel() {
		if ( model == null )
			model = new EditixDocumentModel();
		return model;
	}

	/**
	 * @return the current editor document */
	public EditixDocument getCurrentDocument() {
		return getDocumentModel().getCurrentDocument();
	}

	/**
	 * Create and add a new document
	 * @param type Document Type : XML, DTD, XSD, CSS...
	 * @return a new document */
	public EditixDocument newDocument( String type ) {
		return getDocumentModel().newDocument( type );
	}

	/**
	 * Close the current editor */
	public void closeCurrentDocument() {
		EditixFrame.THIS.closeCurrentContainer();
	}

}
