package com.japisoft.editix.script;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileNotFoundException;

import javax.script.Bindings;
import javax.script.ScriptContext;
import javax.script.ScriptEngine;
import javax.script.ScriptException;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.KeyStroke;

import com.japisoft.editix.editor.jsx.action.NodeFactory;
import com.japisoft.editix.editor.jsx.domapi.Document;
import com.japisoft.editix.javascript.JavaScriptFactory;
import com.japisoft.editix.plugin.EditiXManager;
import com.japisoft.editix.toolkit.Toolkit;
import com.japisoft.editix.ui.EditixFactory;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ScriptAction extends AbstractAction {

	private File path = null;
	
	public ScriptAction( Script s ) {
		putValue( Action.NAME, s.getName() );
		if ( !"".equals( s.getShortkey() ) );
		putValue( Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke( s.getShortkey() ) );
		this.path = s.getPath();
	}

	private boolean jsxMode = false;
	
	public ScriptAction( File path, boolean jsxMode ) {
		this.path = path;
		this.jsxMode = jsxMode;
	}
	
	public ScriptAction( File path ) {
		this( path, false );
	}
	
	public boolean errorFound = false;
	
	public void actionPerformed(ActionEvent e) {
		errorFound = false;
		if ( !path.exists() ) {
			EditixFactory.buildAndShowErrorDialog( "Can't find this script" );
		} else {
	        ScriptEngine engine = JavaScriptFactory.newFactory().engine();

	        try {
	        	Bindings b = engine.getBindings(ScriptContext.GLOBAL_SCOPE);
	        	b.put( "EditiXManager", EditiXManager.getInstance());
	        	b.put( "EditixManager", EditiXManager.getInstance());
	        	
	        	if( jsxMode ) {
	        		try {
	        			if ( EditiXManager.getInstance().getCurrentDocument() == null ) {
	        				EditixFactory.buildAndShowErrorDialog( "Need an XML document for running this script" );
	        				return;
	        			}
	        			String currentDocument = EditiXManager.getInstance().getCurrentDocument().getLocation();
	        			if ( currentDocument != null ) {
	        				Document doc = NodeFactory.newInstance().getDocument( currentDocument );
	        				b.put( "document", doc );
	        			} else
	        				EditixFactory.buildAndShowErrorDialog( "You must save your document before running the script" );
	    			} catch( Exception exc ) {
	    				EditixFactory.buildAndShowErrorDialog( "Can't parse this data file ?" );
	    				return;
	    			}
	        	}
	        	
	        	engine.eval(new java.io.FileReader( path ));
	        } catch( ScriptException exc ) {
	        	EditixFactory.buildAndShowErrorDialog( "Error(s) in your script : line " + exc.getLineNumber() );
	        	errorFound = true;
	        } catch( FileNotFoundException fne ) {
	        }
		}
	}

}
