package com.japisoft.editix.toolkit;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;
import java.net.JarURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import javax.script.Bindings;
import javax.script.ScriptContext;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import javax.swing.SwingUtilities;

import com.japisoft.editix.ui.EditixStatusBar;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.framework.xml.Encoding;
import com.japisoft.framework.xml.XMLFileData;
import com.japisoft.framework.xml.XMLToolkit;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class Toolkit {

	// Available file charset encoding
	static String[] FILE_ENCODING = Encoding.XML_ENCODINGS;

	public static String getCurrentFileEncoding() {
		// Choose the charset encoding
		String[] encoding = 
			encoding = Preferences.getPreference(
					"file", "rw-encoding",
					FILE_ENCODING );
		return encoding[ 0 ];
	}

	public static Reader getReaderForFile( File fileName ) throws IOException {
		String encoding = getCurrentFileEncoding();
		Reader rr;
		if ( "DEFAULT".equals( encoding ) || "AUTOMATIC".equals( encoding ) )
			rr = new FileReader( fileName );
		else
			rr = new InputStreamReader( new FileInputStream( fileName ), encoding );
		return rr;
	}

	public static Reader getReaderForFile( String fileName ) throws IOException {
		return getReaderForFile( new File( fileName ) );
	}
	
	public static Writer getWriterForFile( File fileName ) throws IOException {
		String encoding = getCurrentFileEncoding();
		Writer w = null;
		if ( "DEFAULT".equals( encoding ) )
			w = new FileWriter( fileName );
		else
			w = new OutputStreamWriter( 
				new FileOutputStream( fileName ), encoding );
		return w;
	}

	public static XMLFileData getEncodedString( byte[] data ) throws Throwable {
		String encoding = getCurrentFileEncoding();
		XMLFileData r = XMLToolkit.getContentFromInputStream(
				new ByteArrayInputStream( data ),
				Toolkit.getCurrentFileEncoding() );
		return r;
	}
	
	public static String getContentFromInputStream(InputStream input)
		throws Throwable {
		StringBuffer sb = new StringBuffer();
		char[] buffer = new char[ 1024 ];
		try {
			int c;

			Reader r = null;
			String encoding = getCurrentFileEncoding();
			if ( "DEFAULT".equals( encoding ) )
				r = new InputStreamReader( input );
			else
				r = new InputStreamReader( input, encoding );

			while ((c = r.read( buffer )) != -1) {
				sb.append( new String( buffer,0 , c ) );
			}
		} finally {
			input.close();
		}
		return sb.toString();
	}

	/** Retreive an initial resource location using this resource name
	 */
	public static String getPathForObject(String resource) {
		URL url = ClassLoader.getSystemResource(resource);
		if (url == null)
			return null;
		if (url.getProtocol().equals("jar")) {
			try {
				JarURLConnection jarCon =
					(JarURLConnection) url.openConnection();
				url = jarCon.getJarFileURL();
				String _ = url.toExternalForm();
				_ = _.replaceAll( "%20", " " );
				if (_.startsWith("file://"))
					return _.substring(6);
				else if (_.startsWith("file:/"))
					return _.substring(5);
				return _;
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return url.toExternalForm();
	}

	public static String toURL( File f ) throws MalformedURLException {
		String res = f.toURI().toURL().toString();
		res = res.replace( " ", "%20" );
		return res;
	}
	
	public static void startJob( String name, Runnable run ) {
		EditixStatusBar.ACCESSOR.setMessage( name );
		SwingUtilities.invokeLater( run );
	}
	
	public static void stopJob() {
		EditixStatusBar.ACCESSOR.setXPathLocation( "Completed" );
	}

	public static String toString( URL u ) {
		return u.toExternalForm().replace( " ", "%20" );
	}
	
	
}
