package com.japisoft.editix.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.HeadlessException;
import java.io.File;

import javax.swing.Icon;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileView;

import com.japisoft.editix.document.DocumentModel;
import com.japisoft.xmlpad.XMLDocumentInfo;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DocumentFileChooser extends JFileChooser implements SelectableEncoding {

	private JComboBox combo;

	public DocumentFileChooser() {
		super();
		FileFilter first = null;
		// Create one filter by documentInfo
		for (int i = 0; i < DocumentModel.getDocumentCount(); i++) {
			XMLDocumentInfo info = DocumentModel.getDocumentAt(i);
			addChoosableFileFilter(info.getFileFilter());
			if (first == null)
				first = info.getFileFilter();
		}
		init();
		setFileFilter( first );
	}
	
	public DocumentFileChooser( XMLDocumentInfo doc ) {
		init();
		setFileFilter( doc.getFileFilter() );
	}

	private void init() {
		setFileSelectionMode( JFileChooser.FILES_ONLY );
		setMultiSelectionEnabled(true);
		setFileView( new DocumentFileView() );		
	}
	
	protected JDialog createDialog(Component parent) throws HeadlessException {
		JDialog dialog = super.createDialog( parent );
		JPanel panel = new JPanel();
		panel.setLayout( new BorderLayout() );
		panel.setBorder( new EmptyBorder(10, 10, 10, 10 ) );
		JLabel lbl = new JLabel( 
			"Select a file filter (XML, XSLT...) for your document type" 
		);
		lbl.setForeground( Color.DARK_GRAY );
		panel.add( lbl, BorderLayout.CENTER );
		JPanel encoPanel = new JPanel();
		encoPanel.setLayout( new BorderLayout() );
		combo = EncodingMenuBuilder.encodingComboBox();
		encoPanel.add( new JLabel( "Encoding :" ), BorderLayout.CENTER );
		encoPanel.add( combo, BorderLayout.SOUTH );
		panel.add( encoPanel, BorderLayout.SOUTH );
		dialog.getContentPane().add( BorderLayout.SOUTH, panel );
		return dialog;
	}

	public int showDialog( Component parent, String approveButtonText ) throws HeadlessException {
		return super.showDialog( parent, approveButtonText );
	}
	
	public String getSelectedEncoding() {
		return ( String )combo.getSelectedItem();
	}
	
	class DocumentFileView extends FileView {

		@Override
		public Icon getIcon( File f ) {
			if ( f.isFile() ) {
				String name = f.getName();
				int _i = name.lastIndexOf( "." );
				if ( _i > -1 ) {
					String ext = name.substring( _i + 1 );
					for ( int i = 0; i < DocumentModel.getDocumentCount(); i++ ) {
						if ( DocumentModel.getDocumentAt( i ).matchFileExt( ext ) ) {
							return DocumentModel.getDocumentAt( i ).getDocumentIcon();
						}
					}
				}
				return UIManager.getIcon("FileView.fileIcon");
			}
			return UIManager.getIcon("FileView.directoryIcon");
		}

	}

}
