package com.japisoft.editix.ui;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;

import javax.swing.Action;

import com.japisoft.editix.script.ScriptAction;
import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.application.descriptor.InterfaceBuilderException;
import com.japisoft.framework.application.descriptor.helpers.ActionBuilder;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.parser.walker.TreeWalker;
import com.japisoft.xmlpad.action.XMLAction;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class EditixActionBuilder implements ActionBuilder {

	public Action buildAction(FPNode source, String action)
			throws InterfaceBuilderException {

		// Check for action child
		TreeWalker tw = new TreeWalker( source );
		FPNode actionNode = tw.getFirstTagNodeByName( "action", false );
		if  ( actionNode == null )
			actionNode = source;

		boolean editor = "true".equals(
				actionNode.getAttribute( "editor" ) );

		Action a = null;

		// Search it from the XMLPad ActionModel
		if ( editor )
			a = ( Action ) com.japisoft.xmlpad.action.ActionModel
					.getActionByName( action );
		
		if ( a == null ) {
			try {
				
				ClassLoader loader = getClass().getClassLoader();

				if ( actionNode.hasAttribute( "libraries" ) ) {					
					String path = actionNode.getAttribute( "libraries" );

					boolean jsxMode = path.toLowerCase().endsWith( ".jsx" );
					
					if ( path.toLowerCase().endsWith( ".js" ) || jsxMode ) { 	// JavaScript code
						a = new ScriptAction( new File( path ), jsxMode );
					} else {	// Java code
						
						String[] pathTab = path.split( ";" );
						URL[] u = new URL[ pathTab.length ];
						int i = 0;
						for ( String pt : pathTab) {
							File f = new File( pt );
							if ( !( f.exists() ) ) {
								throw new InterfaceBuilderException( "Can't find the library [" + pt + "] ?" );
							}
							u[ i++ ] = f.toURI().toURL();
						}
						loader = new URLClassLoader( u, loader );

					}
				}

				if ( a == null ) {
				
					Class cl = null;
	
					if ( SOL2.equals( action ) )
						cl = AboutAction.class;
					else if ( SOL1.equals( action ) )
						cl = RA.class;
					else
						cl = loader.loadClass( action );
									
					a = ( Action ) cl.newInstance();
					if ( editor )
						com.japisoft.xmlpad.action.ActionModel
								.addActionForGroup(
										com.japisoft.xmlpad.action.ActionModel.TOOLKIT_GROUP,
										( XMLAction ) a );
					
				}

			} catch ( Throwable th ) {
				ApplicationModel.debug( th );
				throw new InterfaceBuilderException("Cannot build "
						+ th.getMessage(), th);
			}

		}

		String id = source.getFPParent().getAttribute( "id" );
		if ( id != null && a != null )
			ActionModel.storeAction( id, a );
		
		return a;
	}
	
	static String SOL1 = compute("IN", "1");
	static String SOL2 = compute("IN", "2");
	static String compute(String a, String b) {
		return a + "NER_" + b;
	}

}
