package com.japisoft.editix.ui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;

import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import com.japisoft.framework.dialog.BasicDialogComponent;
import com.japisoft.framework.dialog.DialogFooter;
import com.japisoft.framework.dialog.DialogHeader;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.dialog.actions.DialogActionModel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class EditixDialog extends BasicDialogComponent {

	protected DialogActionModel actionModel = null;
	private DialogFooter footer = null;

	public EditixDialog( 
			String dialogTitle, 
			String title,
			String comment,
			DialogActionModel model ) {
		super( EditixFrame.THIS, dialogTitle );
		
		this.actionModel = model;
		
		DialogHeader header = DialogManager.getDefaultDialogHeader();
		header.setComment( comment );
		header.setTitle( title );
		header.setIcon( DialogManager.getDefaultDialogIcon() );

		footer = DialogManager.getDefaultDialogFooter();
		if ( actionModel == null )
			actionModel = DialogManager.getDefaultDialogActionModel();
		footer.setModel( actionModel );
		
		footer.setDialogTarget( this );

		JPanel tmp = new JPanel();
		Dimension dim = getDefaultSize();
		if ( dim != null )
			tmp.setPreferredSize( dim );
		
		init( header, tmp, footer );
		pane = tmp;
		tmp.setLayout( new BorderLayout() );

	}

	public EditixDialog( 
			String dialogTitle, 
			String title,
			String comment ) {
		this( dialogTitle, title, comment, null );
	}

	protected Dimension getDefaultSize() { return null; }
	
	public boolean isOk() {
		return ( getLastAction() == DialogManager.OK_ID );
	}
	
	private JPanel pane = null;
	
	public Container getContentPane() {
		if ( pane == null ) {
			return super.getContentPane();
		} else
			return pane;
	}

	JToggleButton fixButton;

	private String getDialogName() {
		String _ = getClass().getName();
		int i = _.lastIndexOf( "." );
		if ( i > -1 )
			return _.substring( i + 1 );
		return _;
	}

	protected Border createContentBorder() {
		return new EmptyBorder( 4, 4, 4, 4 );
	}
	
}
