package com.japisoft.editix.ui;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.Timer;

import com.japisoft.editix.main.steps.EditiXLookAndFeel;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.ApplicationModel.ApplicationModelListener;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class MessagePanel extends JPanel 
	implements ApplicationModelListener, ActionListener {

	public MessagePanel() {
		ApplicationModel.addApplicationModelListener( this );
		setOpaque( false );
		setFont( getFont().deriveFont( Font.BOLD ) );
	}

	private String currentMessage = null;
	private Timer timer = null;
	
	public void fireApplicationData( 
			String key, Object... values ) {
		if ( "information".equals( key ) || "error".equals( key ) ) {

			currentMessage = ( String )values[ 0 ];
					
			if ( currentMessage == null )
				currentMessage = "[EMPTY Message]";
			
			if ( !EditixFactory.externalMessage() ) {
			
				if ( "information".equals( key ) ) {
					setForeground( EditiXLookAndFeel.INFORMATION_COLOR );
				} else
					setForeground( EditiXLookAndFeel.ERROR_COLOR );
				currentMessage = currentMessage.replace( '\n', '-' );
				visibleModeFlip = true;
				alpha = 0f;
				if ( timer != null ) {
					timer.stop();
				} else
					timer = new Timer( 20, this );
				timer.start();
			
			} else {
				
				if ( "information".equals( key ) ) {
					EditixFactory.buildAndShowInformationDialog( currentMessage );
				} else {
					EditixFactory.buildAndShowErrorDialog( currentMessage );
				}
				
			}
		}
	}

	public void actionPerformed(ActionEvent e) {
		repaint();
	}

	private float alpha = 0f;
	private boolean visibleModeFlip = true;
	
	public void paint( Graphics g ) {
		Graphics2D g2 = ( Graphics2D ) g.create();
		g2.setComposite( 
			AlphaComposite.getInstance( 
					AlphaComposite.SRC_OVER, Math.min( alpha, 1f ) ) 
		);
		if ( currentMessage != null ) {
			if ( visibleModeFlip ) {
				alpha += 0.01;
				if ( alpha > 1.8 ) {
					alpha = 1.0f;
					visibleModeFlip = false;
				}
			} else {
				alpha -= 0.02;
				if ( alpha < 0 ) {
					alpha = 0;
					currentMessage = null;
					timer.stop();
				}
			}
		} else {
			if ( alpha > 0 ) {
				alpha -= 0.02;
				if( alpha < 0 ) {
					alpha = 0;
					timer.stop();
				}
			}
		}
		super.paint( g2 );
		g2.dispose();
	}
	
	public void paintComponent( Graphics g ) {  
		super.paintComponent( g );
		if ( currentMessage != null ) {
			int width = g.getFontMetrics().stringWidth( currentMessage );
			g.setColor( getForeground() );
			int bottom = EditixStatusBar.ACCESSOR.getY() + EditixStatusBar.ACCESSOR.getHeight();
			int left = getWidth() - width - 100; 
			g.fillRoundRect(
				left,
				bottom,
				width + 50,
				g.getFontMetrics().getHeight() + 20,
				10,
				10
			);
			g.setColor( Color.WHITE );
			g.drawString(
				currentMessage,
				left + 25,
				bottom + ( g.getFontMetrics().getHeight() + 20 ) / 2 + g.getFontMetrics().getDescent()
			);
		}
	}

}
