package com.japisoft.editix.ui.container;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.stylededitor.EditorByCSS;
import com.japisoft.stylededitor.Factory;
import com.japisoft.stylededitor.LocationListener;
import com.japisoft.stylededitor.model.XMLDocument;
import com.japisoft.xmlpad.helper.HelperManager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class EditorByCSSFilterView implements
		Factory,
		FilterView, 
		ActionListener,
		LocationListener {

	private JPanel view = null; 
	private JCheckBox cbTagMode = null;
	
	public void init(
		HelperManager helper,
		String location, 
		Document xmlContent ) throws Exception {
		getEditor().setDocument( 
			new XMLDocument( location, xmlContent )
		);
		if ( cbTagMode != null ) {
			cbTagMode.addActionListener( this );
		}
	}

	public void dispose() {
		if ( cbTagMode != null ) {
			cbTagMode.removeActionListener( this );
		}
		getEditor().setLocationListener( null );
	}

	public void requestFocus() {
		getEditor().requestFocus();
	}
	
	public void setLocation( Node n ) {
		StringBuffer sb = new StringBuffer();
		while ( n != null ) {
			if ( n instanceof Element ) {
				sb.insert( 
					0, 
					"/" + n.getNodeName() 
				);
			}
			n = n.getParentNode();
		}
		String location = sb.toString();
		ApplicationModel.fireApplicationValue( "location", location );
	}

	public void copy() {
		editor.copy();
	}
	
	public void cut() {
		editor.cut();
	}

	public void paste() {
		editor.paste();
	}
	
	public String getName() {
		return "CSS";
	}

	private EditorByCSS editor = null;
	
	public JComponent getView() {
		if ( view == null ) {
			view = new JPanel();
			view.setLayout( 
				new BorderLayout() 
			);
			view.add( 
				BorderLayout.CENTER, 
				new JScrollPane( 
					getEditor() ) 
			);
			
			JToolBar tb = new JToolBar();
			tb.setFloatable( false );

			cbTagMode = new JCheckBox( "Display tag" );
			cbTagMode.addActionListener( this );
			tb.add( cbTagMode );

			view.add( 
				BorderLayout.SOUTH, 
				tb 
			);
		}
		return view;
	}

	public void actionPerformed(ActionEvent e) {
		getEditor().setDisplayTag( 
			cbTagMode.isSelected() 
		);
		getEditor().requestFocus();
	}

	private EditorByCSS getEditor() {
		if ( editor == null ) {
			editor = new EditorByCSS();
			editor.setDisplayTag( false );
			editor.setLocationListener( this );
			editor.setFactory( this );
		}
		return editor;
	}

	public boolean isModified() {
		getEditor().prepareDOMDocument();
		return true;
	}

	public void restoreState(String serialize) {
	}

	public String serializeState() {
		return null;
	}
	
	// Factory for the Visual Component
	
	public boolean confirm( String message ) {
		return EditixFactory.buildAndShowConfirmDialog( message ); 
	}

	private String lastNodeNameQuery = null;
	
	public String getNodeName( String defaultValue ) {
		if ( defaultValue == null )
			defaultValue = lastNodeNameQuery; 
		return lastNodeNameQuery = EditixFactory.buildAndShowInputDialog( "Choose a node name ?", defaultValue );
	}

}
