package com.japisoft.editix.ui.panels;

import javax.swing.Icon;
import javax.swing.JComponent;

import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public abstract class AbstractPanel implements Panel {
	private boolean shownState = false;
	protected String params = null;
	
	public void init() {}

	public void setParams(String params) {
		this.params = params;
	}	

	public void setState( boolean shown ) {
		this.shownState = shown;
	}

	public void close() {}
	public void select(Object path) {}	

	public void showPanel() {		
		preShow();
		shownState = true;
		show();
	}
	
	protected void preShow() {}
	
	public boolean isShown() {
		return shownState;
	}	

	public void hidePanel() {
		shownState = false;
		hide();
	}

	public void showHidePanel() {
		if ( EditixFrame.dockingSpace.isHiddenPanes() )
			shownState = false;
		if ( !shownState ) {
			showPanel();
		} else {
			hidePanel();
		}
	}

	protected void show() {
		if ( !EditixFrame.dockingSpace.hasPane( 
				getId() ) ) {
			EditixFrame.dockingSpace.setPane(
					getId(),
					getTitle(),
					getIcon(),
					getView() );
		}
		EditixFrame.dockingSpace.showPane( getId() );
		postShow();
	}

	protected void postShow() {}
	
	protected void hide() {
		preHide();
		PanelManager.saveState( true );
		EditixFrame.dockingSpace.hidePane( getId() );
	}

	protected void preHide() {}
	
	private JComponent builtView;

	protected abstract String getTitle();

	public JComponent getView() {
		if ( builtView == null )
			builtView = buildView();
		return builtView;
	}

	protected abstract JComponent buildView();
	
	private String id;
	public void setId( String id ) { this.id = id; }
	protected String getId() { return id; }
	
	private Icon icon;
	protected Icon getIcon() {
		return icon;
	}
	public void setIcon( Icon icon ) {
		this.icon = icon;
	}
	
	public boolean isVisible() {
		return shownState;
	}
	
	public void setCurrentXMLContainer( XMLContainer container ) {}

}
