package com.japisoft.editix.ui.panels.project;

import javax.swing.JComponent;

import com.japisoft.editix.project.ProjectListener;
import com.japisoft.editix.project.ProjectManager;
import com.japisoft.editix.ui.panels.AbstractPanel;
import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.framework.preferences.Preferences;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ProjectPanel extends AbstractPanel implements ProjectListener {

	public void init() {
		super.init();
		ProjectManager.setProjectListener( this );
		
		// Must load the last one ??
		if ( ProjectUI.isLocked() ) {
			String lastProjectFile = 
				Preferences.getPreference( 
						Preferences.SYSTEM_GP,
						"project.lastFile", ( String )null );
			if ( lastProjectFile != null ) {
				ProjectListener pl = ProjectManager.getProjectListener();
				ProjectManager.setProjectListener( null );
				try {
					ProjectManager.openProject( lastProjectFile );
				} finally {
					ProjectManager.setProjectListener( pl );
				}
			}
		}
		( ( ProjectUI )getView() ).init();
	}

	protected String getTitle() {
		return "Project";
	}

	protected JComponent buildView() {
		return new ProjectUI();
	}

	public void loadProject( String file ) {
		( ( ProjectUI )getView() ).loadProject( file );
		show();
		setState( true );
	}

	public void clean() {
		( ( ProjectUI )getView() ).clean();		
	}

	public void showPanel() {
		super.showPanel();
		ProjectManager.initProjectActions( true );
		( ( ProjectUI )getView() ).setEnabledAction( 
				ProjectUI.CHECK_CMD, true );
		( ( ProjectUI )getView() ).setEnabledAction( 
				ProjectUI.ZIP_CMD, true );
		ActionModel.setEnabled( "prjSaveAs", true );
		ActionModel.setEnabled( "prjSave", true );
		( ( ProjectUI )getView() ).init();
	}

	public void refresh() {
		( ( ProjectUI )getView() ).refresh();
	}	

	public void stop() {		
		Preferences.setPreference( 
				Preferences.SYSTEM_GP,
				"project.lastFile", ProjectManager.getLastOpenProject( true ) );
		// Save it
		ProjectManager.saveProject( ProjectManager.getLastOpenProject( true ) );
	}

	protected void preHide() {
		for ( int i = 0; i < ProjectUI.POPUP_ACTIONS.length; i++ ) {
			if ( ProjectUI.POPUP_ACTIONS[ i ] != null )
				ActionModel.setEnabled( "prj." + ProjectUI.POPUP_ACTIONS[ i ], false );
		}
		ActionModel.setEnabled( "prjSaveAs", false );
		ActionModel.setEnabled( "prjSave", false );		
	}	

}
