package com.japisoft.editix.ui.panels.project2;

import java.io.File;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import com.japisoft.framework.ui.text.FileTextField;
import com.japisoft.framework.ui.text.FilePlainDocument;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class NewProjectPanel extends javax.swing.JPanel implements DocumentListener {

    private static final String DEF_MY_PROJECT = "myProject";

	/** Creates new form NewProjectPanel */
    public NewProjectPanel() {
        initComponents();
        txtProjectLocation.setDirectoryMode( true );
        txtProjectName.setDocument( new FilePlainDocument() );
        txtProjectName.setText( DEF_MY_PROJECT );
    }

	@Override
	public void addNotify() {
		super.addNotify();
        txtProjectName.getDocument().addDocumentListener( this );		
	}

	@Override
	public void removeNotify() {
		super.removeNotify();
        txtProjectName.getDocument().removeDocumentListener( this );		
	}

	public File getProjectLocation() {
		return new File( txtProjectLocation.getText() );
	}
	
    public void changedUpdate(DocumentEvent e) {
    }

    public void insertUpdate(DocumentEvent e) {
    	String currentProject = txtProjectLocation.getText();
    	int i = currentProject.lastIndexOf( "/" );
    	if ( i == -1 )
    		i = currentProject.lastIndexOf( "\\" );
    	if ( i > -1 ) {
    		String pathHeader = currentProject.substring( 0, i + 1 );
    		String fullPath = pathHeader + txtProjectName.getText();
    		txtProjectLocation.setText( fullPath );
    	}    	
    }

    public void removeUpdate(DocumentEvent e) {
    	insertUpdate( e );
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">
    private void initComponents() {

        lblProjectName = new javax.swing.JLabel();
        txtProjectName = new javax.swing.JTextField( DEF_MY_PROJECT );
        lblProjectLocation = new javax.swing.JLabel();
        txtProjectLocation = new FileTextField( 
        		System.getProperty("user.home" ) + System.getProperty( "file.separator" ) + DEF_MY_PROJECT,
        		(String)null
        );
        
        lblProjectName.setText("Project Name");
        lblProjectName.setName("lblProjectName"); // NOI18N

        txtProjectName.setName("txtProjectName"); // NOI18N

        lblProjectLocation.setText("Project Location (a directory with all your project files)");
        lblProjectLocation.setName("lblProjectLocation"); // NOI18N

        txtProjectLocation.setName("txtProjectLocation"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(txtProjectName, javax.swing.GroupLayout.DEFAULT_SIZE, 454, Short.MAX_VALUE)
                    .addComponent(lblProjectName)
                    .addComponent(lblProjectLocation)
                    .addComponent(txtProjectLocation, javax.swing.GroupLayout.DEFAULT_SIZE, 454, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(lblProjectName)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(txtProjectName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(lblProjectLocation)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(txtProjectLocation, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(28, Short.MAX_VALUE))
        );
    }// </editor-fold>

    // Variables declaration - do not modify
    private javax.swing.JLabel lblProjectLocation;
    private javax.swing.JLabel lblProjectName;
    private FileTextField txtProjectLocation;
    private javax.swing.JTextField txtProjectName;
    // End of variables declaration

}
