package com.japisoft.editix.ui.panels.xslscenarios;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.xml.parser.ParseException;
import com.japisoft.framework.xml.parser.FPParser;
import com.japisoft.framework.xml.parser.document.Document;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.parser.node.NodeFactoryImpl;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XSLScenariosTool {

	static FPNode currentRoot;
	
	static FPNode getRoot() throws IOException {
		
		if ( currentRoot != null )
			return currentRoot;

		File f = new File(
				ApplicationModel.getAppUserPath(),
				"xslscenarios.xml" 
		);

		if ( !f.exists() ) {
			return getDefaultRoot();
		} else
			return getRoot( new FileReader( f ) );

	}

	private static FPNode getDefaultRoot() throws IOException {
		
		URL u = XSLScenariosTool.class.getResource( 
				"xslscenarios.xml" 
		);

		if ( u == null ) {
			// Pb with the default snippets ?
			FPNode n = new FPNode( 
					FPNode.TAG_NODE, 
					"group" );
			n.setAttribute( 
					"name", 
					"scenarios" );
			currentRoot = n;
			return n;
		} else {
			return getRoot(
					new InputStreamReader( u.openStream() )
			);
		}
	}

	private static FPNode getRoot( Reader r ) throws IOException {

		try {
			FPParser p = new FPParser();
			Document d = p.parse(r);
			return ( currentRoot = ( FPNode )d.getRoot() );
		} catch (ParseException e) {
			throw new IOException( "Can't parse the styles file : " + e.getMessage() ); 
		}

	}

	public static void storeXSLScenariosFile() throws IOException {

		if ( currentRoot == null )
			return;	//?

		File f = new File(
				ApplicationModel.getAppUserPath(),
				"xslscenarios.xml" );

		Document d = currentRoot.getDocument();
		d.write( new FileWriter( f ) );
		
	}
	
}
