package com.japisoft.editix.ui.xslt.map;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.tree.TreePath;

import org.w3c.dom.Element;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DrawLinkPanel extends JPanel {

	private JTree xsltTree;
	private JTree rightTree;	

	public DrawLinkPanel( 
			JTree xsltTree, 
			JTree rightTree ) {
		this.xsltTree = xsltTree;
		this.rightTree = rightTree;
	}

	private TreePath getPath( VirtualDomNode n ) {
		List<VirtualDomNode> l = new ArrayList<VirtualDomNode>();
		while ( n != null ) {
			l.add( 0, n );
			n = n.getParent();
		}
		if ( l.size() == 0 )
			return null;
		return new TreePath( l.toArray() );
	}

	public void dispose() {
		xsltTree = null;
		rightTree = null;
	}

	private void drawLinks( 
			Graphics g, 
			Rectangle left, 
			TreePath currentXsltPath ) {		

		if ( !( currentXsltPath.getLastPathComponent() instanceof VirtualDomNode ) )
			return;
		
		
		VirtualDomNode xsltNode = ( VirtualDomNode )currentXsltPath.getLastPathComponent();

		for ( int i = xsltNode.getChildCount( MapPanel.HTTP_WWW_W3_ORG_1999_XSL_TRANSFORM ) - 1; i >= 0; i-- ) {
			VirtualDomNode otherNode = xsltNode.getChildAt( MapPanel.HTTP_WWW_W3_ORG_1999_XSL_TRANSFORM, i );
			TreePath tpOther = getPath( otherNode );
			if ( tpOther == null ) {
				xsltNode.getChildAt( MapPanel.HTTP_WWW_W3_ORG_1999_XSL_TRANSFORM, i );
				continue;
			}
			rightTree.expandPath( tpOther );
			
			Element eTmp = ( Element )xsltNode.getSource();
			String lbl = eTmp.getAttribute( "match" );
			if ( "".equals( lbl ) ) {
				lbl = eTmp.getAttribute( "name" );
			}
			if ( "".equals( lbl ) )
				lbl = eTmp.getAttribute( "select" );

			Rectangle r2 = rightTree.getPathBounds( tpOther );
			if ( r2 != null ) {
				int xi = 0;
				int yi = left.y + left.height;
				
				int xj = getWidth();
				int yj = r2.y + r2.height;
			
				yi += 20;
				yj += 20;

				g.setColor( Color.BLACK );
				g.fillOval( -5 + xi, yi - 5, 10, 10 );
				g.fillOval( xj - 5, yj - 5, 10, 10 );

				g.setColor( Color.DARK_GRAY );
				g.drawString( lbl, ( xj - xi ) / 2, ( yi + ( yj - yi ) / 2 ) - 5 );

				g.setColor( Color.BLUE );
				g.drawLine( xi, yi, xj, yj );
			}
		}
	}

	@Override
	protected void paintComponent( Graphics g ) {
		super.paintComponent(g);
		
		g.setColor( Color.GRAY );
		g.drawRect( 0, 0, getWidth(), getHeight() - 2 );
		
		TreePath tp = xsltTree.getSelectionPath();
		Enumeration<TreePath> en = xsltTree.getExpandedDescendants( new TreePath( xsltTree.getModel().getRoot() ) );
		if ( en != null ) {
			while ( en.hasMoreElements() ) {
				TreePath tpCurrent = en.nextElement();
				while ( tpCurrent != null ) {
					Rectangle r = xsltTree.getPathBounds( tpCurrent );				
					drawLinks( g, r, tpCurrent );
					tpCurrent = tpCurrent.getParentPath();
				}
			}			
		}
	}

}
