package com.japisoft.editix.wizard;

import java.util.List;

import com.japisoft.editix.wizard.link.Link;
import com.japisoft.editix.wizard.link.LinkWizardModel;
import com.japisoft.editix.wizard.list.ListWizardModel;
import com.japisoft.editix.wizard.table.TableWizardModel;
import com.japisoft.framework.xml.parser.node.FPNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XSLFOWizardContext extends HTMLWizardContext {

	/*
		txtTable.setText( context.getProperty( "table", txtTable.getText() ) );
		txtTableHeader.setText( context.getProperty( "table.header", txtTableHeader.getText() ) );
		txtTableBody.setText( context.getProperty( "table.body", txtTableBody.getText() ) );
		txtHeaderCell.setText( context.getProperty( "table.header.cell", txtHeaderCell.getText() ) );
		txtTableRow.setText( context.getProperty( "table.row", txtTableRow.getText() ) );
		txtTableCell.setText( context.getProperty( "table.cell", txtTableCell.getText() ) );
	*/
	public XSLFOWizardContext() {
			super( 
				new String[] {
					"table", "fo:table",
					"table.header", "fo:table-header",
					"table.body", "fo:table-body",
					"table.header.cell", "fo:table-cell",
					"table.row", "fo:table-row",
					"table.cell", "fo:table-cell",
					"table.cell.content", "fo:block",
					"list", "list-block",
					"list.item", "list-item",
				}
			);
		}

		@Override
		protected FPNode createTable(TableWizardModel t, int w, int height) {
			FPNode node = super.createTable(t, w, height);
			node.setAttribute( "table-layout", "fixed" );
			for ( int i = 0; i < w; i++ ) {
				String width = t.getHeaderWidth( i );
				if ( isEmpty( width ) ) {
					width = "proportional-column-width(1)";
				}
				node.appendChild( 
					new FPNode( FPNode.TAG_NODE, "table-column" ) ).setAttribute( 
							"column-width", width 
				);
				}
			return node;
		}
		
		@Override
		protected FPNode processCellWidth(FPNode cell, String width) {
			return cell;
		}

		@Override
		protected FPNode createTableCell(TableWizardModel t) {
			FPNode node = super.createTableCell(t);
			node.appendChild( createBlock() );
			return node;
		}

		@Override
		protected FPNode createTableHeaderCell(TableWizardModel t,String content) {
			FPNode node = super.createTableHeaderCell(t,content);
			node.appendChild( createBlock( content ) );
			return node;
		}

		private FPNode createBlock() {
			return createBlock( null );
		}

		private FPNode createBlock( String content ) {
			FPNode node = new FPNode( FPNode.TAG_NODE, "fo:block" );
			node.setAutoClose( false );
			if ( content != null ) {
				node.appendChild( new FPNode( FPNode.TEXT_NODE, content ) );
			}
			return node;
		}

		//////////////////////////////////////////////////////////////////

		public FPNode getResult(ListWizardModel t) {
			FPNode node = t.getList();
			FPNode ul = new FPNode( FPNode.TAG_NODE, "fo:list-block" );
			buildLst( node, ul );
			return ul;
		}

/*
		<list-block provisional-distance-between-starts=".43in" 
            provisional-label-separation=".1in" 
            space-before.optimum="6pt"
>
  <list-item relative-align="baseline">
    <list-item-label text-align="end" 
                     end-indent="label-end()">
      <block>-</block>
    </list-item-label>
    <list-item-body start-indent="body-start()">
      <block font-size="14pt">
        excerpts of formatting objects 
        created through the use of an XSLT stylesheet
      </block>
    </list-item-body>
  </list-item>
</list-block>		
	*/
		
		private void buildLst( FPNode mnode, FPNode hnode ) {
			if ( mnode.matchContent( "ul" ) ) {
				for ( int i = 0; i < mnode.childCount(); i++ )
					buildLst( mnode.childAt( i ), hnode );
			} else
			if ( mnode.matchContent( "li" ) ) {
				FPNode li = new FPNode( FPNode.TAG_NODE, "fo:list-item" );
				hnode.appendChild( li );

				FPNode l = new FPNode( FPNode.TAG_NODE, "fo:list-item-label" );
				l.appendChild( createBlock( "-") );				
				li.appendChild( l );

				l = new FPNode( FPNode.TAG_NODE, "fo:list-item-body" );				
				l.setAttribute( "start-indent", "body-start()" );
				l.appendChild( createBlock( mnode.getApplicationObject().toString() ) );
				li.appendChild( l );
				
				if ( mnode.childCount() > 0 ) {
					FPNode parent = new FPNode( FPNode.TAG_NODE, "fo:list-block" );
					l.appendChild( parent );	
					for ( int i = 0; i < mnode.childCount(); i++ ) {
						buildLst( mnode.childAt( i ), parent );
					}					
				}
			}

		}

		////////////////////////////////////////////////////////////////////////////
		
		public FPNode getResult(LinkWizardModel t) {
			FPNode root = new FPNode( FPNode.TAG_NODE, null );
			List<Link> l = t.getExternalLinks();
			if ( l != null ) {
				for ( Link k : l ) {
					if ( k.isEnabled() && !k.isEmpty() ) {
						FPNode a = new FPNode( FPNode.TAG_NODE, "fo:basic-link" );
						a.setAttribute( "external-destination", "url('" + k.getUri() + "')" );
						a.setAttribute( "text-decoration", "underline" );
						a.appendChild( new FPNode( FPNode.TEXT_NODE, k.getLabel() ) );
						root.appendChild( a );
					}
				}
			}
			l = t.getInternalLinks();
			if ( l != null ) {
				for ( Link k : l ) {
					if ( k.isEnabled() && !k.isEmpty() ) {
						FPNode a = new FPNode( FPNode.TAG_NODE, "fo:basic-link" );					
						a.setAttribute( "internal-destination", k.getUri() );
						a.setAttribute( "text-decoration", "underline" );
						a.appendChild( new FPNode( FPNode.TEXT_NODE, k.getLabel() ) );
						root.appendChild( a );
					}
				}
			}
			return root;			
		}

}
