package com.japisoft.editix.wizard.document;

import java.io.File;

import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JScrollPane;
import javax.swing.filechooser.FileFilter;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.xml.xsd.instance.XSDInstanceGenerator;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XSD2XMLDocument implements DocumentWizard {

	private File source;
	
	@Override
	public String start() {
		JFileChooser fc = EditixFactory.buildFileChooser( new FileFilter() {			
			@Override
			public String getDescription() {
				return "W3C Schema file (*.xsd)";
			}
			@Override
			public boolean accept( File f ) {
				if ( f.isFile() ) {
					String tmp = f.getName().toLowerCase();
					return tmp.endsWith( ".xsd" );
				} else
					return true;
			}
		});
		if ( fc.showOpenDialog( EditixFrame.THIS ) == 
				JFileChooser.APPROVE_OPTION ) {
			try {
				source = fc.getSelectedFile();
				DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
				factory.setNamespaceAware( true );
				DocumentBuilder builder = factory.newDocumentBuilder();
				
				Document doc = builder.parse( source );
				NodeList nl = doc.getDocumentElement().getChildNodes();
				JComboBox<String> combo = new JComboBox<String>();

				for ( int i = 0; i < nl.getLength(); i++ ) {
					if ( nl.item( i ) instanceof Element ) {
						Element e = ( Element )nl.item( i );
						if ( "element".equals( e.getNodeName() ) || "element".equals( e.getLocalName() ) )
							combo.addItem( e.getAttribute( "name" ) );
					}
				}
				
				if ( combo.getItemCount() == 0 ) {
					EditixFactory.buildAndShowWarningDialog( "Can't find element definition inside your schema ?" );
					return null;
				}

				if ( DialogManager.showDialog( 
						EditixFrame.THIS, 
						"Select the root node", 
						"Selection the root node for generating your XML document", 
						"", 
						null, 
						combo ) == DialogManager.OK_ID ) {
					String root = ( String )combo.getSelectedItem();
					if ( root == null ) {
						EditixFactory.buildAndShowErrorDialog( "No root selection ?" );
						return null;
					}
					
					return "<?xml version='1.0'?>\n\n" + XSDInstanceGenerator.generateXMLInstance( root, source.toString() );

				}

			} catch( Throwable exc ) {
				EditixFactory.buildAndShowErrorDialog( "Can't use this file : " + exc.getMessage() );
			}
			
		}
		return null;
	}
	
	public File getSource() { return source; }
	
}
