package com.japisoft.framework.application.descriptor.composer;

import java.awt.Component;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.tree.TreeCellRenderer;

import org.w3c.dom.Element;
import com.japisoft.framework.ui.FastLabel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DescriptorTreeRenderer implements TreeCellRenderer {

	private FastLabel fl = new FastLabel();

	private Map<String,Icon> icons = new HashMap<String,Icon>();

	public DescriptorTreeRenderer() {
		Icon container = new ImageIcon( getClass().getResource( "folder_cubes.png" ) );
		Icon item = new ImageIcon( getClass().getResource( "bullet_square_blue.png" ) );
		Icon separator = new ImageIcon( getClass().getResource( "navigate_minus.png" ) );
		Icon root = new ImageIcon( getClass().getResource( "environment2.png" ) );

		icons.put( "menuBar", container );
		icons.put( "menu", container );
		icons.put( "toolBar", container );
		icons.put( "popup", container );
		icons.put( "item", item );
		icons.put( "itemRef", item );
		icons.put( "separator", separator );
		icons.put( "editix", root );
	}

	public Component getTreeCellRendererComponent(
		JTree tree, 
		Object value,
		boolean selected, 
		boolean expanded, 
		boolean leaf, 
		int row,
		boolean hasFocus ) {
		
		if ( value instanceof Element ) {
		
			Element n = ( Element )value;
			fl.setText( n.getNodeName() + ( n.hasAttribute( "id" ) ? ( " [ " + n.getAttribute( "id" ) + " ]" ) : "" ) );
			fl.setIcon( icons.get( n.getNodeName() ) );

			fl.setBackground( tree.getBackground() );
			fl.setForeground( tree.getForeground() );
			
			if ( selected ) {
				fl.setBackground( UIManager.getColor( "Tree.selectionBackground" ) );
				fl.setForeground( UIManager.getColor( "Tree.selectionForeground" ) );
			}
			
		} else {
		
			fl.setText( value.toString() );	// ?
			
		}
		
		return fl;
	}

}
