package com.japisoft.framework.application.descriptor.helpers;

import java.util.HashMap;
import java.util.Map;

import javax.swing.Action;

import com.japisoft.framework.application.descriptor.InterfaceBuilderException;
import com.japisoft.framework.xml.parser.node.FPNode;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DefaultActionBuilderImpl implements ActionBuilder {

	private static Map CLASS_TRANSLATOR = null;

	/** Translate an action name to another one. This is useful when you
	 * must compute the final action */
	public static void translateActionClass( String actionSource, String finalAction ) {
		if ( CLASS_TRANSLATOR == null )
			CLASS_TRANSLATOR = new HashMap();
		CLASS_TRANSLATOR.put( actionSource, finalAction );
	}

	private static Map CLASS_PROVIDER = null;
	
	/** Provide a class for an action name. Thus you can found a class
	 * anywhere */
	public static void provideClass( String actionSource, Class finalClass ) {
		if ( CLASS_PROVIDER == null )
			CLASS_PROVIDER = new HashMap();
		CLASS_PROVIDER.put( actionSource, finalClass );
	}

	public Action buildAction( FPNode source, String actionName ) throws InterfaceBuilderException {
		try {
			if ( CLASS_TRANSLATOR != null )
				if ( CLASS_TRANSLATOR.containsKey( actionName ) )
					actionName = ( String )CLASS_TRANSLATOR.get( actionName );
			
			Class cl = null;
			
			if ( CLASS_PROVIDER != null && 
					CLASS_PROVIDER.containsKey( actionName ) ) {
				cl = ( Class )CLASS_PROVIDER.get( actionName );
			}

			if ( cl == null )
				cl = Class.forName( actionName );
			
			return ( Action )cl.newInstance();
		} catch( Throwable th ) {
			throw new InterfaceBuilderException( "Cannot find the class : [" + actionName + "]", th );
		}
	}	

}
