package com.japisoft.framework.dialog.actions;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeListener;

import com.japisoft.framework.dialog.BasicDialogComponent;
import com.japisoft.framework.dialog.DialogComponent;
import com.japisoft.framework.dialog.DialogManager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DialogActionProxy extends AbstractDialogAction {

	public DialogAction delegate;
	private DialogComponent target;
	
	public DialogActionProxy( DialogAction action, DialogComponent target ) {
		super( action.getActionId(), false );
		this.delegate = action;
		this.target = target;
	}

	public void putValue(String key, Object newValue) {
		if ( delegate != null )
			delegate.putValue( key, newValue );
		else
			super.putValue( key, newValue );
	}
	
	public Object getValue(String key) {
		if ( delegate != null )
			return delegate.getValue( key );
		return super.getValue( key );
	}

	public synchronized void addPropertyChangeListener(
			PropertyChangeListener listener) {
		if ( delegate != null )
			delegate.addPropertyChangeListener( listener );
		else
			super.addPropertyChangeListener( listener );
	}
	
	public synchronized void removePropertyChangeListener(
			PropertyChangeListener listener) {
		if ( delegate != null )
			delegate.removePropertyChangeListener( listener );
		else
			super.removePropertyChangeListener( listener );
	}

	public void actionPerformed( ActionEvent e ) {
		if ( delegate != null ) {			
			delegate.actionPerformed( e );
			if ( target instanceof BasicDialogComponent ) {
				( ( BasicDialogComponent )target ).setLastAction( 
						delegate.getActionId() );
			}
			if ( delegate instanceof ClosableAction ) {
				// Check for veto
				if ( delegate instanceof AbstractDialogAction ) {
					if ( ( ( AbstractDialogAction )delegate ).isVetoClosingDialog() )
						return;
				}
				if ( target != null ) {
					target.setVisible( false );
				}
				else {
					if( DialogManager.getCurrentDialog() != null ) {
						
						if ( DialogManager.getCurrentDialog() instanceof BasicDialogComponent ) {
							( ( BasicDialogComponent )DialogManager.getCurrentDialog() ).setLastAction( 
									delegate.getActionId() );							
						}

						DialogManager.getCurrentDialog().setVisible( false );
					}
				}
			}
		} else
			System.err.println( "No delegate found ?" );
	}

	public void setEnabled(boolean b) {
		if ( delegate != null )
			delegate.setEnabled( b );
		else
			super.setEnabled( b );
	}

	public boolean isEnabled() {
		if ( delegate != null )
			return delegate.isEnabled();
		return super.isEnabled();
	}

	public boolean isVetoClosingDialog() {
		if ( delegate != null && 
				( delegate instanceof AbstractDialogAction ) )
			return ( ( AbstractDialogAction )delegate ).isVetoClosingDialog();
		return super.isVetoClosingDialog();		
	}

}
