package com.japisoft.framework.dialog.help;

import java.awt.event.ActionEvent;
import java.io.ByteArrayOutputStream;
import java.io.FileWriter;

import javax.swing.AbstractAction;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.dialog.BasicOKDialogComponent;
import com.japisoft.framework.job.Job;
import com.japisoft.framework.job.JobManager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class AutoHelpAction extends AbstractAction {

	DocumentDialog dialog;

	public void actionPerformed(ActionEvent e) {
		dialog = new DocumentDialog( "Please wait while generating documentation..." );
		JobManager.addJob( new DocGeneratorJob() );
		dialog.setVisible( true );
		dialog.dispose();
		dialog = null;
	}

	class DocGeneratorJob implements Job {
		public Object getSource() {
			return AutoHelpAction.this;
		}

		public boolean isAlone() {
			return true;
		}

		public void dispose() {}
		
		public void run() {

			if ( ApplicationModel.USERINTERFACE_FILE == null ) {
				throw new RuntimeException( "No value for the property INTERFACE_BUILDER of the ApplicationModel" );
			}
			
			if ( ApplicationModel.AUTODOC_FILE == null ) {
				throw new RuntimeException( "No stylesheet, set the AUTODOC_FILE property" );
			}

			TransformerFactory tf = 
				TransformerFactory.newInstance();
			try {
				Transformer transformer = tf.newTransformer(
					new StreamSource( 
							ClassLoader.getSystemResourceAsStream( 
									ApplicationModel.AUTODOC_FILE ) ) );

				transformer.setParameter( "appname", ApplicationModel.LONG_APPNAME );
				ByteArrayOutputStream output = new ByteArrayOutputStream();

				transformer.transform( 
					new StreamSource( ClassLoader.getSystemResourceAsStream( 
							ApplicationModel.USERINTERFACE_FILE ) ),
					new StreamResult( output ) );

				dialog.setContent( output.toString() );
				
				if ( ApplicationModel.DEBUG_AUTODOC_FILE != null && ApplicationModel.DEBUG_MODE ) {
					ApplicationModel.debug( "Debugging the auto doc file : " + ApplicationModel.DEBUG_AUTODOC_FILE );
					try {
						FileWriter fw = new FileWriter( ApplicationModel.DEBUG_AUTODOC_FILE );
						fw.write( output.toString() );
						fw.close();
					} catch (RuntimeException e) {
						e.printStackTrace();
					}
				}
				
			} catch( Throwable th ) {
				dialog.setContent( "Error : " + th.getMessage() );
			}
		}

		public void stopIt() {
		}
		
		public boolean hasErrors() {

			return false;
		}
	}

	class DocumentDialog extends BasicOKDialogComponent {
		JEditorPane ep = null;
		
		public DocumentDialog( String htmlContent ) {
			super( 
					ApplicationModel.MAIN_FRAME, 
					"Documentation", 
					"Menu Documentation", 
					ApplicationModel.LONG_APPNAME + " menu content", null );

			setUI( new JScrollPane( ep = new JEditorPane( "text/html", htmlContent ) ) );
			ep.setEditable( false );
			ep.setCaretPosition( 0 );
			pack();
			setSize( 600, 600 );
		}

		public void setContent( String html ) {
			ep.setText( html );
			ep.setCaretPosition( 0 );
		}
	}

}
