package com.japisoft.framework.dockable;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;

import com.japisoft.framework.dockable.action.ActionModel;
import com.japisoft.framework.dockable.action.BasicActionModel;
import com.japisoft.framework.dockable.action.CommonActionManager;
import com.japisoft.framework.dockable.action.DockableAction;
import com.japisoft.framework.dockable.action.ModelStateListener;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BasicInnerWindow extends JPanel implements Windowable {

	private InnerWindowProperties properties;
	private boolean maximized = false;

	public BasicInnerWindow( InnerWindowProperties properties ) {
		this.properties = properties;
		if ( properties.isAutoScroll() )
			realPanel = new JScrollPane();
		else
			realPanel = ComponentFactory.getComponentFactory().buildContentPaneForFrame();
		initUI( null );
		if ( realPanel instanceof JScrollPane )
			((JScrollPane)realPanel).setViewportView( properties.getView() );
		else
			realPanel.add( properties.getView() );
		
		dispatchProperties();
		
		setMinimumSize( new Dimension( 10, 10 ) );
	}

    public Dimension getMinimumSize() {
    	Dimension d = null;
    	try { 
    		d = super.getMinimumSize();
    	} catch( NullPointerException e ) {
    		e.printStackTrace();
    	}
    	if ( d == null )
    		d = new Dimension( 0, 0 );
    	return d;
    }

	private void dispatchProperties() {
		setTitle( properties.getTitle() );
		setIcon( properties.getIcon() );
		actions = properties.getActionModel();
	}

	public void requestFocus() {
		tb.focusMode( true );
	}

	Rectangle frameBounds = null; // When extracting the content
	
	private JComponent realPanel;
	private DockableFrameTitleBar tb = null;

	private void initUI( String title ) {
		super.setLayout( new BorderLayout( 0, 0 ) );
		super.add( ( tb = ComponentFactory.getComponentFactory().buildTitleBar( title ) ).getView(), BorderLayout.NORTH );
		super.add( realPanel, BorderLayout.CENTER );
		if ( !( realPanel instanceof JScrollPane ) ) {	// For the autoscroll mode
			realPanel.setLayout( new MonoLayout() );
			//realPanel.setLayout( new BorderLayout() );
		}
	}

	private CustomFocusAdapter adapter = null;
	
	public void addNotify() {
		super.addNotify();
		if ( commonActionsListener == null )
			commonActionsListener = new CustomModelStateListener();
		getActionModel().addModelStateListener( commonActionsListener );
		if ( realPanel.getComponentCount() > 0 ) {
			Component c = realPanel.getComponent( 0 );
			c.addFocusListener( adapter = new CustomFocusAdapter() );
			recursiveAddFocus( c );
		}
				
		tb.getView().addMouseListener( interactionListener = new InteractionMouseListener() );
		tb.getView().addMouseMotionListener( interactionListener );
		if ( !toolbarUpdatedOnce ) {
			notifyCommonActionModelUpdated();
			toolbarUpdatedOnce = true;
		}
	}

	private boolean toolbarUpdatedOnce = false;
	
	public String toString() {
		return "Frame:" + properties.getId();
	}
	
	private void recursiveAddFocus( Component c ) {
		if ( c instanceof Container ) {
			Container c2 = ( Container )c;
			for ( int i = 0; i < c2.getComponentCount(); i++ ) {
				c2.getComponent( i ).addFocusListener( adapter );
				if ( c2.getComponent( i ) instanceof Container )
					recursiveAddFocus( c2.getComponent( i ) );
			}
		}
	}

	private void recursiveRemoveFocus( Component c ) {
		if ( c instanceof Container ) {
			Container c2 = ( Container )c;
			for ( int i = 0; i < c2.getComponentCount(); i++ ) {
				c2.getComponent( i ).removeFocusListener( adapter );
				if ( c2.getComponent( i ) instanceof Container )
					recursiveRemoveFocus( c2.getComponent( i ) );
			}
		}
	}

	public void removeNotify() {
		super.removeNotify();
		getActionModel().removeModelStateListener( commonActionsListener );
		if ( realPanel.getComponentCount() > 0 && adapter != null ) {
			Component c = realPanel.getComponent( 0 );
			c.removeFocusListener( adapter );
			recursiveRemoveFocus( c );
			adapter = null;
		}
		tb.getView().removeMouseListener( interactionListener );
		tb.getView().removeMouseMotionListener( interactionListener );
	}

	public String getTitle() {
		return tb.getTitle();
	}
	
	public String getId() {
		return properties.getId();
	}

	public JComponent getUserView() {
		return properties.getView();
	}

	public JComponent getView() {
		return this;
	}

	public void dispose() {
		properties.dispose();
	}

	public void setTitle( String title ) {
		tb.setTitle( title );
	}

	public Icon getIcon() {
		return tb.getIcon();
	}
	
	public void setIcon(Icon icon) {
		tb.setIcon( icon );
	}
	
	public void setBackground( Color color ) {
		if ( tb != null )
			tb.setBackground( color );
	}
	
	public void setForeground( Color color ) {
		if ( tb != null )
			tb.setForeground( color );
	}
	
	public Rectangle getFrameBounds() {
		return frameBounds;
	}
	
	public void setFrameBounds(Rectangle r) {
		this.frameBounds = r;
	}	
	
	public JComponent getContentPane() {
		return realPanel;	
	}

	public void setContentPane( JComponent component ) {
		add( realPanel = component, BorderLayout.CENTER );
		invalidate();
		validate();
		repaint();
	}

	private boolean fixed = false;
	
	public void setFixed( boolean fixed ) {
		this.fixed = fixed;
	}

	public boolean isFixed() {
		return fixed;
	}

	private boolean resize = true;
	
	public boolean isResizable() {
		return resize;
	}
	public void setResizable(boolean resize) {
		this.resize = resize;
	}	

	public boolean isMaximized() {
		return maximized;
	}

	public void setMaximized( boolean max ) {
		this.maximized = max;
	}

	private ActionModel actions = null;
	
	/** @return a model for user actions. If empty this is filled with common actions from the CommonActionManager */
	public	ActionModel getActionModel() {
		if ( actions == null ) {
			actions = new BasicActionModel();
			CommonActionManager.fillModelWithCommonActions( this, actions );
		}
		for ( int i = 0; i < actions.getActionCount(); i++ ) {

			Action a = actions.getAction( i );
			if ( a instanceof DockableAction ) {
				DockableAction da = ( DockableAction )a;
				da.setDockableContext( this );
			}	
		}
		return actions;
	}

	private void notifyCommonActionModelUpdated() {
		CommonActionManager.fillWindowTitleBar( tb, getActionModel() );
	}

	public JDock getJDock() {
		return ( (JDock.InnerPanel)getParent() ).getJDock();
	}
	
	public void fireDockEvent( String id, int type ) {
		getJDock().fireJDockEvent( id, type );
	}

	private JPopupMenu popup = null;

	// Popup for the window bar action set
	private boolean activatePopup( int x, int y ) {
		if ( !getJDock().isEnabledActionPopup() )
			return false;
		if ( tb.getIcon() != null ) {
			if ( x <= tb.getIcon().getIconWidth() && 
					y <= tb.getIcon().getIconHeight() ) {
				ActionModel model = 
					getJDock().getInnerWindowActionsForId( getId() );
				if ( model.getActionCount() > 0 ) {
					popup = new JPopupMenu();
					for ( int i = model.getActionCount() - 1; i >= 0; i-- ) {
						Action a = model.getAction( i );
						if ( a == null ) {
							if ( popup.getComponentCount() > 0 && i > 0 )
								popup.addSeparator();
							continue;
						}
						if ( a.getValue( Action.NAME ) == null ) {
							if ( a.getValue( Action.SHORT_DESCRIPTION ) == null ) {
								continue;
							}
							JMenuItem item = 
								new JMenuItem();
							item.setAction( a );
							item.setText( ( String )a.getValue( Action.SHORT_DESCRIPTION ) );
							popup.add( item );
						} else {
							popup.add( a );
						}
					}
					popup.show( this, x, y );
					return true;
				}
			}
		}
		return false;
	}

	private void desactivatePopup() {
		popup = null;
	}
	
	private CustomModelStateListener commonActionsListener = null;
	private InteractionMouseListener interactionListener = null;

	class CustomFocusAdapter implements FocusListener {
		
		public void focusGained(FocusEvent e) {
			tb.focusMode( true );
		}

		public void focusLost(FocusEvent e) {
		}
	}
	
	class CustomModelStateListener implements ModelStateListener {
		public void modelModified( ActionModel source ) {
			if ( source == actions )
				notifyCommonActionModelUpdated();
		}
	}

	class InteractionMouseListener implements MouseListener, MouseMotionListener {
		
		public void mouseClicked( MouseEvent e ) {
			if ( getJDock().getInnerLayout().hasMaximizedComponent() )
				return;
		}

		public void mouseDragged( MouseEvent e ) {
			if ( getJDock().getInnerLayout().hasMaximizedComponent() )
				return;
			if ( popup == null )
				getJDock().workingDrag( BasicInnerWindow.this, e );
		}

		public void mouseEntered( MouseEvent e ) {
		}

		public void mouseExited( MouseEvent e ) {
		}

		public void mouseMoved( MouseEvent e ) {
		}

		public void mousePressed( MouseEvent e ) {
			if ( !activatePopup( e.getX(), e.getY() ) ) {
				if ( getJDock().getInnerLayout().hasMaximizedComponent() )
					return;
				getJDock().startDrag( e, BasicInnerWindow.this );
			}
		}

		public void mouseReleased( MouseEvent e ) {
			desactivatePopup();
			if ( getJDock().getInnerLayout().hasMaximizedComponent() )
				return;	
			getJDock().stopDrag();
		}
	}

	//////////////////////////////////////////////////////////

}
