package com.japisoft.framework.dockable;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ComponentFactory {

	private static ComponentFactory SINGLETON = null;
	
	/** Override the default factory by this sub class */
	public static void setComponentFactory( ComponentFactory factory ) {
		SINGLETON = factory;
	}

	/** @return the current component factory */
	public static ComponentFactory getComponentFactory() {
		if ( SINGLETON == null )
			SINGLETON = new ComponentFactory();
		return SINGLETON;
	}

	private boolean autoScrollMode = false;

	/** This is a mode for adding a JScrollPane automatically inside any inner window. By default <code>false</code> */
	public void setAutoScrollMode( boolean autoScrollMode ) {
		this.autoScrollMode = autoScrollMode;
	}

	/** Build a button for the title bar of the <code>DockableFrame</code> */
	public JButton buildButton( Action a ) {
		JButton b = new JButton( a );
		b.setBorderPainted( false );
		b.setBorder( null );
		b.setOpaque( false );
		return b;
	}

	/** Build a new title bar for the <code>DockableFrame</code> */
	public DockableFrameTitleBar buildTitleBar( String title ) {
		return new DefaultTitleBar( title );
	}

	/** Build an individual frame for the <code>DockableFrame</code> 
	 * @param source The Action that has changed the state of the <code>DockableFrame</code>
	 * @param panel The initial DockableFrame
	 * */	
	public JFrame buildDockedFrame( Action source, Windowable panel ) {
		return new DefaultDockedFrame( source, panel );
	}

	/** @return a new contentPane for the <code>DockableFrame</code> */
	public JComponent buildContentPaneForFrame() {
		if ( autoScrollMode )
			return new JScrollPane();
		else
			return new JPanel();
	}

}
