package com.japisoft.framework.dockable;

import javax.swing.Icon;
import javax.swing.JComponent;

import com.japisoft.framework.dockable.action.ActionModel;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class InnerWindowProperties {

	private String id;
	private String title;
	private Icon icon;
	private ActionModel model;
	private JComponent view;
	private boolean autoScroll;

	/**
	 * @param id The inner window id, it cannot be <code>null</code>
	 * @param title The inner window title
	 * @param icon The inner window icon
	 * @param model The inner window actions model for the toolbar
	 * @param view The inner window component
	 * @param autoScroll The inner window autoScroll management
	 */
	public InnerWindowProperties(
			String id, 
			String title, 
			Icon icon, 
			ActionModel model, 
			JComponent view, 
			boolean autoScroll ) {
		
		if ( id == null )
			throw new RuntimeException( "Invalid id, it cannot be null !" );
		if ( view == null )
			throw new RuntimeException( "Invalid view, it cannot be null !");
		
		this.id = id;
		this.title = title;
		this.icon = icon;
		this.model = model;
		this.view = view;
		this.autoScroll = autoScroll;
	}

	/**
	 * @param id The inner window id, it cannot be <code>null</code>
	 * @param title The inner window title
	 * @param icon The inner window icon
	 * @param model The inner window actions model for the toolbar
	 * @param view The inner window component
	 */
	public InnerWindowProperties(
			String id, 
			String title, 
			Icon icon, 
			ActionModel model, 
			JComponent view ) {
		this( id, title, icon, model, view, false );
	}
	
	/**
	 * @param id The inner window id, it cannot be <code>null</code>
	 * @param title The inner window title
	 * @param icon The inner window icon
	 * @param view The inner window component */
	public InnerWindowProperties(
			String id, 
			String title, 
			Icon icon, 
			JComponent view ) {
		this( id, title, icon, null, view, false );
	}	

	/**
	 * @param id The inner window id, it cannot be <code>null</code>
	 * @param title The inner window title
	 * @param icon The inner window icon
	 * @param view The inner window component 
	 * @param autoScroll The inner window autoScroll management
	 * */
	public InnerWindowProperties(
			String id, 
			String title, 
			Icon icon, 
			JComponent view,
			boolean autoScroll ) {
		this( id, title, icon, null, view, autoScroll );
	}	
	
	/**
	 * @param id The inner window id, it cannot be <code>null</code>
	 * @param title The inner window title
	 * @param view The inner window component */
	public InnerWindowProperties(
			String id,
			String title,
			JComponent view ) {
		this( id, title, null, null, view, false );
	}	
	
	/** @return the id for this inner window */
	public String getId() { return id; }
	/** @return the title for this inner window */
	public String getTitle() { return title; }
	/** @return the icon for this inner window */
	public Icon getIcon() { return icon; }
	/** @return the action model for this inner window toolbar */
	public ActionModel getActionModel() { return model; }
	/** @return the main component for this inner window */
	public JComponent getView() { return view; }
	/** @return the autoscroll status. By default <code>false</code> */
	public boolean isAutoScroll() { return autoScroll; }

	void dispose() {
		this.view = null;
	}
}
