package com.japisoft.framework.dockable.action;

import java.util.ArrayList;
import javax.swing.Action;

import com.japisoft.framework.dockable.ComponentFactory;
import com.japisoft.framework.dockable.BasicInnerWindow;
import com.japisoft.framework.dockable.DockableFrameTitleBar;
import com.japisoft.framework.dockable.action.common.*;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public final class CommonActionManager {

	static ArrayList model = new ArrayList();

	static {
		//addCommonAction( ExtractAction.class );
		addCommonAction( MaxMinAction.class );
	}

	/** Add a common action by its class */
	public static void addCommonAction( Class a ) {
		model.add( a );
	}

	/** Add a common action by its class */
	public static void addCommonAction( Class a, int location ) {
		model.add( location, a );
	}

	/** Remove an action by its class */
	public static void removeCommonAction( Class a ) {
		model.remove( a );
	}

	/** @return the available action count */
	public static int getCommonActionCount() {
		return model.size();
	}

	/** @return a common action class for this index starting from 0 */
	public static Class getCommonActionAt( int index ) {
		return ( Class )model.get( index );
	}

	/** @return a new instance of the Action for this index starting from 0 working for this panel */	
	public static Action buildCommonActionAt( int index, BasicInnerWindow panel ) {
		Class cl = ( Class )getCommonActionAt( index );
		try {
			Action a = ( Action )cl.newInstance();
			if ( a instanceof DockableAction ) {
				( ( DockableAction )a ).setDockableContext( panel );
			}
			return a;
		} catch( Throwable th ) { 
			th.printStackTrace();
			return null;
		}
	}

	/** Update the model with the following common actions */
	public static void fillModelWithCommonActions( BasicInnerWindow panel, ActionModel am ) {
		am.removeAll();
		for ( int i = 0; i < model.size(); i++ ) {
			am.addAction( buildCommonActionAt( i, panel ) );		
		}
	}

	/** Fill the inner window header with buttons from the following action model */
	public static void fillWindowTitleBar( DockableFrameTitleBar titleBar, ActionModel am ) {
		titleBar.removeAllButtons();

		for ( int i = 0; i < am.getActionCount(); i++ ) {
			if ( !am.isSeparator( i ) ) { 
				titleBar.addButton( 
						ComponentFactory.getComponentFactory().buildButton( 
								am.getAction( i ) ) );
			} else {
				titleBar.addSeparator();
			}
		}

	}
	
}
 
