package com.japisoft.framework.job;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class HeavyJobProxy extends BasicJob implements HeavyJob {

	private HeavyJob job;

	/** If the job implements the interface SilenceJob, no notification will be
	 * done
	 */
	public HeavyJobProxy( HeavyJob job ) {
		this.job = job;
	}

	public String getName() {
		return job.getName();
	}

	public boolean isAlone() {
		return job.isAlone();
	}
	
	public String getErrorMessage() {
		return job.getErrorMessage();
	}

	public void stopIt() {
		job.stopIt();
	}
	
	public Object getSource() {
		return job.getSource();
	}
	
	public void dispose() {
		job.dispose();
	}
	
	public void run() {
		try {
			if ( !( job instanceof SilenceJob ) )
				JobManager.COMMON_MANAGER.notifyStartJob( job, true );
			job.run();
		} finally {
			if ( !( job instanceof SilenceJob ) )
				JobManager.COMMON_MANAGER.notifyStopJob( job, true );
			job.dispose();
		}
	}
	
}
