package com.japisoft.framework.spreadsheet;

import java.io.InputStream;
import java.io.OutputStream;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XlsSpreadsheet extends AbstractSpreadsheet {

	protected void read( Workbook wb ) {
		Sheet sh = wb.getSheetAt( 0 );
		for ( int i = 0; i <= sh.getLastRowNum(); i++ ) {
			Row row = sh.getRow( i );
			String[] content = new String[ row.getLastCellNum() - row.getFirstCellNum() ];
			int col = 0;
			for ( int j = row.getFirstCellNum(); j< row.getLastCellNum(); j++ ) {
				Cell c = row.getCell( j );
				String value = null;
				if ( c.getCellType() == Cell.CELL_TYPE_NUMERIC ) {
					value = Double.toString( c.getNumericCellValue() );
				} else
				if ( c.getCellType() == Cell.CELL_TYPE_BOOLEAN ) {
					value = Boolean.toString( c.getBooleanCellValue() );
				} else
				if ( c.getCellType() == Cell.CELL_TYPE_FORMULA ) {
					value = c.getCellFormula();
				} else
					value = c.getStringCellValue();
				content[ col++ ] = value;
			}
			readLine( content );
		}
	}

	protected void write( Workbook wb ) {
		Sheet sh = wb.createSheet();
		Row header = sh.createRow( 0 );	// header
		for ( int i = 0; i < getColumnCount(); i++ ) {
			Cell cell = header.createCell( i );
			cell.setCellValue( getColumnName( i ) );
		}
		for ( int j = 0; j < getRowCount(); j++ ) {
			Row r = sh.createRow( j + 1 );
			for ( int i = 0; i < getColumnCount(); i++ ) {
				Cell cell = r.createCell( i );
				cell.setCellValue( getValueAt( j, i ) );
			}
		}
	}

	public void write( OutputStream output ) throws Exception {
		HSSFWorkbook wb = null;
		write( wb = new HSSFWorkbook() );
		wb.write( output );
	}
	
	public void read( InputStream input ) throws Exception {
		HSSFWorkbook wb = new HSSFWorkbook( input );
		read( wb );
	}

}
