package com.japisoft.framework.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Polygon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JToolBar;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class MultiChoiceButton extends JButton implements ActionListener {

	public MultiChoiceButton( Action a ) {
		super( a );
		setBorderPainted( false );
	}
	
	public void addNotify() {
		super.addNotify();
		addActionListener( this );
	}
	
	public void removeNotify() {
		super.removeNotify();
		removeActionListener( this );
	}

	private Insets finalInsets = null;
	
	public Insets getInsets() {
		if ( finalInsets != null )
			return finalInsets;
		Insets insets = super.getInsets();
		insets.right = insets.right + ICON.getIconWidth();
		return ( finalInsets = insets );
	}
	
	private void showPopup( JComponent component, Action a ) {
		JPopupMenu popup = new JPopupMenu();
		for ( int i = 1; i <= 20; i++ ) {
			if ( a.getValue( "label" + i ) != null ) {
				String lbl = ( String )a.getValue( "label" + i );
				String cmd = ( String )a.getValue( "cmd" + i );

				JMenuItem item = new JMenuItem( a );
				item.setText( lbl );
				item.setActionCommand( cmd );
				
				if ( a.getValue( "icon" + i ) != null )
					item.setIcon( ( Icon )a.getValue( "icon" + i ) );
				
				popup.add( item );
			} else
				break;
		}
		popup.show( component, 0, component.getHeight() );
	}
	
	public void paintComponent( Graphics gc ) {
		super.paintComponent( gc );
		ICON.paintIcon( this, gc, 0, 0 );
	}
			
	static EmptyIcon ICON = new EmptyIcon();
	
	/** Empty icon for alignment on popup */
	public static class EmptyIcon extends Object implements Icon {
		private final int height;
		private final int width;
		
		public EmptyIcon() {
			height = 10;
			width = 10;
		}

		public int getIconHeight() {
			return height;
		}

		public int getIconWidth() {
			return width;
		}

		private Polygon p;
		
		public void paintIcon(Component c, Graphics g, int x, int y) {
			g.setColor( Color.BLACK );
			if ( p == null ) {
				p = new Polygon(
						new int[] { 
								1, 1 + ( width / 2 ), 1 + width
						},
						new int[] {
								height / 4, ( 3 * height ) / 4, height / 4
						}, 3 );
				p.translate( x + c.getWidth() - width - 4, y + ( c.getHeight() - height ) / 2 );
			} 
			g.fillPolygon( p );
		}
	}
	
	public void actionPerformed( ActionEvent e ) {
		showPopup( this, getAction() );
	}
	
	public static void main( String[] args ) {
		AbstractAction a = new AbstractAction() {
			public void actionPerformed( ActionEvent e ) {}
		};
		a.putValue( Action.NAME, "TEST" );
		
		a.putValue( "label1", "Test 1" );
		a.putValue( "label2", "Test 2" );
		a.putValue( "cmd1", "CMD 1" );
		a.putValue( "cmd2", "CMD 2" );
		
		JFrame f = new JFrame();
		JToolBar t = new JToolBar();
		t.add( new MultiChoiceButton( a ) );
		f.getContentPane().add( t, BorderLayout.NORTH );
		f.setSize( 100, 100 );
		f.setVisible( true );
	}
	
}
