package com.japisoft.framework.ui.table;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

import com.japisoft.framework.ApplicationModel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class StringTableCellRenderer implements TableCellRenderer {

	private static JLabel lblCommon = new JLabel() {
		public void invalidate() {}
		public void validate() {}
		public boolean isDoubleBuffered() { return false; }
	};

	private JLabel lbl = lblCommon;

	/** Status usage */
	public StringTableCellRenderer() {
		this( lblCommon );
	
	}
	
	public void setFontSize( int size ) {
		lbl.setFont( new Font( Font.SANS_SERIF, Font.PLAIN, size ) );	
	}
	
	/** A simple table renderer working with a label */
	public StringTableCellRenderer( JLabel lbl ) {
		this.lbl = lbl;
		lbl.setOpaque( true );
		background = ( Color )ApplicationModel.getSharedProperty( "table.background.odd.color" );
		alternedBackground = ( Color )ApplicationModel.getSharedProperty( "table.background.even.color" );
		foreground = ( Color )ApplicationModel.getSharedProperty( "table.foreground.color" );
		alterned = ( alternedBackground != null );
	}

	public static void fillIt( JTable t ) {
		fillIt( t, null );
	}
	
	public static void fillIt( JTable t, boolean ... bolds ) {
		StringTableCellRenderer renderer = new StringTableCellRenderer();
		for ( int i = 0; i < t.getColumnCount(); i++ ) {
			StringTableCellRenderer tmpRenderer = renderer;
			if ( bolds != null && bolds[ i ] ) {
				tmpRenderer = new StringTableCellRenderer(   
					new JLabel() {
						public void invalidate() {}
						public void validate() {}
						public boolean isDoubleBuffered() { return false; }
					}
				);
				tmpRenderer.lbl.setForeground( Color.DARK_GRAY );
				tmpRenderer.lbl.setFont(
						tmpRenderer.lbl.getFont().deriveFont(
								Font.BOLD )
				);
			}
			t.getColumnModel().getColumn( i ).setCellRenderer( tmpRenderer );
		}
	}

	private Color background,foreground,alternedBackground;
	private boolean alterned;

	/** A simple table renderer working with a label
	 * @param background Force a column background
	 * @param alternedBackgroundColor for creating a light line difference of background. It can be null
	 * @param foreground Force a column foreground
	 */
	public StringTableCellRenderer( 
			Color background, 
			Color alternedBackgroundColor, 
			Color foreground ) {
		this();
		this.background = background;
		this.foreground = foreground;
		this.alternedBackground = alternedBackgroundColor;
		alterned = alternedBackgroundColor != null;
	}

	public Component getTableCellRendererComponent(
			JTable table, 
			Object value,
			boolean isSelected, 
			boolean hasFocus, 
			int row, 
			int column ) {

		if ( isSelected ) {
			lbl.setBackground( table.getSelectionBackground() );
			lbl.setForeground( table.getSelectionForeground() );
		} else {

			if ( background != null ) {
				if ( alterned ) {
					if ( row % 2 == 0 )
						lbl.setBackground( alternedBackground );
					else
						lbl.setBackground( background );
				} else
				lbl.setBackground( background );
			} else
				lbl.setBackground( table.getBackground() );

			lbl.setForeground( foreground == null ? table.getForeground() : foreground ); 
		}

		if ( table.getModel() instanceof EnableTableModel ) {
			boolean enabled =
				( ( EnableTableModel )table.getModel() ).isEnabled( row );
			if ( !enabled ) {
				if ( !isSelected ) {
					lbl.setBackground( SharedProperties.DISABLED_BGCOLOR );
					lbl.setForeground( SharedProperties.DISABLED_FGCOLOR );
				}
			}
		}

		if ( table.getModel() instanceof ErrorTableModel ) {
			boolean error = 
				( ( ErrorTableModel )table.getModel() ).hasError( row );
			if ( error ) {
				if ( !isSelected )
					lbl.setBackground( SharedProperties.ERROR_BGCOLOR );
			}
		}

		if ( value != null )
			lbl.setText( value.toString() );
		else
			lbl.setText( "" );
		return lbl;
	}	

}
