package com.japisoft.framework.wizard;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public interface WizardStepContext {

	/** @return the main component, user can access to other wizard step using the <code>WizardStepModel</code> */
	public JWizard getSource();

	/** @return the previous step or <code>null</code> for the first wizard step */
	public WizardStep getPreviousWizardStep();

	/** @return the next step or <code>null</code> for the last wizard step */
	public WizardStep getNextWizardStep();

	/** Force the wizard to go the following step */
	public void goTo( WizardStep step );
	
	/** Notify the main wizard component that the next step is accessible. By default the next
		step is never accessible without calling acceptNext( true ) */
	public void acceptNext( boolean accept );

	/** Notify the main wizard component that the previous step is accessible. By default the
		previous step is always accessible */ 
	public void acceptPrevious( boolean accept );

	/** Store a data, this value will be available for all the wizard next step and user can retreive
		these values on the JWizard component */
	public void setSharedData( String key, Object data );

	/** Get a value. This value has been set by previous wizard step */
	public Object getSharedData( String key );

	/** Get a value. This value has been set by previous wizard step
	 * @param key the Shared data key
	 * @param defaultValue the value returned if the key is unknown
	 *  */
	public Object getSharedData( String key, Object defaultValue );

}
