package com.japisoft.framework.xml;

import java.io.FileInputStream;
import java.io.StringReader;
import java.lang.reflect.Array;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.xalan.templates.OutputProperties;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import com.japisoft.framework.xml.format.Formatter;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DOMToolkit {

	static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
	static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";
	static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";
	
	public static Element getFirstElement( 
			Element parent, 
			String matching ) {
		NodeList nl = parent.getChildNodes();
		for ( int i = 0; i < nl.getLength(); i++ ) {
			if ( nl.item( i ) instanceof Element ) {
				if ( matching.equals( ( ( Element )nl.item( i ) ).getLocalName() ) ) {
					return ( Element )nl.item( i );
				}
			}
		}
		return null;
	}

	public static Document parse(
			String content, 
			String location ) throws Exception {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();
		InputSource is = new InputSource( new StringReader( content ) );
		if ( location != null ) {
			is.setSystemId( location );
		}
		return db.parse( is );
	}
	
	public static Document parse(
		boolean lightMode,
		String location ) throws Exception {
		
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		
		if ( lightMode ) {
			dbf.setNamespaceAware( true );	
			dbf.setXIncludeAware( false );
			dbf.setValidating( false );
			dbf.setFeature( "http://xml.org/sax/features/validation", false );
			dbf.setFeature( "http://apache.org/xml/features/nonvalidating/load-external-dtd", false );
		}
		
		javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();
		InputSource is = new InputSource( new FileInputStream( location ) );
		if ( location != null ) {
			is.setSystemId( location );
		}
		return db.parse( is );
	}
	
	public static String DOM2String( 
			Node n, 
			int indentSize,
			boolean autoProlog ) throws Exception {
		String tmp = Formatter.format( n );
		if ( autoProlog && !tmp.contains( "<?xml" ) ) {
			tmp = "<?xml version ='1.0'?>\n\n" + tmp;
		}
		return tmp;
	}
	
	public static void save( String filePath, Node node ) throws Exception {
		Transformer transformer = TransformerFactory.newInstance().newTransformer();
		transformer.setOutputProperty( OutputKeys.INDENT, "yes" );
		transformer.transform( new DOMSource( node ), new StreamResult( filePath ) );
	}
	
	public static Node[] toArray( NodeList nl ) {
		ArrayList<Node> nodes = new ArrayList<Node>();
		for ( int i = 0; i < nl.getLength(); i++ )
			nodes.add( nl.item( i ) );
		return nodes.toArray( new Node[ nodes.size() ] );
	}
	
}

