package com.japisoft.framework.xml.dtd.instance;

import com.japisoft.dtdparser.node.AttributeDTDNode;
import com.japisoft.dtdparser.node.ElementDTDNode;
import com.japisoft.dtdparser.node.ElementRefDTDNode;
import com.japisoft.dtdparser.node.ElementSetDTDNode;
import com.japisoft.dtdparser.node.RootDTDNode;
import com.japisoft.dtdparser.node.DTDNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DTDBuildInstance {

	public void buildElement( StringBuffer res, String elementName, RootDTDNode root ) {
		for ( int i = 0; i < root.getChildCount(); i++ ) {
			DTDNode node = root.getDTDNodeAt( i );
			if ( node.isElement() ) {
				ElementDTDNode element = ( ElementDTDNode )node;
				if ( elementName.equals( element.getName() ) ) {
					buildElement( res, element, root );
					break;
				}
			}
		}
	}

	private void buildNode( StringBuffer res, DTDNode node, RootDTDNode root ) {
		if ( node.isElement() )
			buildElement( res, ( ElementDTDNode )node, root );
		else
		if ( node.isElementRef() ) {
			ElementRefDTDNode ref = ( ElementRefDTDNode )node;
			if ( ref.hasPCDATA() ) {
				res.append( "VALUE" );
			} else {
				if ( ref.getOperator() == ElementDTDNode.ZERO_MORE_ITEM_OPERATOR )
					return;
				if ( ref.getOperator() == ElementDTDNode.ZERO_ONE_ITEM_OPERATOR )
					return;
				if ( ref.getReferenceNode() != null )
					buildElement( res, ref.getReferenceNode(), root );
			}
		} else
		if ( node.isElementSet() )
			buildElementSet( res, ( ElementSetDTDNode )node, root );
	}

	private void buildElement( StringBuffer res, ElementDTDNode element, RootDTDNode root ) {
		if ( element.getOperator() == ElementDTDNode.ZERO_MORE_ITEM_OPERATOR )
			return;
		if ( element.getOperator() == ElementDTDNode.ZERO_ONE_ITEM_OPERATOR )
			return;
		
		res.append( "<" ).append( element.getName() );
		for ( int i = 0; i < element.getChildCount(); i++ ) {
			DTDNode node = element.getDTDNodeAt( i );
			if ( node.isAttribute() ) {
				AttributeDTDNode att = ( AttributeDTDNode )node;
				buildAttribute( res, att );
			}
		}
		// Look at the content
		if ( element.isEmpty() ) {
			res.append( "/>" );
		} else {
			res.append( ">" );

			for ( int i = 0; i < element.getChildCount(); i++ ) {
				DTDNode node = element.getDTDNodeAt( i );
				if ( node.isElement() ) {					
					ElementDTDNode e = ( ElementDTDNode )node;
					if ( e.hasPCDATA() ) {
						res.append( "VALUE" );
					}

					if ( e.getOperator() == 
						ElementDTDNode.ZERO_MORE_ITEM_OPERATOR ||
							e.getOperator() ==
								ElementDTDNode.ZERO_ONE_ITEM_OPERATOR )
						continue;
					buildElement( res, e, root );
				} else
					buildNode( res, node, root );
			}

			res.append( "</" ).append( element.getName() ).append( ">" );
		}
	}

	private void buildElementSet( StringBuffer res, ElementSetDTDNode set, RootDTDNode root ) {
		if ( set.getType() == ElementSetDTDNode.SEQUENCE_TYPE ) {
			for ( int i = 0; i < set.getChildCount(); i++ ) {
				buildNode( res, set.getDTDNodeAt( i ), root );
			}
		} else {
			if ( set.hasPCDATA() ) {
				res.append( "VALUE" );
			} else
				if ( set.getType() == ElementSetDTDNode.CHOICE_TYPE ) {
					for ( int i = 0; i < set.getChildCount(); i++ ) {
						DTDNode node = set.getDTDNodeAt( i );
						if ( node.isElementSet() ) {
							ElementSetDTDNode set2 = ( ElementSetDTDNode )node;
							if ( set2.getOperator() == ElementSetDTDNode.ZERO_MORE_ITEM_OPERATOR ||
									set2.getOperator() == ElementSetDTDNode.ZERO_ONE_ITEM_OPERATOR ) {								
							} else {
								// Good one
								buildElementSet( res, set2, root );
								break;
							}
						} else {
							if ( node.isElementRef() ) {
								ElementRefDTDNode ref2 = ( ElementRefDTDNode )node;
								if ( ref2.getOperator() == ElementSetDTDNode.ZERO_MORE_ITEM_OPERATOR ||
										ref2.getOperator() == ElementSetDTDNode.ZERO_ONE_ITEM_OPERATOR ) {								
								} else {
									// Good one
									buildNode( res, node, root );
									break;
								}								
							}
						}
					}
				}
		}
	}

	private void buildAttribute( StringBuffer res, AttributeDTDNode att ) {
		if ( att.getUsage() == AttributeDTDNode.REQUIRED_ATT )
			res.append( " " ).append( att.getName() ).append( "=\"VALUE\"" );
	}

}
