package com.japisoft.framework.xml.grammar.xsd;

import org.w3c.dom.Element;

import com.japisoft.framework.xml.grammar.GrammarAttribute;
import com.japisoft.framework.xml.grammar.GrammarText;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XSDGrammarAttribute extends XSDGrammarNode implements GrammarAttribute {

	public XSDGrammarAttribute( XSDGrammar grammar, Element attributeDef ) {
		super( grammar, attributeDef );
		minOccurs = 0;	// Optional by default
		if ( "required".equalsIgnoreCase(
				attributeDef.getAttribute( "use" ) ) ) {
			minOccurs = 1;
		}
		if ( attributeDef.hasAttribute( "default" ) ) {
			defaultValue = new XSDGrammarText(
				grammar,
				attributeDef.getAttribute( "default" ) 
			);
		}
	}

	public boolean isRequired() {
		return ( minOccurs > 0 );
	}	

	public String infer() {
		return getName() + "=\"" + ( getDefaultValue() == null ? getType().infer() : getDefaultValue() ) + "\"";
	}	
	
	@Override
	public String getName() {
		Element e = getDOMElement();
		if ( e.hasAttribute( "name" ) )
			return e.getAttribute( "name" );
		else
			if ( e.hasAttribute( "ref" ) )
				return e.getAttribute( "ref" );
			else
				return "?";
	}

	private GrammarText defaultValue = null;

	public GrammarText getDefaultValue() {
		return defaultValue;
	}

}
