package com.japisoft.framework.xml.grammar.xsd;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;

import com.japisoft.framework.xml.grammar.GrammarAttribute;
import com.japisoft.framework.xml.grammar.GrammarElement;
import com.japisoft.framework.xml.grammar.GrammarNode;
import com.japisoft.framework.xml.grammar.GrammarType;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XSDGrammarElement extends XSDGrammarNode 
		implements GrammarElement {

	public XSDGrammarElement( 
		XSDGrammar grammar, 
		Element node ) {
		super( grammar, node );
	}

	public boolean isEmpty() {
		GrammarType type = getType();
		return !type.isComplex();
	}

	public String infer() {
		StringBuffer sb = new StringBuffer();
		sb.append( "<" );
		sb.append( getName() );

		List<GrammarAttribute> l = getAttributes( true );
		if ( l != null ) {
			if ( l.size() > 0 ) {		
				sb.append( " " );
				for ( GrammarAttribute ga : l ) {
					sb.append( ga.infer() );
				}
			}
		}
		if ( isEmpty() ) {
			sb.append( "/>" );
		} else {
			sb.append( ">" );
			sb.append( "</" ).append( getName() ).append( ">" );
		}

		return sb.toString();
	}

	public List<GrammarAttribute> getAttributes( boolean required ) {
		List<GrammarNode> values = getType().getValues();
		if ( values != null ) {
			List<GrammarAttribute> attributes = new ArrayList<GrammarAttribute>();
			for ( GrammarNode gn : values ) {
				if ( gn instanceof GrammarAttribute ) {
					if ( required && !( ( GrammarAttribute )gn ).isRequired() )
						continue;
					attributes.add( ( GrammarAttribute )gn );
				}
			}
			return attributes;
		}
		return null;
	}

	@Override
	public String getName() {
		Element e = getDOMElement();
		if ( e.hasAttribute( "name" ) )
			return e.getAttribute( "name" );
		else
			if ( e.hasAttribute( "ref" ) )
				return e.getAttribute( "ref" );
			else
				return "?";
	}

}
