package com.japisoft.framework.xml.imp;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.JFileChooser;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.xml.XMLToolkit;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CSVImport extends AbstractAction {

	public void actionPerformed(ActionEvent e) {
		JFileChooser jfc = new JFileChooser();
		CSVPanel confImport = new CSVPanel();
		if (jfc.showOpenDialog(ApplicationModel.MAIN_FRAME) == JFileChooser.APPROVE_OPTION) {
			File file = jfc.getSelectedFile();
			if (DialogManager.showDialog(ApplicationModel.MAIN_FRAME, "CVS Import",
					"CVS Import", "Import a CSV document to XML", null,
					confImport, new Dimension(400, 550)) == DialogManager.OK_ID) {
				try {
					impCSV(file, confImport);
				} catch( IOException exc ) {
				}
			}
		}
	}

	public static String impCSV(File dataFile, CSVImportParams config) throws IOException {
		StringBuffer sb = new StringBuffer();
		sb.append("<?xml version=\"1.0\"?>\n\n<document>\n");

		char sep = ',';
		if (config.isSemiColonSelected())
			sep = ';';
		else if (config.isSpaceSelected())
			sep = ' ';
		else if (config.isTabSelected())
			sep = '\t';
		else if (config.isOtherSelected()) {
			if (config.getOther() != null
					&& config.getOther().length() > 0)
				sep = config.getOther().charAt(0);
		}

		char chrDelim = 0;
		if (config.getTextQualifier() != null)
			chrDelim = ((String) config.getTextQualifier())
					.charAt(0);

		String row = "row";
		String col = "col";

		if (config.getRowName() != null
				&& config.getRowName().length() > 0)
			row = config.getRowName();

		try {

			BufferedReader br = new BufferedReader(new FileReader(dataFile));

			try {

				String line = null;
				int startingRow = config.getStartingRow();
				int rowNumber = 1;

				while ( ( line = br.readLine() ) != null ) {
					startingRow--;
					if (startingRow <= 0) {
						// Process it
						sb.append("\t<" + row + " id=\"" + rowNumber + "\">\n");

						int i = 0;
						int c = 0;
						StringBuffer sbCol = null;
						boolean inQualifiedString = false;
						while ( i < line.length() ) {
							char ch = line.charAt( i );
							if (sbCol == null) {
								sbCol = new StringBuffer();
							}
							if (ch == sep && !inQualifiedString) {

								String columnName = "" + ( c + 1 );
								if (config.getColumnName().getSize() > c) {
									columnName = ( String ) config.getColumnName().getElementAt( c );
								}

								// Add a new colum
								sb.append("\t\t<" + col + " id=\"" + columnName
										+ "\">");
								sb.append( XMLToolkit.resolveCharEntities( sbCol.toString() ) );
								sb.append("</" + col + ">\n");

								sbCol = null;
								c++;
							} else {

								if (chrDelim != 0) {

									if (ch == chrDelim) {
										inQualifiedString = !inQualifiedString;
									} else
										sbCol.append(ch);

								} else
									sbCol.append(ch);
							}
							i++;
						}

						if ( sbCol != null ) {
							String columnName = "" + ( c + 1 );
							if (config.getColumnName().getSize() > c) {
								columnName = (String) config.getColumnName().getElementAt( c );
							}							
							sb.append("\t\t<" + col + " id=\"" + columnName
									+ "\">");
							sb.append( XMLToolkit.resolveCharEntities( sbCol.toString() ) );
							sb.append("</" + col + ">\n");							
						}
						
						sb.append("\t</" + row + ">\n");
						
					}
					rowNumber++;
				}

				sb.append("\n</document>");

				return sb.toString();
				
			} finally {
				br.close();
			}

		} catch (FileNotFoundException exc) {
			return null;
		}
	}

}
