package com.japisoft.framework.xml.parser;

import java.io.*;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
final class FastByteArrayInputStream extends InputStream {
    byte buf[];
    int pos;
    int mark = 0;
    int count;

    public FastByteArrayInputStream(byte buf[]) {
	this.buf = buf;
        this.pos = 0;
	this.count = buf.length;
    }

    public int read() {
	return (pos < count) ? (buf[pos++] & 0xff) : -1;
    }

    public  int read(byte b[], int off, int len) {
	if (pos >= count) {
	    return -1;
	}
	if (pos + len > count) {
	    len = count - pos;
	}
	if (len <= 0) {
	    return 0;
	}
	System.arraycopy(buf, pos, b, off, len);
	pos += len;
	return len;
    }

    public  long skip(long n) {
	if (pos + n > count) {
	    n = count - pos;
	}
	if (n < 0) {
	    return 0;
	}
	pos += n;
	return n;
    }

    public  int available() {
	return count - pos;
    }

    public boolean markSupported() {
	return true;
    }

    public void mark(int readAheadLimit) {
	mark = pos;
    }

    public  void reset() {
	pos = mark;
    }

    public void close() throws IOException {
    }

}

// FastByteArrayInputStream ends here
