package com.japisoft.framework.xml.parser.dom;

import com.japisoft.framework.xml.parser.node.*;

import org.w3c.dom.*;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class AttrImpl extends NodeImpl implements Attr {
	private FPNode ref;
	private String name;

	/** @param name Attribute name
	@param ref Reference node */
	public AttrImpl(String name, FPNode ref) {
		super(-1, ref != null ? ref.getAttribute(name) : null );
		this.name = name;
		this.ref = ref;
	}

	public int getStartingLine() {
		return ref.getStartingLine();
	}

	public int getStartingOffset() {
		return ref.getStartingOffset();
	}
	
	/**
	 * Returns the name of this attribute. 
	 */
	public String getName() {
		return name;
	}

	public String getLocalName() {
		return name;
	}

	public String getNodeName() {
		return name;	
	}

	/**
	 * If this attribute was explicitly given a value in the original document, 
	 * this is <code>true</code>; otherwise, it is <code>false</code>. Note 
	 * that the implementation is in charge of this attribute, not the user. If 
	 * the user changes the value of the attribute (even if it ends up having 
	 * the same value as the default value) then the <code>specified</code> 
	 * flag is automatically flipped to <code>true</code>.  To re-specify the 
	 * attribute as the default value from the DTD, the user must delete the 
	 * attribute. The implementation will then make a new attribute available 
	 * with <code>specified</code> set to <code>false</code> and the default 
	 * value (if one exists).
	 * <br>In summary: If the attribute has an assigned value in the document 
	 * then  <code>specified</code> is <code>true</code>, and the value is the  
	 * assigned value. If the attribute has no assigned value in the document 
	 * and has  a default value in the DTD, then <code>specified</code> is 
	 * <code>false</code>,  and the value is the default value in the DTD. If 
	 * the attribute has no assigned value in the document and has  a value of 
	 * #IMPLIED in the DTD, then the  attribute does not appear  in the 
	 * structure model of the document.
	 */
	public boolean getSpecified() {
		return false;
	}

	/**
	 * On retrieval, the value of the attribute is returned as a string. 
	 * Character and general entity references are replaced with their values.
	 * <br>On setting, this creates a <code>Text</code> node with the unparsed 
	 * contents of the string.
	 */
	public String getValue() {
		if ( ref == null )
			return null;
		return ref.getAttribute(name);
	}

	public void setValue(String value) {
		ref.setAttribute(name, value);
	}

	public Element getOwnerElement() {
		return (Element) ref;
	}
	
	public void setNodeValue( String value ) {
		if ( ref != null )
			setValue( value );
	}

	public String getNodeValue() {
		if ( ref != null )
			return getValue();
		return null;
	}

	public boolean isId() {
		return false;
	}

	public TypeInfo getSchemaTypeInfo() {
		return null;
	}

	public short compareDocumentPosition(Node other) throws DOMException {
		return 0;
	}

	public String getBaseURI() {
		return null;
	}

	public Object getFeature(String feature, String version) {
		return null;
	}

	public String getTextContent() throws DOMException {
		return null;
	}

	public Object getUserData(String key) {
		return null;
	}

	public boolean isDefaultNamespace(String namespaceURI) {
		return false;
	}

	public boolean isEqualNode(Node arg) {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean isSameNode(Node other) {
		// TODO Auto-generated method stub
		return false;
	}

	public String lookupNamespaceURI(String prefix) {
		// TODO Auto-generated method stub
		return null;
	}

	public String lookupPrefix(String namespaceURI) {
		// TODO Auto-generated method stub
		return null;
	}

	public void setTextContent(String textContent) throws DOMException {
		// TODO Auto-generated method stub
		
	}

	public Object setUserData(String key, Object data, UserDataHandler handler) {
		// TODO Auto-generated method stub
		return null;
	}	
	
}

// AttrImpl ends here
