package com.japisoft.framework.xml.parser.dom;

import com.japisoft.framework.xml.parser.node.*;

import org.w3c.dom.*;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TextImpl extends CharacterDataImpl implements Text {
	public TextImpl(String content) {
		super(FPNode.TEXT_NODE, content);
	}

	/**
	 * Breaks this <code>Text</code> node into two Text nodes at the specified 
	 * offset, keeping both in the tree as siblings. This node then only 
	 * contains all the content up to the <code>offset</code> point. And a new 
	 * <code>Text</code> node, which is inserted as the next sibling of  this 
	 * node, contains all the content at and after the <code>offset</code> 
	 * point.
	 * @param offset The offset at which to split, starting from 0.
	 * @return The new <code>Text</code> node.
	 * @exception DOMException
	 *   INDEX_SIZE_ERR: Raised if the specified offset is negative or greater 
	 *   than the number of characters in <code>data</code>.
	 *   <br>NO_MODIFICATION_ALLOWED_ERR: Raised if this node is readonly.
	 */
	public Text splitText(int offset) throws DOMException {
		if (getContent() == null)
			throw new DOMExceptionImpl(
				DOMException.INVALID_CHARACTER_ERR,
				"No content");
		if (offset >= getContent().length())
			throw new DOMExceptionImpl(
				DOMException.INDEX_SIZE_ERR,
				"Index " + offset + " errors ");
		String s = getContent();
		Text t = new TextImpl(s.substring(0, offset));
		setContent(s.substring(offset));
		return (Text) insertBefore(t, this);
	}

	public String getWholeText() {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean isElementContentWhitespace() {
		// TODO Auto-generated method stub
		return false;
	}

	public Text replaceWholeText(String content) throws DOMException {
		// TODO Auto-generated method stub
		return null;
	}

}

// TextImpl ends here
