package com.japisoft.framework.xml.parser.walker;

import com.japisoft.framework.xml.parser.node.*;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class AttributeCriteria extends AbstractCriteria {
	private String attribute;
	private String value;

	/** Any attribute name */
	public static String ANY_ATTRIBUTE = "*";

	/** Match the following attribute */
	public AttributeCriteria( String attribute ) {
		super();
		this.attribute = attribute;
		if (attribute == null)
			throw new RuntimeException( "Invalid criteria attribute" );
	}

	/** Match the following attribute and value. If you want to ignore the attribute name,
	 * use the ANY_ATTRIBUTE name */
	public AttributeCriteria( String attribute, String value ) {
		super();
		this.attribute = attribute;
		this.value = value;
		ignoreAttribute = ( ANY_ATTRIBUTE.equals( attribute ) );
		if (attribute == null || value == null)
			throw new RuntimeException( "Invalid criteria attribute name & value" );
	}

	private boolean namespace = false;

	public AttributeCriteria( String attribute, String value, boolean namespace ) {
		this( attribute, value );
		this.namespace = namespace;
		if ( namespace ) {
			if ( value != null ) {
				int i = value.indexOf( ":" );
				if ( i > -1 ) {
					try {
						this.value = value.substring( i + 1 );
					} catch( IndexOutOfBoundsException exc ) {
					}
				}
			}
		}
	}

	private boolean ignoreAttribute = false;

	public boolean isValid( FPNode node ) {
		if ( node.isTag() ) {
			if ( !ignoreAttribute ) {
				
				if ( containsMode ) {
					
					for ( int i = 0; i < node.getViewAttributeCount(); i++ ) {
						String att = node.getViewAttributeAt( i );

						if ( !ignoreCase ) {						
							if ( att.contains( attribute ) ) {
								return true;
							}
						} else {
							if ( att.toLowerCase().contains( attribute.toLowerCase() ) )
								return true;
						}
					}
					
				} else {
					
					if ( ignoreCase ) {
						
						for ( int i = 0; i < node.getViewAttributeCount(); i++ ) {
							String att = node.getViewAttributeAt( i );
							if ( att.equalsIgnoreCase( attribute ) ) 
								return true;
						}
						
						return false;
						
					} else
					
						if ( value == null )
						
							return node.hasAttribute( attribute );
					
						else
							
							return value.equals( node.getAttribute( attribute ) );
				
				}

			} else {
				
				// Recherche de la valeur
				
				for ( int i = 0; i < node.getViewAttributeCount(); i++ ) {
					String att = node.getViewAttributeAt( i );
					
					String attValue = node.getAttribute( att );
					
					if ( containsMode ) {
						
						if ( ignoreCase ) {
							
							if ( attValue.toLowerCase().contains( value.toLowerCase() ) )
								return true;
							
						} else {
							
							if ( attValue.contains( value ) )
								return true;
							 
						}
						
					} else {

						if ( ignoreCase ) {
							if ( attValue.equalsIgnoreCase( value ) )
								return true;
						} else {
							if ( attValue.equals( value ) )
								return true;
						}
						
					}
					
				}
			}
			return false;
		} else
				return false;
	}

}

// AttributeCriteria ends here
