package com.japisoft.framework.xml.parser.walker;

import com.japisoft.framework.xml.parser.node.*;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TextCriteria extends AbstractCriteria {
    private String text;

    /** Match all Text node */
    public TextCriteria() {
    	super();
    }

    /** Match text nodes with the following sub-text */
    public TextCriteria( String text ) {
    	super();   
    	this.text = text;
    }
    
    public boolean isValid( FPNode node ) {
    	
    	if ( node.isText() ) {
    		
    		if ( text == null )
    			return true;
    		
    		String content = node.getContent();
    		
    		if ( content == null )
    			return false;
    		
    		if ( ignoreCase ) {

    			if ( containsMode ) {
    				
    				return content.toLowerCase().contains( text.toLowerCase() );
    				
    			} else {
    				
    				return content.equalsIgnoreCase( text );
    				
    			}
    			
    		} else {
    			
    			if ( containsMode ) {

    				return content.contains( text );
    				
    			} else {
    				
    				return text.equals( content );
    				
    			}
    			
    		}
    		
    	} else
    		
    		return false;

    }

}

// NodeNameCriteria ends here
