package com.japisoft.framework.xml.refactor.elements;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import com.japisoft.framework.xml.refactor.RefactorManager;
import com.japisoft.framework.xml.refactor.ui.RefactorTable;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class RefactorModel extends HashMap {
	private boolean relativeRefactoring = false;

	public boolean isRelativeRefactoring() {
		return relativeRefactoring;
	}

	public void setRelativeRefactoring( boolean relativeRefactoring ) {
		this.relativeRefactoring = relativeRefactoring;
	}

	public String getConfig() {
		StringBuffer sb = new StringBuffer();
		Iterator it = keySet().iterator();
		while ( it.hasNext() ) {
			String key = ( String )it.next();
			RefactorTable rt = ( RefactorTable )get( key );
			RefactorAction[] rat = rt.getActions();
			if ( rat != null ) {
				if ( sb.length() > 0 )
					sb.append( "$$$" );
				sb.append( key );
				for ( int i = 0; i < rat.length; i++ ) {
					sb.append( "!!!" );
					sb.append( rat[ i ].toString() );
				}
			}
		}		
		return sb.toString();
	}

	public void setConfig( String config ) {
		if ( config == null )
			return;
		StringTokenizer st = new StringTokenizer( config, "$$$", false );
		while ( st.hasMoreTokens() ) {
			String token = st.nextToken();
			StringTokenizer st2 = new StringTokenizer( token, "!!!", false );
			String key = st2.nextToken();
			ArrayList al = new ArrayList();
			while ( st2.hasMoreTokens() ) {
				String token2 = st2.nextToken();
				RefactorAction ra = new RefactorAction( token2 );
				al.add( ra );
			}
			RefactorAction[] ra = new RefactorAction[ al.size() ];
			for ( int i = 0; i < al.size(); i++ )
				ra[ i ] = ( RefactorAction )al.get( i );
			RefactorTable rt = new RefactorTable( key );
			rt.setActions( ra );
			put( key, rt );
		}
	}

	public RefactorObj[] getRefactorObjs() {
		ArrayList lr = new ArrayList();
		Iterator it = keySet().iterator();

		while ( it.hasNext() ) {
			String key = ( String )it.next();
			RefactorTable rt = ( RefactorTable )get( key );
			RefactorAction[] rat = rt.getActions();
			try {	
				if ( rat != null ) 
					for ( int i = 0; i < rat.length; i++ ) {
						RefactorAction ra = rat[ i ];
						lr.add( 
							RefactorManager.buildRefactorObj( key, ra ) );
					}
			} catch (Throwable e) {
			}
		}

		if ( lr.size() == 0 ) {
			return null;
		} else {
			RefactorObj[] res = new RefactorObj[ lr.size() ];
			for ( int i = 0; i < lr.size(); i++ )
				res[ i ] = ( RefactorObj )lr.get( i );
			return res;
		}		
	}

}
