package com.japisoft.framework.xml.refactor2.elements;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import com.japisoft.framework.xml.refactor2.AbstractRefactor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DeletePrefix extends AbstractRefactor {

	public void startPrefixMapping(String prefix, String uri)
			throws SAXException {
		if (!oldValue.equals(prefix))
			super.startPrefixMapping(prefix, uri);
	}

	public void endPrefixMapping(String prefix) throws SAXException {
		if (!oldValue.equals(prefix))
			super.endPrefixMapping(prefix);
	}

	public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
		
		if ( qName != null ) {
			int i = qName.lastIndexOf( ":" );
			if ( i > -1 ) {
				String tmp = qName.substring( 0, i );
				if ( oldValue.equals( tmp ) )
					qName = qName.substring( i + 1 );
			}
		}

		if ( atts != null && atts.getLength() > 0 ) {
			atts = new AttributesProxy( atts );
		}
		
		super.startElement(uri, localName, qName, atts);
	}

	public void endElement(String uri, String localName, String qName) throws SAXException {

		if ( qName != null ) {
			int i = qName.lastIndexOf( ":" );
			if ( i > -1 ) {
				String tmp = qName.substring( 0, i );
				if ( oldValue.equals( tmp ) )
					qName = qName.substring( i + 1 );
			}
		}

		super.endElement(uri, localName, qName);
	}
	
	// -----------------------------------------------------------------------------------

	class AttributesProxy implements Attributes {

		private Attributes ref;

		AttributesProxy(Attributes ref) {
			this.ref = ref;
		}

		public int getIndex(String uri, String localName) {
			return ref.getIndex(uri, localName);
		}

		public int getIndex(String qName) {
			return ref.getIndex(qName);
		}

		public int getLength() {
			return ref.getLength();
		}

		public String getLocalName(int index) {
			String name = ref.getLocalName(index);
			if (oldValue.equals(name))
				return null;
			else
				return name;
		}

		public String getQName(int index) {
			String name = ref.getQName(index);
			int i = name.indexOf(":");
			if (i > -1) {
				String tmp = name.substring(0, i );				
				if (oldValue.equals(tmp))
					return name.substring( i + 1 );
			}

			return name;
		}

		public String getType(int index) {
			return ref.getType(index);
		}

		public String getType(String uri, String localName) {
			return ref.getType(uri, localName);
		}

		public String getType(String qName) {
			return ref.getType(qName);
		}

		public String getURI(int index) {
			return ref.getURI(index);
		}

		private String prepareValue( String value ) {
			if ( value.startsWith( oldValue + ":" ) ) {
				return value.substring( oldValue.length() + 1 );
			} else
				return value;
		}

		public String getValue(int index) {
			String value = ref.getValue(index);
			return prepareValue( value );
		}

		public String getValue(String uri, String localName) {
			String value = ref.getValue(uri, localName);
			return prepareValue( value );
		}

		public String getValue(String qName) {
			String value = ref.getValue(qName);
			return prepareValue( value );
		}

	}

}
