package com.japisoft.framework.xml.validator;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XSDValidator implements ErrorHandler {

	private Validator validator = null;
	
	public XSDValidator( String schemaURI ) throws Exception {
		
		System.setProperty( 
				"javax.xml.validation.SchemaFactory:" + XMLConstants.W3C_XML_SCHEMA_NS_URI, 
				"org.apache.xerces.jaxp.validation.XMLSchemaFactory" );

		SchemaFactory schemaFactory = SchemaFactory.newInstance(
				XMLConstants.W3C_XML_SCHEMA_NS_URI );

		Schema schema = null;
		if ( schemaURI.indexOf( "://" ) > -1 )
			schema =
				schemaFactory.newSchema( 
					new URL( schemaURI ) );
		else
			schema = 
				schemaFactory.newSchema(
						new File( schemaURI )
				);

		validator = schema.newValidator();
		validator.setErrorHandler( this );
		
	}

	private boolean error = false;
	private String errorMessage = null;
	
	public boolean validate( Document source ) {
		error = false;
		errorMessage = null;
		try {
			listOfErrorNodes = null;
			validator.validate( new DOMSource( source ) );
		} catch (SAXException e) {
			errorMessage = e.getMessage();
			error = true;
		} catch (IOException e) {
			errorMessage = e.getMessage();
			error = true;
		}
		return !error;
	}

	public String getLastErrorMessage() { 
		if ( errorMessage == null )
			return "";
		return errorMessage; }
	
	private ArrayList<ErrorValidationNode> listOfErrorNodes = null;
	public List<ErrorValidationNode> getErrors() { return listOfErrorNodes; }

	public void error(SAXParseException exception) throws SAXException {
		processException( exception );
	}

	private void processException( SAXParseException exception ) throws SAXException {
		if ( listOfErrorNodes == null )
			listOfErrorNodes = 
				new ArrayList<ErrorValidationNode>();
		listOfErrorNodes.add(
				new ErrorValidationNode(
					exception.getMessage(),
					( Node )validator.getProperty(
						"http://apache.org/xml/properties/dom/current-element-node" ) ) );
		error = true;		
	}

	public void fatalError(SAXParseException exception) throws SAXException {
		processException( exception );
		error = true;
	}

	public void warning(SAXParseException exception) throws SAXException {}
	
	public static void main( String[] args ) throws Exception {
		XSDValidator validator = new XSDValidator( "c:/editix/livre.xsd" );
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setNamespaceAware( true );
		DocumentBuilder db = factory.newDocumentBuilder();
		Document doc = db.parse( new File( "C:/Documents and Settings/alex/toto.xml" ) );
		validator.validate( doc );
	}
}
