package com.japisoft.sc;

import java.util.Vector;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
class SyntaxLexer {

	private Token[] t;
	private LineLexer ll;
	private Vector cacheList;

	final int cacheLimit = 30;

	/** Set the token to manage. This token will be parsed once to produce
	 * an optimized tree analyser */
	public void setState(boolean icase, Token[] t) {
		this.t = t;
		cacheList = new Vector();
		ll = new LineLexer();
		ll.setIgnoreCase(icase);
		for (int i = 0; i < t.length; i++) {
			Token t1 = t[i];
			ll.addToken(t1);
		}

		///////////////////////////////////////////////////////////////////////
	}

	boolean toRepaint = false;
	
	boolean mustRepaint() {
		if (  ll == null )
			return false;
		return toRepaint;
	}

	private int tokenCount;
	public int getTokenCount() {
		return tokenCount;
	}

	private char[] startDelimiters;
	private char[] stopDelimiters;

	/** Define a set of starting characters for validating the token */
	public void setSyntaxStart(char[] startDelimiters) {
		this.startDelimiters = startDelimiters;
	}

	/** Define a set of stopping characters for validating the token */
	public void setSyntaxStop(char[] stopDelimiters) {
		this.stopDelimiters = stopDelimiters;
	}

	/** Parse a line into tokens */
	public Token[] getTokenForLine(String str, int line) {
		tokenCount = 1;
		if (ll == null) {
			return new Token[] { new Token(str.toCharArray())};
		} else {
			Token[] t;
			// Cache usage
			int i = cacheList.indexOf(str);
			if (i > -1) {
				return (Token[]) cacheList.get(i + 1);
			} else {
				if (cacheList.size() > cacheLimit) {
					cacheList.remove(0);
					cacheList.remove(0);
				}
				t = ll.getTokensForLine(str, line);

				// Eliminate bad tokens
				if ((startDelimiters != null) || (stopDelimiters != null)) {
					for (int j = 0; j < ll.getTokenSize(); j++) {
				
						Token t1 = t[j];
						if (t1.hasAttributes() && !t1.isIgnoreDelimiter()) {
							boolean valid = true;
							// Validate if for the previous one
							if (startDelimiters != null && j > 0) {
								Token tIni = t[j - 1];
								char[] signature = tIni.getTokenSignature();
								if (signature.length > 0) {
									if (signature[signature.length-1] < startDelimiters.length && startDelimiters[signature[signature.length - 1 ]]
										== Character.MAX_VALUE) {
										// Is not valid
										valid = false;
										Token clone = t1.cloneToken();
										clone.setDefaultToken( true );
										t[ j ] = clone;
									} else
										t1.setDefaultToken(false);
								}
							}
							if (valid
								&& stopDelimiters != null
								&& j < (ll.getTokenSize() - 1)) {
								Token tEnd = t[j + 1];
								char[] signature = tEnd.getTokenSignature();
								if (signature.length > 0) {
									if (signature[signature.length-1] < stopDelimiters.length && stopDelimiters[signature[0]]
										== Character.MAX_VALUE) {
										Token clone = t1.cloneToken();
										clone.setDefaultToken( true );
										t[ j ] = clone;
									} else
										t1.setDefaultToken(false);
								}
							}
						}
					}
				}

				tokenCount = ll.getTokenSize();
				
				toRepaint = ll.mustRepaint();
				
				return t;
			}
		}
	}
}

// SyntaxLexer ends here
