package com.japisoft.stylededitor.ui.node;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;

import com.japisoft.stylededitor.EditorByCSS;
import com.japisoft.stylededitor.model.NodeElement;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TextElementView extends NodeElementView {

	private Dimension getSize( EditorByCSS editor, NodeElement element, String text, Graphics gc ) {
		Font f = getFont( element );
		FontMetrics fm = gc.getFontMetrics( f );
		int height = fm.getHeight();
		int width = 0;
		width = fm.stringWidth( text );
		Dimension dim = new Dimension( width, height );
		element.setData( "size", dim );
		return dim;
	}

	public Dimension getSize(EditorByCSS editor, NodeElement element, Graphics gc) {
		return getSize( editor, element, element.getName(), gc );		
	}

	@Override
	public void paint(EditorByCSS editor, NodeElement element, int x, int y, Graphics gc) {
		String text = element.getName();
		if ( text != null ) {
			Font f = getFont( element );
			FontMetrics fm = gc.getFontMetrics( f );
			gc.setFont( f );
			gc.setColor( ( Color )element.getCSSProperty( "color", Color.BLACK ) );
			( ( Graphics2D )gc ).setRenderingHint( 
				RenderingHints.KEY_TEXT_ANTIALIASING, 
				RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB 
			);
			gc.drawString( text, 0, fm.getAscent() );
			if ( isUnderline( element ) ) {
				gc.drawLine( 0, fm.getAscent(), fm.stringWidth( text ), fm.getAscent() );
			}
		}
	}

	@Override
	public Rectangle getBounds(EditorByCSS editor, NodeElement element, int offset, Graphics gc) {
		Point location = ( Point )( ( NodeElement )element ).getData( "location" );
		if ( location == null )
			return null;	// ?
		offset -= element.getStartOffset();
		Dimension dim = getSize( editor, element, element.getName().substring( 0, Math.min( offset, element.getName().length() ) ), gc );
		return new Rectangle( location, dim );
	}
	
	@Override
	public boolean isInside(EditorByCSS editor, NodeElement e, int x, int y, Graphics gc) {		
		if ( e instanceof NodeElement ) {
			NodeElement ne = ( NodeElement )e;
			Dimension dim = ( Dimension )ne.getData( "size" );
			Point loc = ( Point )ne.getData( "location" );
			if ( ( dim != null ) && 
					( loc != null ) ) {
				if ( x >= loc.x && 
						y >= loc.y &&  
								y <= loc.y + dim.height ) {
					if ( !isBlock( editor,e ) )
						if ( x > loc.x + dim.width )
							return false;
					return true;	
				}
			}
		}		
		return false;
	}

	@Override
	public int getOffset(
			EditorByCSS editor, 
			NodeElement element, 
			int x, 
			int y, 
			Graphics gc ) {
		
		String text = element.getName();
		Point location = ( Point )( ( NodeElement )element ).getData( "location" );
		if ( location == null )
			location = new Point( 0, 0 );
		
		for ( int i = 1; i <= text.length(); i++ ) {
			Dimension dim = getSize( editor, element, text.substring( 0, i ), gc );
			if ( location.x + dim.width >= x )
				return ( i - 1 );
		}

		return element.getEndOffset() - element.getStartOffset();
	}

}
