package com.japisoft.universalbrowser;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;

import javax.swing.tree.TreeNode;

import org.apache.commons.vfs.FileObject;
import org.apache.commons.vfs.FileSystemException;
import org.apache.commons.vfs.FileType;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
class FileObjectTreeNode implements TreeNode, Comparable {

	private FileObject fo;
	
	FileObjectTreeNode( FileObject fo ) {
		this.fo = fo;
	}

	public Enumeration children() {
		prepareChildren();
		return Collections.enumeration( children );
	}

	public boolean getAllowsChildren() {
		return true;
	}
	
	FileType currentType = null;
	
	public boolean isFolder() {
		try {
			if ( currentType != null )
				return currentType == FileType.FOLDER;
			return ( ( currentType = fo.getType() ) == FileType.FOLDER );
		} catch (FileSystemException e) {
			return false;
		}
	}

	public int compareTo(Object arg0) {
		FileObjectTreeNode fotn = ( FileObjectTreeNode )arg0;

		// Directory case

		if ( isFolder() ) {

			if ( fotn.isFolder() ) {
				
				return fo.getName().compareTo( fotn.fo.getName() );

			} else
				return +1;

		} else {
			
			if ( fotn.isFolder() ) {
				return -1;
			} else
				return fo.getName().compareTo( fotn.fo.getName() );
		}

	}	

	private ArrayList children = null;

	private void prepareChildren() {
		if ( children == null ) {
			children = new ArrayList();
			try {
				FileObject[] fos = fo.getChildren();
				if ( fos != null ) {
					for ( int i = 0; i < fos.length; i++ ) {
						children.add( 
								new FileObjectTreeNode( 
										fos[ i ] 
								) 
						);
					}
				}
				Collections.sort( children );
			} catch (FileSystemException e) {
			}				
		}
	}

	public TreeNode getChildAt(int childIndex) {
		prepareChildren();
		return ( TreeNode )children.get( childIndex );
	}

	public int getChildCount() {
		prepareChildren();
		return children.size();
	}

	public int getIndex(TreeNode node) {
		prepareChildren();
		return children.indexOf( node );
	}

	private TreeNode parent = null;
	
	public TreeNode getParent() {
		try {
			if ( parent == null )
				parent = new FileObjectTreeNode( fo.getParent() );
		} catch (FileSystemException e) {
		}
		return parent;
	}

	public boolean isLeaf() {
		prepareChildren();
		return children.size() == 0;
	}

	boolean isRoot() {
		return "/".equals( fo.getName().getPath() );
	}
	
	String cachedName = null;
	
	public String toString() {
		if ( isRoot() ) {
			if ( cachedName == null ) {
				cachedName = fo.getName().getURI();
				int i = cachedName.indexOf( "file:///" );
				int end = cachedName.lastIndexOf( "!" );
				
				if ( i > -1 ) {
					if ( end > -1 )
						cachedName = cachedName.substring( 
							i + 8,
							end );
					else
						cachedName = cachedName.substring( 
								i + 8 );
				}
				
				i = cachedName.lastIndexOf( "/" );
				if ( i > -1 && ( i + 1 < cachedName.length() ) )
					cachedName = cachedName.substring( i + 1 );
				
			} 
			return cachedName;
		}
		return fo.getName().getBaseName();
	}

	FileObject getSource() { return fo; }
}
