package com.japisoft.xflows;

import java.io.File;
import java.io.FileInputStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import com.japisoft.editix.main.steps.ConfigurationApplicationStep;
import com.japisoft.framework.xml.XMLFileData;
import com.japisoft.framework.xml.XMLToolkit;
import com.japisoft.framework.xml.parser.ParseException;
import com.japisoft.framework.xml.parser.FPParser;
import com.japisoft.framework.xml.parser.document.Document;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xflows.task.Task;
import com.japisoft.xflows.task.TaskManager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class RunScenario {

	static {
		// Force static resolution
		try {
			new ConfigurationApplicationStep();
		} catch( Throwable th ) {}
	}

	/**
	 * @param args */
	public static void main(String[] args) {
		if ( args.length == 0 ) {
			System.err.println( "Wrong parameter, no scenario file ?" );
			System.err.println( "RunScenario [Path to a scenario file]" );
			System.exit( 1 );
		} else {
			
			File scenario = new File( args[ 0 ] );
			if ( !scenario.exists() ) {
				System.err.println( "Can't find your scenario file ? => " + scenario );
				System.exit( 1 );
			} else {
				
				XMLFileData xfd = null;
				
				try {
					xfd = XMLToolkit.getContentFromInputStream( 
						new FileInputStream( scenario ), null
					);
				} catch( Throwable exc ) {
					System.err.println( "Can't read your scenario " + scenario + " :" + exc.getMessage() );
					System.exit( 1 );
				}

				FPParser p = new FPParser();
				try {
					Document doc = p.parse( new StringReader( xfd.getContent() ) );					
					List<Task> tasks = new ArrayList<Task>();
					FPNode root = ( FPNode )doc.getRoot();
					for ( int i = 0; i < root.childCount(); i++ ) {
						FPNode taskNode = ( FPNode )root.childAt( i );
						Task t = new Task();
						t.updateFromXML( taskNode );
						tasks.add( t );
					}

					System.out.println( "Running " + tasks.size() + " task(s)..." );
					TaskManager.run( tasks, TaskManager.FOREGROUND );
					System.out.println( "Terminated" );

				} catch( ParseException pe ) {
					System.err.println( "XML error found inside your scenario " + scenario + " : " + pe.getMessage() );
					System.exit( 1 );
				}
			}
			
		}
	}

}
