package com.japisoft.xflows.log;

import java.io.FileWriter;
import java.io.IOException;
import com.japisoft.framework.toolkit.LoggerListener;
import com.japisoft.xflows.LoggerModel;
import com.japisoft.xflows.XFlowsApplicationModel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FileLogger implements LoggerListener {
	private FileWriter logWarning = null;
	private FileWriter logError = null;
	private boolean checkErrorOnce = false;

	private void log( String message, String target ) {
		FileWriter logInfo = null;
		if ( !checkErrorOnce ) {
			try {
				logInfo = 
					new FileWriter(
							target, true );
			} catch( Throwable th ) {
				checkErrorOnce = true;
				System.err.println( "Can't write to " + target );
			}
		}

		if ( logInfo != null ) {
			try {
				try {
					logInfo.write( message );
					logInfo.write( System.getProperty( "line.separator" ) );
				} finally {
					logInfo.close();
					logInfo = null;
				}
			} catch( IOException exc ) {
				System.err.println( "Can't write to " + target );
				checkErrorOnce = true;
			}
		}
	}

	public void addInfo( String message ) {
		LoggerModel model = XFlowsApplicationModel.ACCESSOR.getLogger();
		if ( model.isFileLogInfoEnabled() && model.getFileLogInfo() != null )
			log( message, model.getFileLogInfo() );
	}

	public void addWarning( String message ) {
		LoggerModel model = XFlowsApplicationModel.ACCESSOR.getLogger();
		if ( model.isFileLogWarningEnabled() && model.getFileLogWarning() != null )
			log( message, model.getFileLogWarning() );		
	}

	public void addError( String message ) {
		LoggerModel model = XFlowsApplicationModel.ACCESSOR.getLogger();
		if ( model.isFileLogErrorEnabled() && model.getFileLogError() != null )
			log( message, model.getFileLogError() );				
	}

}
