package com.japisoft.xflows.task;

import java.util.HashMap;
import java.util.Iterator;

import com.japisoft.framework.xml.parser.node.FPNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TaskParams {

	public static final int XSLTPARAMS = 1; 

	private HashMap map = null;
	
	public void setParam( String name, String value ) {
		if ( map == null )
			map = new HashMap();
		if ( value != null )
			map.put( name, new ParamValue( 0, value ) );
		else
			map.remove( name );
	}

	public void setParam( String name, String value, int type ) {
		if ( map == null )
			map = new HashMap();
		if ( value != null )
			map.put( name, new ParamValue( type, value ) );
		else
			map.remove( name );
	}
		
	public String getParamValue( String name ) {
		if ( map == null )
			return null;
		if ( map.containsKey( name ) )
			return ( ( ParamValue )map.get( name ) ).value;
		return null;
	}

	public String getParamValue( String name, String defaultValue ) {
		if ( map == null )
			return defaultValue;
		if ( map.containsKey( name ) )
			return ( ( ParamValue )map.get( name ) ).value;
		return defaultValue;
	}
		
	public boolean getParamValueBoolean( String name ) {
		String v = getParamValue( name );
		return "true".equals( v );
	}

	public int getParamValueInteger( String name ) {
		String v = getParamValue( name );
		if ( v == null )
			return 0;
		try {
			return Integer.parseInt( v );
		} catch( NumberFormatException exc ) {
			return 0;
		}
	}

	public boolean hasParamValue( String name ) {
		if ( map == null )
			return false;
		ParamValue pv = ( ParamValue )map.get( name );
		return pv != null && !"".equals( pv.value );
	}

	public int getParamType( String name ) {
		if ( map == null )
			return 0;
		return ( ( ParamValue )map.get( name ) ).type;
	}

	public Iterator getParams() {
		if ( map == null )
			return new Iterator() {
				public boolean hasNext() {
					return false;
				}
				public Object next() {
					return null;
				}
				public void remove() {
				}
			};
		else
			return map.keySet().iterator();
	}

	public String toString() {
		return "" + map;
	}

	public FPNode toXML() {
		if ( map == null )
			return null;

		FPNode params = new FPNode( FPNode.TAG_NODE, "params" );
		
		for ( Iterator it = getParams(); it.hasNext(); ) {
			String key = ( String )it.next();
			String value = getParamValue( key );
			int type = getParamType( key );
			FPNode paramNode = new FPNode( FPNode.TAG_NODE, "param" );
			paramNode.setAttribute( "key", key );
			paramNode.setAttribute( "value", value );
			paramNode.setAttribute( "type", "" + type );
			params.appendChild( paramNode );
		}
		
		return params;
	}

	public void updateFromXML( FPNode params ) {
		map = new HashMap();
		for ( int i = 0; i < params.childCount(); i++ ) {
			FPNode node = params.childAt( i );
			String name = node.getAttribute( "key", "?" );
			String value = node.getAttribute( "value", "?" );
			int type = Integer.parseInt( node.getAttribute( "type", "0" ) );
			setParam( name, value, type );
		}
	}

	public TaskParams cloneParams() {
		TaskParams params = new TaskParams();
		if ( map != null )
			params.map = ( HashMap )map.clone();
		return params;
	}
	
	//////////////////////////////////////////////////////////////////////////////////////

	class ParamValue {
		public int type;
		public String value;

		public ParamValue( int type, String value ) {
			this.type = type;
			this.value = value;
		}
		
		public String toString() {
			return type + "," + value;
		}
	}

}
