package com.japisoft.xflows.task.concat;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import com.japisoft.xflows.task.TaskContext;
import com.japisoft.xflows.task.TaskRunner;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FileConcatRunner implements TaskRunner {

	
	private FileWriter fileWriter = null;
	
	public void setFileWriter( FileWriter fileWriter ) {
		this.fileWriter = fileWriter;
	}

	public boolean run( TaskContext context ) {

		File source = context.getCurrentSourceFile();
		char[] data = new char[ ( int )source.length() ];

		try {

			FileReader fr = new FileReader( source );
			try {
				fr.read( data );
			} finally {
				fr.close();
			}

			boolean tagFound = false;
			
			// Search for the starting tag
			for ( int i = 0; i < data.length - 1; i++ ) {
				if ( data[ i ] == '<' && !( data[ i + 1 ] == '?' || data[ i + 1 ] == '!' ) ) {
					fileWriter.write( data, i, data.length - i );
					tagFound = true;
					break;
				}
			}
			
			if ( !tagFound )
				context.addWarning( "No tag found" );

		} catch( FileNotFoundException exc ) {
			context.addError( "Can't find " + source );
			return ERROR;
		} catch( IOException exc ) {
			context.addError( "Error while reading " + 
					source + " : " + exc.getMessage() );
			return ERROR;
		}

		return OK;
	}

}
