package com.japisoft.xflows.task.delete;

import java.util.List;
import java.util.regex.PatternSyntaxException;

import com.japisoft.datasource.DataSource;
import com.japisoft.datasource.DataSourceContainer;
import com.japisoft.datasource.DataSourceContainerProxy;
import com.japisoft.datasource.DataSourceItem;
import com.japisoft.datasource.DataSourceManager;
import com.japisoft.xflows.task.TaskContext;
import com.japisoft.xflows.task.TaskRunner;
import com.japisoft.xflows.task.copy.CopyUI;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DeleteRunner implements TaskRunner {

	public boolean run(TaskContext context) {

		if ( !context.hasParam( CopyUI.SOURCEPATH ) ) {
			context.addError( "No source path found" );
			return ERROR;
		}

		DataSourceContainer dsc = DataSourceManager.getDataSourceContainer( context
				.getParam(CopyUI.SOURCEPATH) );

		if (dsc == null)
			return ERROR;

		if ( context.hasParam( CopyUI.SOURCEFILTER ) ) {
			dsc = new DataSourceContainerProxy( dsc, context
					.getParam(CopyUI.SOURCEFILTER));
		}				

		try {
			
			List<DataSource> l = dsc.list();

			for (int i = 0; i < l.size(); i++) {

				DataSource ds = l.get( i );

				if ( ds instanceof DataSourceItem ) {

					((DataSourceItem) ds).delete();
					
				}
				
			}
			
		} catch (PatternSyntaxException exc) {
			context.addError("Wrong source filter "
					+ context.getParam(CopyUI.SOURCEFILTER) + " : "
					+ exc.getMessage());
		} catch( Exception exc ) {
			context.addError( "Can't delete " + exc.getMessage() );
		}
		
		return OK;
	}

}
