package com.japisoft.xflows.task.docbook;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import com.japisoft.framework.preferences.Preferences;
import com.japisoft.xflows.task.BasicTaskContext;
import com.japisoft.xflows.task.TaskContext;
import com.japisoft.xflows.task.TaskElementFactory;
import com.japisoft.xflows.task.TaskRunner;
import com.japisoft.xflows.task.fop.FOPFileRunner;
import com.japisoft.xflows.task.fop.FOPUI;
import com.japisoft.xflows.task.xslt.XSLTFileRunner;
import com.japisoft.xflows.task.xslt.XSLTUI;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DocBookFileRunner implements TaskRunner {

	private static URL checkForResource( String resource ) {
		URL u = ClassLoader.getSystemResource( resource );
		if ( u != null )
			return u;
		File f = new File( resource );
		if ( f.exists() ) {
			try {
				return f.toURL();
			} catch( MalformedURLException exc ) {
			}
		}
		throw new RuntimeException( "Can't find " + resource );
	}

	public boolean run( TaskContext context ) {
		String renderer = context.getParam( DocBookUI.OUTPUT_TYPE );
		File output = context.getCurrentTargetFile();
		
		String xsltFile = "";
		URL url = null;
		boolean foMode = false;

		if ( "HTML".equals( renderer ) ) {
			url = checkForResource( Preferences.getPreference( "docbook", "htmlstylesheet", "docbook/html/docbook.xsl" ) );
			if ( url != null )
				xsltFile = 				
					url.toString();
			else
				return false;			
		} else
		if ( "HTML Help".equals( renderer ) ) {
			url = checkForResource( Preferences.getPreference( "docbook", "htmlHelperstylesheet", "docbook/htmlhelp/htmlhelp.xsl" ) );
			if ( url != null )
				xsltFile = url.toString();
			else
				return false;
		} else
		if ( "XHTML".equals( renderer ) ) {	
			url = checkForResource( Preferences.getPreference( "docbook", "xhtmlstylesheet", "docbook/xhtml/docbook.xsl" ) );
			if ( url != null )
				xsltFile = url.toString();
			else
				return false;
		} else
		if ( "Java Help".equals( renderer ) ) {
			url = checkForResource( Preferences.getPreference( "docbook", "javaHelpstylesheet", "docbook/javahelp/javahelp.xsl" ) );
			if ( url != null )
				xsltFile = url.toString();
			else
				return false;
		} else
		if ( "FO".equals( renderer ) ) {
			
			url = checkForResource( Preferences.getPreference( "docbook", "fostylesheet", "docbook/fo/docbook.xsl" ) );
			if ( url != null )
				xsltFile = url.toString();
			else
				return false;
			
		} else {
			
			url = checkForResource( Preferences.getPreference( "docbook", "fostylesheet", "docbook/fo/docbook.xsl" ) );
			if ( url != null )
				xsltFile = url.toString();
			else
				return false;

			output = new File( "xflows.tmp" );

			// Faire d'abord la conversion en FO
			foMode = true;
		}

		TaskRunner xsltRunner = new XSLTFileRunner();
		TaskContext newContext = new BasicTaskContext( context );
		newContext.setCurrentSourceFile( context.getCurrentSourceFile() );
		newContext.setCurrentTargetFile( output );

		if ( xsltFile.startsWith( "file:" ) )
			xsltFile = xsltFile.substring( 5 );

		newContext.getParams().setParam( XSLTUI.STYLESHEET, xsltFile );
		newContext.setQuietInfoMode( true );

		context.addInfo( "Transforming " + context.getCurrentSourceFile() );

		boolean ok = ( xsltRunner.run( newContext ) == OK );

		if ( foMode && ok ) {

			TaskRunner fopRunner = new FOPFileRunner();
			newContext = new BasicTaskContext( context );
			newContext.setCurrentSourceFile( output );
			newContext.setCurrentTargetFile( context.getCurrentTargetFile() );
			newContext.getParams().setParam( FOPUI.OUTPUT_TYPE, renderer );
			newContext.setQuietInfoMode( true );

			ok = ( fopRunner.run( newContext ) == OK );
			output.delete();
			return ok;

		} else {
			context.addError( "Can't use the following stylesheet " + xsltFile );
			return ERROR;
		}

	}

}
