package com.japisoft.xflows.task.imp.cvs;

import java.util.StringTokenizer;

import javax.swing.DefaultListModel;
import javax.swing.ListModel;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.dialog.BasicOKCancelDialogComponent;
import com.japisoft.framework.xml.imp.CSVImportParams;
import com.japisoft.framework.xml.imp.CSVPanel;
import com.japisoft.xflows.task.TaskParams;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CSVParametersDialog extends BasicOKCancelDialogComponent {

	private TaskParams params;
	private CSVPanel csvParams; 

	public CSVParametersDialog( TaskParams params ) {
		super(
			ApplicationModel.MAIN_FRAME,
			"CSV Option",
			"CSV Parameters",
			"Fix here your CSV format", 
			null );

		this.params = params;
		setUI( csvParams = new CSVPanel() );
		setSize( 400, 550, true );
	}

	protected void beforeClosing() {
		super.beforeClosing();
		if ( isOk() ) {

			CSVImportParams p = ( CSVImportParams )csvParams;
			
			// Store the result

			params.setParam( TAB_KEY, "" + p.isTabSelected() );
			params.setParam( SEMICOL_KEY, "" + p.isSemiColonSelected() );
			params.setParam( COMMA_KEY, "" + p.isCommaSelected() );
			params.setParam( SPACE_KEY, "" + p.isSpaceSelected() );
			params.setParam( OTHER_KEY, "" + p.isOtherSelected() ); 
			params.setParam( TEXTQUALIF_KEY, p.getTextQualifier() );
			params.setParam( STARTINGROW_KEY, "" + p.getStartingRow() );
			
			ListModel model = p.getColumnName();
			StringBuffer sb = new StringBuffer();
			for ( int i = 0; i < model.getSize(); i++ ) {
				if ( i > 0 )
					sb.append( "@@" );
				sb.append( model.getElementAt( i ) );
			}
			
			params.setParam( COLNAME_KEY, sb.toString() ); 
			params.setParam( ROWNAME_KEY, p.getRowName() );
			params.setParam( OTHERVAL_KEY, p.getOther() );
		}
	}

	protected void beforeShowing() {
		super.beforeShowing();
		
		csvParams.setTabSelected(
				params.getParamValueBoolean( TAB_KEY ) );
		
		csvParams.setSemiColonSelected(
				params.getParamValueBoolean( SEMICOL_KEY ) );
		
		csvParams.setCommaSelected(
				params.getParamValueBoolean( COMMA_KEY ) );
		
		csvParams.setSpaceSelected(
				params.getParamValueBoolean( SPACE_KEY ) );
		
		csvParams.setOtherSelected(
				params.getParamValueBoolean( OTHER_KEY ) );

		csvParams.setTextQualifier(
				params.getParamValue( TEXTQUALIF_KEY ) );

		csvParams.setStartingRow(
				params.getParamValueInteger( STARTINGROW_KEY ) );

		String colNameVal = params.getParamValue( COLNAME_KEY );
		if ( colNameVal != null ) {
			DefaultListModel model = new DefaultListModel();
			StringTokenizer st = new StringTokenizer( colNameVal, "@@" );
			while ( st.hasMoreTokens() ) {
				model.addElement( st.nextToken() );
			}
			csvParams.setColumnName( model );
		}

		csvParams.setRowName(
				params.getParamValue( ROWNAME_KEY, "row" ) );

		csvParams.setOther(
				params.getParamValue( OTHERVAL_KEY ) );
	}

	static final String TAB_KEY = "tab";
	static final String SEMICOL_KEY = "semicol";
	static final String COMMA_KEY = "comma";
	static final String SPACE_KEY = "space";
	static final String OTHER_KEY = "other";
	static final String TEXTQUALIF_KEY = "textq";
	static final String STARTINGROW_KEY = "startingrow";
	static final String COLNAME_KEY = "colname";
	static final String ROWNAME_KEY = "rowname";
	static final String OTHERVAL_KEY = "otherval";

}
