package com.japisoft.xflows.task.ui;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;

import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import com.japisoft.editix.action.xsl.result.DocumentTypeModel;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.preferences.Preferences;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XFlowsFactory {

	public static void buildAndShowErrorDialog(String message) {
		EditixFactory.buildAndShowErrorDialog( message );
	}

	public static String buildAndShowInputDialog(String title) {
		return EditixFactory.buildAndShowInputDialog( title, "" );
	}

	public static void buildAndShowInformationDialog(String message) {
		EditixFactory.buildAndShowInformationDialog( message );
	}

	public static int buildAndShowChoiceDialog(String message) {
		return EditixFactory.buildAndShowConfirmDialog( message ) ? 1 : 0;
	}

	/** @return a FileChooser for opening a project file */
	public static JFileChooser buildProjectFileChooser() {
		JFileChooser fc = new JFileChooser();
		fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fc.setFileFilter(new FileFilter() {
			public String getDescription() {
				return "*.xfl (XFlows project file)";
			}
			public boolean accept(File f) {
				return (
					f.isDirectory()
						|| f.toString().toLowerCase().endsWith(".xfl" ) );
			}
		});

		String currentDir = 
			Preferences.getPreference( "defaultpath", "project", ( String )null );

		if ( currentDir != null )
			fc.setCurrentDirectory( new File( currentDir ) );

		return fc;
	}

	public static JComboBox getSourceFilter( String type ) {

		String[] filters = null;

		if ( "jsx".equals( type ) ) {
			filters = new String[] {"(.*).xml"};
		} else
		if ( "all".equals( type ) ) {
			filters = new String[] {
				"(.*)"	
			};
		} else
		if ( "zip".equals( type ) ) {
			filters = new String[] {
				"(.*).zip",
				"(.*).docx",
				"(.*).xlsx",
				"(.*).idml"
			};
		} else
		if ( "csv".equals( type ) ) {
			filters = new String[] {
					"(.*).csv",
					"(.*)\\.(.*)"					
			};
		} else
		if ( "htmlxml".equals( type ) ) {
			filters = new String[] { 
					"(.*).html",
					"(.*).htm",
					"(.*)\\.(.*)"
			};
		} else
			filters = new String[] {
				"(.*).xml",
				"(.*).jso",
				"(.*).json",
				"(.*)\\.(.*)"
			};

		JComboBox combo = new JComboBox(
				filters
		);
		return combo;
	}

	public static JComboBox getTargetName( String type ) {
		String [] filters = null;
		if ( "xml".equals( type ) ) {
			filters = new String[] {
				"$1.xml"
			};
		} else {
			filters = new String[] { 
				"$1.html",
				"$1.htm",
				"$1.xml" };
			
			if ( type != null && type.startsWith( "xsl" ) ) {
				
				ArrayList<String> tout = new ArrayList<String>();
				Collections.addAll( tout, filters );
				
				// Add complex output document like docx...
				for ( int i = 0; i < DocumentTypeModel.instance().size(); i++ ) {
					tout.add( "$1." + DocumentTypeModel.instance().fileExt( i ) );
				}
				
				filters = tout.toArray( new String[ tout.size() ] );
			}
			
		}
		JComboBox combo = new JComboBox( filters );
		return combo;
	}

	public static ImageIcon getImageIcon(String resource) {
		URL url = ClassLoader.getSystemClassLoader().getResource(resource);
		if (url != null)
			return new ImageIcon(url);
		else
			com.japisoft.framework.toolkit.Logger.addWarning("Can't find " + resource);
		return null;
	}
	
}
