package com.japisoft.xmlform.component;

import java.awt.Component;
import java.awt.FocusTraversalPolicy;
import java.awt.Rectangle;

import javax.swing.FocusManager;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CustomFocusManager extends FocusManager {

	private AbstractXMLFormComponent getAncestor( Component c ) {
		if ( c == null )
			return null;		
		if ( c instanceof AbstractXMLFormComponent )
			return ( AbstractXMLFormComponent )c;
		return getAncestor( c.getParent() );
	}

	private JScrollPane getScrollPaneAncestor( Component c ) {
		if ( c == null )
			return null;		
		if ( c instanceof JScrollPane )
			return ( JScrollPane )c;
		return getScrollPaneAncestor( c.getParent() );
	}
	
	@Override
	public void focusNextComponent( Component component ) {

		component = 
			getCurrentManager().getFocusOwner();
		
		AbstractXMLFormComponent c = getAncestor( component );
		
		if ( c != null  ) {
			c.nextXMLFormFocus();
			c.scrollRectToVisible( c.getBounds() );
		} else
			super.focusNextComponent( component );
	}

	@Override
	public void focusPreviousComponent( Component component ) {

		component = 
			getCurrentManager().getFocusOwner();
		
		JScrollPane sp = getScrollPaneAncestor( component );
		if ( sp != null ) {
			
			if ( component.getParent() instanceof JViewport ) {
				component = component.getParent().getParent();
			}
			
			Rectangle destination = 
				SwingUtilities.convertRectangle( 
						component, 
						component.getBounds(), 
						sp );

			destination.translate( 0, -100 );			
			
			sp.getViewport().scrollRectToVisible( destination );

		}
	}

}
