package com.japisoft.xmlform.component;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.util.ArrayList;

import javax.swing.JLabel;
import javax.swing.JOptionPane;

import com.japisoft.xmlform.component.container.GridComponent;
import com.japisoft.xmlform.designer.properties.PropertyDescriptor;
import com.japisoft.xmlform.designer.properties.PropertyDescriptorImpl;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XMLFormLabelComponent extends StaticXMLFormComponent {

	private JLabel label = null;
	
	public XMLFormLabelComponent( boolean designMode, 
			ComponentContext context ) {
		super( designMode, context );
		add( label = new JLabel() );
		setLabel( "Your label" );
		Dimension dim = label.getPreferredSize();
		dim.width += 40;
		dim.height = 2 * GridComponent.getGridSize();
		setPreferredSize( dim );
	}

	private String text = null;
	
	public void setLabel( String text ) {
		firePropertyChange( "label", this.text, text );
		this.text = text;
		label.setText( text );
	}

	public String getLabel() {
		return text;
	}

	@Override
	public void setFont(Font font) {
		label.setFont( font );
	}
	
	@Override
	public Font getFont() {
		return label.getFont();
	}
	
	@Override
	public void setBackground( Color bg ) {
		label.setBackground( bg );
	}

	@Override
	public Color getBackground() {
		return label.getBackground();
	}

	@Override
	public void setForeground(Color fg) {
		label.setForeground(fg);
	}

	@Override
	public Color getForeground() {
		return super.getForeground();
	}

	protected void activeAction() {
		String newText = JOptionPane.showInputDialog( "New text", getLabel() );
		if ( newText != null )
				setLabel( newText );
	}

	protected void prepareProperties( ArrayList<PropertyDescriptor> l ) throws Exception {
		super.prepareProperties( l );
		l.add( new PropertyDescriptorImpl( "label", String.class, this ) );
	}

}
