package com.japisoft.xmlform.designer;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URL;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.japisoft.framework.app.toolkit.Toolkit;
import com.japisoft.xmlform.UIToolkit;
import com.japisoft.xmlform.component.AbstractXMLFormComponent;
import com.japisoft.xmlform.component.ComponentContext;
import com.japisoft.xmlform.component.XMLDeserizalizer;
import com.japisoft.xmlform.component.container.XMLFormContainer;
import com.japisoft.xmlform.designer.data.DataPanel;
import com.japisoft.xmlform.designer.data.DataPanelListener;
import com.japisoft.xmlform.designer.data.GrammarNodeTreeNode;
import com.japisoft.xmlform.designer.library.ComponentsPanel;
import com.japisoft.xmlform.designer.properties.PropertiesPanel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DesignerComponent extends JPanel implements DataPanelListener {
	
	private XMLFormContainer form = null;
	private DataPanel dp = null;
	private PropertiesPanel pp = null;
	private ComponentsPanel cp = null;
	private JSplitPane sp = null;
	private JScrollPane scrollPane = null;

	private int version = 1;
	
	public DesignerComponent() {
		initUI();		
	}

	private void initUI() {
		setLayout( 
			new BorderLayout() 
		);	
		
		sp = new JSplitPane( 
				JSplitPane.HORIZONTAL_SPLIT );

		dp = new DataPanel( this );		
		
		sp.setLeftComponent( 
				scrollPane = new JScrollPane(
						form = 
							dp.getComponentFactory().newRootContainer() ) );

		JTabbedPane tp = new JTabbedPane( 
				JTabbedPane.TOP );

		tp.setPreferredSize( new Dimension( 250, 0 ) );
		
		cp = 
			new ComponentsPanel();
		pp = 
			new PropertiesPanel();

		tp.addTab( "Data", 
				Toolkit.getImageIcon( "images/data.png" ), dp );
		tp.addTab( "Components", 
				Toolkit.getImageIcon( "images/component.png"), cp );
		tp.addTab( "Properties", 
				Toolkit.getImageIcon( "images/palette2.png" ), pp );

		sp.setOneTouchExpandable( true );

		sp.setRightComponent( tp );
		add( sp );
		
		sp.setDividerLocation( 700 );
		
	}

	public void setCurrentComponent( AbstractXMLFormComponent component ) {
		pp.init( component.getProperties() );
		GrammarNodeTreeNode gnt = component.getGrammarNode();
		if ( gnt != null ) {
			dp.selectNode( gnt );
		}

/*		Point p = SwingUtilities.convertPoint( 
				component, 
				new Point( 0, 0 ), 
				scrollPane );

		JViewport port = scrollPane.getViewport();
		port.setViewPosition( p ); */

	}
	
	public void cut() {
		form.cut();
	}

	public void scrollTo( AbstractXMLFormComponent component ) {

		Point p = UIToolkit.getLocation( component );

		scrollPane.getViewport().setViewPosition( p );

	}	

	public void newRoot(String selectedItem) {
		form.setXpath( "/" + selectedItem );
	}	

	public void newForm() {
		int dividerLocation = sp.getDividerLocation();
		dp.resetSchema( null );
		sp.setLeftComponent( null );
		sp.setLeftComponent(
			scrollPane = new JScrollPane(
				form = 
					dp.getComponentFactory().newRootContainer() ) );
		sp.setDividerLocation( dividerLocation );
		XmlFormModel.CURRENT_DOCUMENT = null;
	}
	
	public void save( Document doc ) {

		Element root = doc.createElementNS( null, "xf" );
		root.setAttribute( "appVersion", XmlFormModel.getAppVersion() );
		root.setAttribute( "version", "" + version );
		if ( XmlFormModel.CURRENT_SPELLCHECK != null )
			root.setAttribute( "spell", XmlFormModel.CURRENT_SPELLCHECK );

		if ( dp.getSchemaLocation() != null ) {
			root.setAttribute( "schema", "" + dp.getSchemaLocation() );
		}

		if ( dp.getRoot() != null ) {
			root.setAttribute( "root", dp.getRoot() );
		}

		Element fields = doc.createElementNS( null, "fields" );
		root.appendChild( fields );

		form.save( fields );
		doc.appendChild( root );

	}

	public void load( File f ) throws Exception {
		load( f.toString(), new FileInputStream( f ) );
	}

	public void load( String uri ) throws Exception {
		if ( uri.indexOf( "://" ) > -1 ) {
			load( uri, new URL( uri ).openStream() );
		} else
			load( new File( uri ) );
	}
	
	public void load( String uri, InputStream input ) {
		try {

			DocumentBuilderFactory factory = 
				DocumentBuilderFactory.newInstance();
			factory.setNamespaceAware( true );
			DocumentBuilder db = 
				factory.newDocumentBuilder();
			Document doc = 
				db.parse( input );
			load( uri, doc );
			
		} catch ( Exception e ) {
			XmlFormModel.debug( e );
			UIToolkit.dispatchError( 
					"Can't load " + uri + " : " + e.getMessage() 
			);
		}
		
	}

	public void load( String uri, Document doc ) throws Exception {
			
			Element root = doc.getDocumentElement();
			
			if ( root.hasAttribute( "spell" ) )
				XmlFormModel.CURRENT_SPELLCHECK = 
					root.getAttribute( "spell" ); 

			String schema = root.getAttribute( "schema" );
			
			if ( !"".equals( schema ) ) {

				schema = com.japisoft.xmlform.Toolkit.getAbsolutePath(
						uri,
						schema );

				dp.resetSchema( schema );
				
			} else {
				
				UIToolkit.warn( "No schema found ?" );

			}

			String schemaRoot = root.getAttribute( "root" );
			if ( !"".equals( schemaRoot ) ) {
				dp.resetRoot( schemaRoot );
			}
						
			AbstractXMLFormComponent rootComponent = 
				XMLDeserizalizer.build( 
					doc,
					true,
					dp.getGrammarNodeTreeNodeRoot(),
					null,
					new ComponentContext() {
						public com.japisoft.xmlform.component.XMLFormComponentFactory getComponentFactory() {
							return dp.getComponentFactory();
						};
						public com.japisoft.xmlform.designer.data.GrammarNodeTreeNode getCurrentTreeNode() {
							return dp.getCurrentTreeNode();
						};
						public Document getDocument() {
							return null;
						}
						public boolean inBuildingXMLDocument() {
							return false;
						}

						public AbstractXMLFormComponent getComponentById(
								String id) {
							return null;
						}
						public void setComponentById(String id,
								AbstractXMLFormComponent component) {
						}
						
						public void action(int actionCode,Object parameter) {
							dp.action( actionCode, parameter );
						}
					} );
			if ( !( rootComponent instanceof XMLFormContainer ) ) {
				throw new Exception( "Illegal root component" );
			}
			XMLFormContainer newRoot = 
				( XMLFormContainer )rootComponent;
			this.form = newRoot;

			// For removing the previous one
			int dividerLocation = sp.getDividerLocation();
			sp.setLeftComponent( null );
			sp.setLeftComponent( 
					scrollPane = new JScrollPane( newRoot ) );
			sp.setDividerLocation( 
					dividerLocation );

			newRoot.resetRootContainerSize();

			XmlFormModel.CURRENT_DOCUMENT = uri;
			
	}
	
}
