package com.japisoft.xmlform.designer;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URL;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.framework.application.descriptor.InterfaceBuilder;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DesignerFrame extends JFrame {

	private InterfaceBuilder builder = null;
	private DesignerComponent panel = null;
	
	public DesignerFrame( InterfaceBuilder builder ) {
		this.builder = builder;
		setTitle( "XML Form Designer" );
		setDefaultCloseOperation( 
				DO_NOTHING_ON_CLOSE ); 
		addWindowListener(
				new WindowAdapter() {
					@Override
					public void windowClosing(WindowEvent e) {
						ActionModel.activeActionById( 
								"quit", 
								null );
					}
				} );
	}
	
	
	
	public void initUI() {
		setLayout( new BorderLayout() );
		if ( builder.getMenuBar() != null ) {
			setJMenuBar( builder.getMenuBar() );
		}

		if ( builder.getToolBarById( "DEFAULT" ) != null ) {
			add( builder.getToolBarById( "DEFAULT" ), 
					BorderLayout.NORTH );
		}

		add( panel = new DesignerComponent() );
		
	}

	public void newForm() {
		panel.newForm();
	}
	
	public void save( Document doc ) {
		panel.save( doc );
	}

	public void load( File f ) throws Exception {
		panel.load( f );
	}

	public void load( String uri ) throws Exception {
		panel.load( uri );
	}
	
	public void load( String uri, InputStream input ) {
		panel.load( uri, input );
	}
	
}
