package com.japisoft.xmlform.designer.actions.file;

import java.awt.event.ActionEvent;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;

import com.japisoft.framework.ui.toolkit.FileManager;
import com.japisoft.xmlform.UIToolkit;
import com.japisoft.xmlform.designer.XmlFormModel;
import com.japisoft.xmlform.designer.actions.CommonAction;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SaveAction extends CommonAction {

	public void actionPerformed2(ActionEvent e) {

		String uri = XmlFormModel.CURRENT_DOCUMENT;

		if ( uri == null ) {
			File f = FileManager.getSelectedFile( 
					false, 
					OpenAction.EXT, 
					"XML form description" );
			if ( f != null )
				uri = f.toString();
		}
		if ( uri!= null ) {
			saveTo( uri );
		}

	}

	public void saveTo( String uri ) {
		try {
			DocumentBuilderFactory factory = 
				DocumentBuilderFactory.newInstance();
			factory.setNamespaceAware( true );
			Document d = 
				factory.newDocumentBuilder().newDocument();
			frame.save( d );

			Transformer t = 
				TransformerFactory.newInstance().newTransformer();
			t.setOutputProperty( OutputKeys.ENCODING, "UTF-8" );
			t.setOutputProperty( OutputKeys.INDENT, "yes" );
			t.setOutputProperty( OutputKeys.METHOD, "xml" );

			ByteArrayOutputStream out = 
				new ByteArrayOutputStream();

			StreamResult res = new StreamResult( out );
			t.transform( 
				new DOMSource( d ), 
				res );

			if ( uri.indexOf( "://" ) > -1 ) {

				URL u = new URL( uri );
				URLConnection c = 
					u.openConnection();
				c.setDoOutput( true );
				c.setUseCaches( false );

				String s = new String( out.toByteArray(), "UTF-8" );
				DataOutputStream dout = new DataOutputStream( c.getOutputStream() );
				try {
					dout.writeBytes( "file=" + URLEncoder.encode( s ) );
				} finally {
					dout.close();
				}
				
				InputStream input = 
					c.getInputStream();

				int cc;
				while ( ( cc = input.read() ) != -1 ) {}

			} else {
				FileOutputStream output = new FileOutputStream( uri );
				try {
					output.write( out.toByteArray() );
				} finally {
					output.close();
				}
			}

			XmlFormModel.CURRENT_DOCUMENT = uri;

		} catch ( Exception e1 ) {
			UIToolkit.dispatchError( "Can't save : " + e1.getMessage() );
		}		
	}
	
}
