package com.japisoft.xmlform.designer.data;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;

import javax.swing.JComponent;
import javax.swing.JTree;
import javax.swing.TransferHandler;
import javax.swing.tree.TreePath;

import com.japisoft.framework.xml.grammar.GrammarContainer;
import com.japisoft.framework.xml.grammar.GrammarNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TreeDragDrop extends TransferHandler implements MouseMotionListener {

	private JTree tree = null;
	
	public TreeDragDrop( JTree tree ) {
		this.tree = tree;
		tree.setTransferHandler( this );
		this.tree.addMouseMotionListener( this );
	}

	public void mouseDragged( MouseEvent e ) {
		exportAsDrag( tree, e, TransferHandler.MOVE );
	}

	public void mouseMoved(MouseEvent e) {	
	}

	GrammarNodeTreeNode dragNode = null;
	
	@Override
	public void exportAsDrag(
			JComponent comp, 
			InputEvent e, 
			int action ) {
		JTree tree = ( JTree )comp;
		MouseEvent me = ( MouseEvent )e;
		TreePath tp = 
			tree.getPathForLocation( 
					me.getX(), 
						me.getY() );
		if ( tp != null ) {
			// Can't drag the root node
			GrammarNodeTreeNode node = ( GrammarNodeTreeNode )tp.getLastPathComponent(); 
			
			if ( node.getUserObject() == null ) {
			
				if ( node.getParent() != null ) {
					
					GrammarNode source = node.getSource();
					if ( source instanceof GrammarContainer ) {
						return;
					}
					
					dragNode = node;
					super.exportAsDrag(comp, e, action);
				}
			
			}
		}
	}

	@Override
	public boolean canImport(
			JComponent comp, DataFlavor[] transferFlavors ) {
		return false;
	}

	@Override
	protected Transferable createTransferable( JComponent c ) {
		if ( dragNode != null )
			return new StringSelection( "xpath:" + dragNode.toXPath() );
		else
			return super.createTransferable( c );
	}

	@Override
	public int getSourceActions(JComponent c) {
		return TransferHandler.MOVE;
	}

}
