package com.japisoft.xmlform.designer.properties;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import com.japisoft.framework.ApplicationModel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class PropertyDescriptorImpl implements PropertyDescriptor {

	private Object component = null;
	private String name = null;
	private Class paramType = null;
	
	private Method setter = null;
	private Method getter = null;
	
	public PropertyDescriptorImpl( 
			String name, 
			Class paramType, 
			Object component ) throws Exception {
		this.name = name;
		this.component = component;
		this.paramType = paramType;
		
		name = Character.toUpperCase( name.charAt( 0 ) ) + 
			name.substring( 1 );
		
		if ( paramType == Boolean.class )
			paramType = boolean.class;
		
		setter = component.getClass().getMethod( 
				"set" + name, 
				paramType );

		getter = component.getClass().getMethod( 
				"get" + name, 
				null );
	}

	public String getName() {
		return name;
	}
	
	public Class getType() {
		return paramType;
	}

	private boolean displayable = true;

	public boolean displayable() {
		return displayable;
	}	
	
	public Object getValue() {
		try {
			return getter.invoke( component, null );
		} catch (IllegalArgumentException e) {
			ApplicationModel.debug( e );
		} catch (IllegalAccessException e) {
			ApplicationModel.debug( e );
		} catch (InvocationTargetException e) {
			ApplicationModel.debug( e );
		}
		return null;
	}

	public void setValue( Object value ) {
		try {
			setter.invoke( component, value );
		} catch (IllegalArgumentException e) {
			ApplicationModel.debug( e );
		} catch (IllegalAccessException e) {
			ApplicationModel.debug( e );
		} catch (InvocationTargetException e) {
			ApplicationModel.debug( e );
		}
	}

	@Override
	public String toString() {
		return getName();
	}
	
}
