package com.japisoft.xmlform.designer.properties;

import java.awt.Color;
import java.awt.Font;
import java.util.HashMap;

import com.japisoft.xmlform.designer.properties.editors.BooleanEditor;
import com.japisoft.xmlform.designer.properties.editors.ColorEditor;
import com.japisoft.xmlform.designer.properties.editors.DictionnaryEditor;
import com.japisoft.xmlform.designer.properties.editors.FontEditor;
import com.japisoft.xmlform.designer.properties.editors.TextEditor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class PropertyEditorFactory {

	private static HashMap<Class,PropertyEditor> mapEditors = 
		new HashMap<Class, PropertyEditor>();

	private static HashMap<Class,PropertyEditor> mapRenderers = 
		new HashMap<Class, PropertyEditor>();
	
	static {
		mapEditors.put( String.class, new TextEditor() );
		mapEditors.put( Integer.class, new TextEditor( TextEditor.INT_TYPE ) );
		mapEditors.put( Color.class, new ColorEditor() );
		mapEditors.put( Font.class, new FontEditor() );
		mapEditors.put( Boolean.class, new BooleanEditor() );
		mapEditors.put( HashMap.class, new DictionnaryEditor() );
		
		mapRenderers.put( String.class, new TextEditor() );
		mapRenderers.put( Integer.class, new TextEditor( TextEditor.INT_TYPE ) );
		mapRenderers.put( Color.class, new ColorEditor() );
		mapRenderers.put( Font.class, new FontEditor() );
		mapRenderers.put( Boolean.class, new BooleanEditor() );
		mapRenderers.put( HashMap.class, new DictionnaryEditor() );
	}

	public static PropertyEditor getEditor( 
			PropertyDescriptor descriptor ) {

		PropertyEditor editor = mapEditors.get( 
				descriptor.getType() );

		if ( editor == null )
			editor = mapEditors.get( String.class );
		editor.setValue( descriptor.getValue() );
		return editor;

	}

	public static PropertyEditor getEditorForRendering( 
			PropertyDescriptor descriptor ) {

		PropertyEditor editor = mapRenderers.get( 
				descriptor.getType() );

		if ( editor == null )
			editor = mapRenderers.get( String.class );
		editor.setValue( descriptor.getValue() );
		return editor;

	}
	
}
