package com.japisoft.xmlform.designer.properties.editors;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DictionnaryPanel extends JPanel {

	public DictionnaryPanel() {
		initUI();
	}

	private JTable table = null;
	
	private void initUI() {
		setLayout( new BorderLayout() );
		add( new JScrollPane( table = new JTable() ) );
	}

	public void stopEditing() {
		if ( table.getCellEditor() != null )
			table.getCellEditor().stopCellEditing();
	}

	public void init( HashMap<String,String> map ) {
		table.setModel( new CustomTableModel( map ) );
	}

	class CustomTableModel implements TableModel {

		private HashMap<String,String> map = null;
		private List<String> lst = null;
		
		CustomTableModel( HashMap<String,String> map ) {
			this.map = map;
			this.lst = new ArrayList<String>();
			for ( String key : map.keySet() ) {
				lst.add( key );
			}
			Collections.sort( lst );
		}

		public void addTableModelListener(TableModelListener l) {
		}

		public Class<?> getColumnClass(int columnIndex) {
			return String.class;
		}

		public int getColumnCount() {
			return 2;
		}

		public String getColumnName(int columnIndex) {
			if ( columnIndex == 0 )
				return "Value";
			else
				return "Visible value";
		}

		public int getRowCount() {
			return lst.size();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			if ( columnIndex == 0 ) {
				return lst.get( rowIndex );
			} else {
				return map.get( lst.get( rowIndex ) );
			}
		}

		public boolean isCellEditable(int rowIndex, int columnIndex) {
			if ( columnIndex == 1 )
				return true;
			return false;
		}

		public void removeTableModelListener( TableModelListener l ) {
		}

		public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
			if ( columnIndex == 1 ) {
				
				// Reset to the default value
				if ( "".equals( aValue ) )
					aValue = lst.get( rowIndex );

				map.put( 
					lst.get( rowIndex ), 
					( String )aValue );
			}
		}

	}
	
}
