package com.japisoft.xmlform.designer.properties.editors;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GraphicsEnvironment;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultListModel;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FontChooser extends JPanel implements ListSelectionListener, ActionListener {

	/** Creates new form DesignerPanel */
    public FontChooser() {
        initComponents();
        String[] fonts = 
        	GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
        lstFont.setListData( fonts );
        DefaultListModel model = new DefaultListModel();
        for (int i = 8; i <= 72; i += 2)
        	model.addElement( i );
        lstSize.setModel( model );
        model = new DefaultListModel();
        model.addElement( "Plain" );
        model.addElement( "Bold" );
        model.addElement( "Italic" );
        lstStyle.setModel( model );
        lstStyle.setSelectedIndex( 0 );
        lstSize.setSelectedIndex( 0 );
        lstFont.setSelectedIndex( 0 );
    }

    @Override
	public void addNotify() {
		super.addNotify();
		lstFont.addListSelectionListener( this );
		lstSize.addListSelectionListener( this );
		lstStyle.addListSelectionListener( this );
		btnOk.addActionListener( this );
		btnCancel.addActionListener( this );
	}

	@Override
	public void removeNotify() {
		super.removeNotify();
		lstFont.removeListSelectionListener( this );
		lstSize.removeListSelectionListener( this );
		lstStyle.removeListSelectionListener( this );
		btnOk.removeActionListener( this );
		btnCancel.removeActionListener( this );
	}

	public void actionPerformed(ActionEvent e) {
		if ( e.getSource() == btnCancel )
			currentFont = null;
		Window w = ( Window )SwingUtilities.getRoot( this );
		w.setVisible( false );
	}
	
	private Font currentFont = null;

	public Font getFont() { return currentFont; }

	public void setFont( Font f ) {
		this.currentFont = f;
		if ( f != null ) {
			if ( lstFont == null )
				return;
			lstFont.setSelectedValue( f.getName(), true );
			lstSize.setSelectedValue( f.getSize(), true );
			if ( f.getStyle() == Font.PLAIN )
				lstStyle.setSelectedIndex( 0 );
			else
			if ( f.getStyle() == Font.BOLD )
				lstStyle.setSelectedIndex( 1 );
			else
				lstStyle.setSelectedIndex( 2 );
			txtPreview.setFont( currentFont );
		}
	}

    public void valueChanged( ListSelectionEvent e ) {
    	int style = Font.PLAIN;
    	if ( "Bold".equals( lstStyle.getSelectedValue() ) )
    		style = Font.BOLD;
    	else
    	if ( "Italic".equals( lstStyle.getSelectedValue() ) )
    		style = Font.ITALIC;
    	currentFont = new Font(
    			( String )lstFont.getSelectedValue(),
    			style,
    			( Integer )lstSize.getSelectedValue() );
    	txtPreview.setFont( currentFont );
    }

    public static Font showFontChooser( Component source, String title, Font defaultFont ) {
    	Window w = ( Window )SwingUtilities.getRoot( source );
    	JDialog d = null;
    	if ( w instanceof Frame ) {
    		d = new JDialog( ( Frame )w );
    	} else
    		d = new JDialog( ( Dialog )w );
    	d.setTitle( title );
    	FontChooser f = new FontChooser();
    	f.setFont( defaultFont );
    	d.add( f );
    	d.pack();
    	d.setModal( true );
    	d.setVisible( true );
    	return f.getFont();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">                          
    private void initComponents() {
        jLabel1 = new javax.swing.JLabel();
        spFont = new javax.swing.JScrollPane();
        lstFont = new javax.swing.JList();
        jLabel2 = new javax.swing.JLabel();
        spStyle = new javax.swing.JScrollPane();
        lstStyle = new javax.swing.JList();
        jLabel3 = new javax.swing.JLabel();
        spSize = new javax.swing.JScrollPane();
        lstSize = new javax.swing.JList();
        jSeparator1 = new javax.swing.JSeparator();
        jLabel4 = new javax.swing.JLabel();
        spPreview = new javax.swing.JScrollPane();
        txtPreview = new javax.swing.JTextArea();
        btnOk = new javax.swing.JButton();
        btnCancel = new javax.swing.JButton();

        jLabel1.setText("Font");

        lstFont.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        spFont.setViewportView(lstFont);

        jLabel2.setText("Style");

        lstStyle.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        spStyle.setViewportView(lstStyle);

        jLabel3.setText("Size");

        lstSize.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        spSize.setViewportView(lstSize);

        jLabel4.setText("Preview");

        txtPreview.setColumns(20);
        txtPreview.setRows(5);
        txtPreview.setText("Your text");
        spPreview.setViewportView(txtPreview);

        btnOk.setText("Ok");

        btnCancel.setText("Cancel");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jSeparator1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 400, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel1)
                            .add(spFont, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 158, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(18, 18, 18)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel2)
                            .add(spStyle, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 143, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 17, Short.MAX_VALUE)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel3)
                            .add(spSize, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 64, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(jLabel4)
                    .add(spPreview, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 400, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(btnOk)
                        .add(12, 12, 12)
                        .add(btnCancel)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(jLabel2)
                    .add(jLabel3))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(spSize)
                    .add(spStyle)
                    .add(spFont, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 166, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jSeparator1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 10, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel4)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(spPreview, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 51, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btnOk)
                    .add(btnCancel))
                .addContainerGap())
        );
    }// </editor-fold>                        
    
    
    // Variables declaration - do not modify                     
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnOk;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JList lstFont;
    private javax.swing.JList lstSize;
    private javax.swing.JList lstStyle;
    private javax.swing.JScrollPane spFont;
    private javax.swing.JScrollPane spPreview;
    private javax.swing.JScrollPane spSize;
    private javax.swing.JScrollPane spStyle;
    private javax.swing.JTextArea txtPreview;
    // End of variables declaration                   
    
	
}
