package com.japisoft.xmlform.editor;

import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.List;

import javax.swing.JFrame;

import org.w3c.dom.Node;

import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.framework.application.descriptor.InterfaceBuilder;
import com.japisoft.xmlform.component.AbstractXMLFormComponent;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class EditorFrame extends JFrame {

	private InterfaceBuilder builder = null;
	
	public EditorFrame( InterfaceBuilder builder ) {
		this.builder = builder;
		setTitle( "XML Form Editor" );
		setDefaultCloseOperation( 
				DO_NOTHING_ON_CLOSE ); 
		addWindowListener(
				new WindowAdapter() {
					@Override
					public void windowClosing(WindowEvent e) {
						ActionModel.activeActionById( "quit", null );
					}
				} 
		);
	}

	private EditorComponent editor = null;
	
	public void initUI() {
		setLayout( new BorderLayout() );
		
		if ( builder.getMenuBar() != null ) {
			setJMenuBar( builder.getMenuBar() );
		}
		
		add( 
			builder.getToolBarById( "DEFAULT" ), 
			BorderLayout.NORTH 
		);
		
		if ( builder.getToolBarById( "main" ) != null ) {
			add( 
				builder.getToolBarById( "main" ), 
				BorderLayout.NORTH );
		}
		
		editor = new EditorComponent();
		add( editor );
	}

	public void newDocument( File formPath ) throws Exception {
		editor.newDocument( formPath );
	}
	
	public void newDocument( String formURI ) throws Exception {
		editor.newDocument( formURI );
	}
	
	public void loadDocument( String uri ) throws Exception {
		editor.loadDocument( uri );
	}
	
	public void loadDocument( File f ) throws Exception {
		editor.loadDocument( f );
	}

	public void saveDocument( String uri ) throws Exception {
		editor.saveDocument( uri );
	}
	
	public boolean validateDocument() {
		return editor.validateDocument();
	}
	
	public void setFocusTo( AbstractXMLFormComponent component ) {
		editor.setFocusTo( component );
	}
	
	public List<Node> checkEmptyFields() {
		return editor.checkEmptyFields();
	}
}
