package com.japisoft.xmlform.editor;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.util.List;

import javax.swing.Icon;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;

import com.japisoft.framework.app.toolkit.Toolkit;
import com.japisoft.framework.internationalization.Traductor;
import com.japisoft.framework.ui.FastLabel;
import com.japisoft.framework.xml.validator.ErrorValidationNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class EditorValidationErrorsPanel extends JPanel implements ListSelectionListener {

	private ErrorHighlighter highlighter;
	private JTable table;

	public EditorValidationErrorsPanel( ErrorHighlighter highligher ) {
		this.highlighter = highligher;
		initUI();
		setPreferredSize( 
			new Dimension( 
				0, 
				table.getFontMetrics( 
					table.getFont() ).getHeight() * 6 ) );
	}

	@Override
	public void addNotify() {
		super.addNotify();
		table.getSelectionModel().addListSelectionListener( this );
	}

	@Override
	public void removeNotify() {
		super.removeNotify();
		table.getSelectionModel().removeListSelectionListener( this );
	}

	private void initUI() {
		setLayout( new BorderLayout() );
		add( new JScrollPane( 
				table = new JTable() ) );
		table.setDefaultRenderer( 
				ErrorValidationNode.class, 
				new CustomRenderer() );
		table.getSelectionModel().setSelectionMode( 
				ListSelectionModel.SINGLE_SELECTION );
	}

	public void init( List<ErrorValidationNode> errors ) {
		table.setModel( 
			new ErrorsTableModel( errors ) );
		table.getColumnModel().getColumn( 0 ).setMaxWidth( 16 );
	}

	public void valueChanged(ListSelectionEvent e) {
		ErrorValidationNode node = 
			( ErrorValidationNode )table.getModel().getValueAt( 
					e.getFirstIndex(), 
					0 );
		highlighter.highlight( node );
	}	

	class ErrorsTableModel implements TableModel {

		private List<ErrorValidationNode> errors = null;

		ErrorsTableModel( List<ErrorValidationNode> errors ) {
			this.errors = errors;
		}

		public void addTableModelListener(TableModelListener l) {}

		public Class<?> getColumnClass(int columnIndex) {
			return ErrorValidationNode.class;
		}

		public int getColumnCount() {
			return 2;
		}

		public String getColumnName( int columnIndex ) {
			if ( columnIndex == 1 ) {
				if ( errors.size() == 1 )
					return Traductor.traduce( "oneerror", "1 error" );
				else
				return errors.size() + " " + Traductor.traduce( "errors2", "errors" );
			} else
				return "";
		}

		public int getRowCount() {
			return errors.size();
		}

		public Object getValueAt( int rowIndex, int columnIndex ) {
			return errors.get( rowIndex );
		}

		public boolean isCellEditable( int rowIndex, int columnIndex ) {
			return false;
		}

		public void removeTableModelListener( TableModelListener l ) {}

		public void setValueAt( Object aValue, int rowIndex, int columnIndex ) {}
		
	}

	class CustomRenderer extends FastLabel implements TableCellRenderer {

		private Icon bugIcon = null;

		CustomRenderer() {
			bugIcon = 
				Toolkit.getImageIcon( "images/bug_red16.png" );
		}

		public Component getTableCellRendererComponent(
				JTable table, 
				Object value, 
				boolean isSelected, 
				boolean hasFocus, 
				int row, 
				int column ) {

			if ( column == 0 ) {
				setIcon( bugIcon );
				setText( null );
			} else {
				ErrorValidationNode evn = ( ErrorValidationNode )value;
				setText( evn.getMessage() );
				setIcon( null );
			}

			if ( isSelected ) {
				setBackground( table.getSelectionBackground() );
				setForeground( table.getSelectionForeground() );
			} else {
				setBackground( table.getBackground() );
				setForeground( table.getForeground() );
			}
			
			return this;
		}
	}

}
