package com.japisoft.xmlpad;

import java.util.Iterator;
import javax.swing.Action;
import javax.swing.JComponent;

import com.japisoft.xmlpad.bookmark.BookmarkContext;
import com.japisoft.xmlpad.tree.parser.Parser;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public interface IXMLPanel {
	
	public IXMLPanel getPanelParent();
	
	/** @return the current container */
	public XMLContainer getMainContainer();

	public String getCurrentDocumentLocation();	
	
	/** Particular case with several XMLContainer (like XSLT tab) */
	public XMLContainer getSelectedContainer();

	/** For special case with several XMLContainers like for the XSLT debugger */
	public BookmarkContext getBookmarkContext();
	
	/** @return another container for this document type */
	public XMLContainer getSubContainer( String type );

	/** @return a new parser for the document tree */
	public Parser createNewParser( boolean lightweightMode );
	
	// Special case for XSLT
	/** @return set of sub container */
	public int getSubContainerCount();

	/** @return set of sub container */
	public XMLContainer getSubContainerAt( int index );

	/** Force selection of this panel */
	public void selectSubContainer( IXMLPanel panel );

	/** @return the global ui view */
	public JComponent getView();

	/** reset a component property */
	public void setProperty(String name, Object content);
		
	/** @return a property value */
	public Object getProperty(String name);

	/** @return a property value of the default one */
	public Object getProperty(String name, Object def );

	/** @return a list of properties */
	public Iterator getProperties();
		
	/** Dispose this container */
	public void dispose();

	/** Prepare to save the editor */
	public void prepareToSave();
	
	/** Call when loading a new content */
	public void postLoad();
	
	/** Reload the content and return <code>true</code> if the operation is a success */
	public boolean reload();
	
	public void cut();
	public void copy();
	public void paste();
	
	public Object print();
	
	public void setDocumentInfo( XMLDocumentInfo info );	
	
	/**
	 * Set the component in a special mode for freeing internal resource. By default
	 * to <code>false</code> */
	public void setAutoDisposeMode( boolean disposeMode );
	
	/** For custom copy/cut/paste action as samples */
	public Action getAction( String actionId );

}
