package com.japisoft.xmlpad.action.file;

import java.io.*;
import javax.swing.*;

import com.japisoft.xmlpad.UIFactory;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SaveAsAction extends SaveAction {
	
	public static final String ID = SaveAsAction.class.getName();
	public static final String CURRENT_DIR_PROPERTY = "currentDir";
	
	public SaveAsAction() {
		super();
	}
	
	private String currentDir = null;
	
	public void setProperty(String propertyName, Object value) {
		if ( CURRENT_DIR_PROPERTY.equals( propertyName ) ) {
			currentDir = ( String )value;
		}
	}
	
	public Object getProperty(String propertyName, Object defaultValue) {
		if ( CURRENT_DIR_PROPERTY.equals( propertyName ) )
			return currentDir;
		return super.getProperty( propertyName, defaultValue );
	}

	private boolean enabledExt = true;

	/** Force the XML extension when choosing a file name */
	public void setForceExtension(boolean enable) {
		this.enabledExt = enable;
	}

	public boolean notifyAction() {
		if ( container == null )
			return false;

		JFileChooser chooser = UIFactory.getInstance().getSaveFileChooser();
		if ( currentDir != null ) {
			chooser.setCurrentDirectory( new File( currentDir ) );
		}

		chooser.setFileFilter(container.getDocumentInfo().getFileFilter());
		if (container.getDocumentInfo().getWorkingDirectory() != null) {
			chooser.setCurrentDirectory(
				new File(container.getDocumentInfo().getWorkingDirectory()));
		} else
			if ( container.getDocumentInfo().getCurrentDocumentLocation() != null )
				chooser.setCurrentDirectory( 
					new File( container.getDocumentInfo().getCurrentDocumentLocation() ).getParentFile() );

		int returnVal = chooser.showSaveDialog(editor);
		if ( returnVal == JFileChooser.APPROVE_OPTION ) {
			String fileName = "" + chooser.getSelectedFile();
			
			try {
				currentDir = chooser.getSelectedFile().getParentFile().toString();
			} catch( Throwable th ) {}
			
			if (enabledExt) {
				// Check if the file name only contains a '.' or not
				if (chooser.getSelectedFile().getName().indexOf(".") == -1)
					fileName += "."
						+ container.getDocumentInfo().getDefaultFileExt();
			}			
			try {
				
				// Check if this is to another file name than the current one
				if ( container.getCurrentDocumentLocation() == null ||
						!container.getCurrentDocumentLocation().equals( fileName ) ) {
					File f = new File( fileName );
					if ( f.exists() ) {
						if ( JOptionPane.showConfirmDialog(
								container.getView(),
								"Overwrite " + fileName + " ?" ) !=
									JOptionPane.YES_OPTION ) {
							return false;
						}
					}
				}

				if (saveDocument(fileName)) {
					container.setCurrentDocumentLocation(fileName);
					return VALID_ACTION;
				} else
					return INVALID_ACTION;
			} catch (Throwable th) {
				JOptionPane.showMessageDialog(
					editor,
					th.getMessage(),
					"Error",
					JOptionPane.ERROR_MESSAGE);
				return INVALID_ACTION;
			}
		} else
			return INVALID_ACTION;
	}

}
