package com.japisoft.xmlpad.action.search;

import java.util.List;

import com.japisoft.framework.collection.FastVector;
import com.japisoft.framework.xml.parser.document.Document;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.action.XMLAction;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SearchElement extends XMLAction {

	public static final String ID = SearchElement.class.getName();
	
	public static final int NEXT_TAG = 0;
	public static final int PREVIOUS_TAG = 1;
	public static final int TAG_START = 2;
	public static final int TAG_END = 3;
	
	private int type;

	public SearchElement( int type ) {
		this.type = type;
	}

	protected boolean autoRequestFocus() { return false; }

	public boolean notifyAction() {
		FPNode node = ( FPNode )container.getTree().getModel().getRoot();
		if ( node == null )
			return INVALID_ACTION;

		FPNode current = container.getCurrentNode(); 
		if ( current == null ) {
			switch( type ) {
				case PREVIOUS_TAG : return INVALID_ACTION;
				case NEXT_TAG : 
					highlightLine( node.getStartingLine() );
					return VALID_ACTION;
				case TAG_START :
				case TAG_END : 
					return INVALID_ACTION;
			}
		}

		Document doc = node.getDocument();
		List<FPNode> v = doc.getFlatNodes();
		int index = v.indexOf( current );

		switch( type ) {
			case PREVIOUS_TAG : 
				if ( index > 0 ) {
					FPNode n = ( FPNode )v.get( index - 1 );
					highlightLine( n.getStartingLine() );
					return VALID_ACTION;
				} else
					return INVALID_ACTION;
			case NEXT_TAG : 
				if ( index + 1 < v.size() ) {
					FPNode n = ( FPNode )v.get( index + 1 );
					highlightLine( n.getStartingLine() );
					return VALID_ACTION;
				} else
					return INVALID_ACTION;
			case TAG_START :
				highlightLine( current.getStartingLine() );
				return VALID_ACTION;
			case TAG_END :
				highlightLine( current.getStoppingLine() );
				return VALID_ACTION;
		}
		return VALID_ACTION;		
	}

	private void highlightLine( int line ) {
		container.getEditor().asynchronousHighlightLine( line );
	}

}
