package com.japisoft.xmlpad.action.xml;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.*;
import javax.swing.text.BadLocationException;

import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.action.XMLAction;
import com.japisoft.xmlpad.dialog.DialogManager;
import com.japisoft.xmlpad.editor.XMLPadDocument;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CommentAction extends XMLAction {
	
	public static final String ID = CommentAction.class.getName();
	
	public CommentAction() {
		super();
	}

	public boolean notifyAction() {
		String ini = null;
		int loc = editor.getCaretPosition();

		if ( ( (XMLPadDocument)editor.getDocument() ).isInsideTagExceptAttributeValue( loc ) ) {
			JOptionPane.showMessageDialog( editor, "Can't insert a comment here !" );
			return INVALID_ACTION;
		}
		
		int startSelection = editor.getSelectionStart();
		int stopSelection = editor.getSelectionEnd();

		Integer[] startStop = ( ( XMLPadDocument)editor.getDocument() ).getCommentDelimiters( loc );
		int start = -1;
		int stop = -1;

		int starter = 4;
		int stopper = 7;

		if ( startStop != null ) {
			start = startStop[ 0 ].intValue() + 4;
			stop = startStop[ 1 ].intValue() - 2;
		
			try {
				ini = editor.getDocument().getText( start, stop - start );
				if ( !checkComment( ini ) )
					return !XMLAction.VALID_ACTION;
			} catch( BadLocationException exc ) {
			}
		} else {
			if ( startSelection < stopSelection ) {
				try {
					ini = editor.getDocument().getText( startSelection, stopSelection - startSelection );
					if ( !checkComment( ini ) )
						return !XMLAction.VALID_ACTION;
					start = startSelection;
					stop = stopSelection;
					starter = 0;
					stopper = 0;
				} catch( BadLocationException exc ) {
				}
			}
		}
		
		CommentPane mDia = new CommentPane();		
		mDia.setText( ini );
		
		if ( DialogManager.showDialog(
				SwingUtilities.getWindowAncestor( container.getView() ),
				"Comment", 
				"Insert/Update", 
				"Insert or update at the current selection of your XML comment\nMultiple lines can be used",
				null,
				mDia ) != DialogManager.OK )
			return VALID_ACTION;

		String comment = mDia.getText();

		if (comment != null) {
			boolean empty = ( comment.length() == 0 );
			comment = "<!--" + comment + "-->";

			if (ini == null)
				editor.insertText(comment);
			else {
				try {
					if ( start != - 1 )
						editor.getDocument().remove(start - starter, stop - start + stopper);
					if ( !empty )
						editor.getDocument().insertString(start - starter, comment, null);
				} catch (Throwable th) {
					//th.printStackTrace();
				}
			}
		}
		return VALID_ACTION;
	}
	
	/** Check if the text has XML comment and showes a dialog box if this is the case */
	private boolean checkComment( String text ) {
		if ( text.indexOf( "<!--" ) > -1 ) {
			JOptionPane.showMessageDialog( 
					container.getView(), 
					XMLContainer.getLocalizedMessage( "COMMENT_ERROR2", "Can't comment this selected text, you must remove the comments" ),
					XMLContainer.getLocalizedMessage( "ERROR", "Error" ),
					JOptionPane.ERROR_MESSAGE );
			return false;
		}
		return true;
	}
	
	protected void notifyXMLContainer() {
		setEnabled( container.isEditable() );
	}

	///////////////////////////

	class CommentPane extends JPanel {
		private JTextArea text;

		public CommentPane() {
			init();
			setPreferredSize( new Dimension( 400, 300 ) );
		}

		void init() {
			setLayout( new BorderLayout() );
			add( new JScrollPane( text = new JTextArea() ) );
			text.setForeground( editor.getColorForComment() );
			text.setBackground( editor.getBackground() );
		}

		public void addNotify() {
			super.addNotify();
			text.requestFocus();
		}

		public void setText( String content ) {
			text.setText( content );
		}
		
		public String getText() {
			return text.getText();
		}
	}

}

// CommentAction ends here
