package com.japisoft.xmlpad.dialog;

import java.awt.Container;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class OkCancelDialog extends JDialog implements ActionListener {
	private JPanel mainPanel;

	public static final String CANCEL_ACTION = "cancel";
	public static final String OK_ACTION = "ok";

	/** Use bundle for changing title/icon.. By defaul true */
	public static boolean BUNDLE = true;
	private String[] actions = new String[] { OK_ACTION, CANCEL_ACTION };

	public OkCancelDialog(String dialogTitle, String title, String comment) {
		super();
		init(dialogTitle, title, comment);
	}

	public OkCancelDialog(
		Dialog owner,
		String dialogTitle,
		String title,
		String comment) {
		super(owner);
		init(dialogTitle, title, comment);
	}

	public OkCancelDialog(
		Frame owner,
		String dialogTitle,
		String title,
		String comment) {
		super(owner);
		init(dialogTitle, title, comment);
	}

	protected void setEnabledButton( String label, boolean state ) {
		for (int i = pnlOkCancel.getComponentCount() - 1; i >= 0; i--) {
			if ( label.equals( 
			 	( (JButton ) pnlOkCancel.getComponent(i)).getText() ) ) {
				pnlOkCancel.getComponent(i).setEnabled( state );
				break;
			}				
		}
	}

	//JPF
	public void removeNotify() {
		super.removeNotify();
		if (pnlOkCancel != null) {
			for (int i = pnlOkCancel.getComponentCount() - 1; i >= 0; i--) {
				((JButton) pnlOkCancel.getComponent(i)).removeActionListener(
					this);
			}
		}
	}

	JPanel pnlOkCancel;

	/** @return the panel that contains action buttons */
	public JPanel getButtonPanel() {
		return pnlOkCancel;
	}

	/** Add-on panel button */
	protected JPanel getAddonPanel() {
		return null;
	}

	private void init(String dialogTitle, String title, String comment) {
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		setModal(true);
		init(title, comment);
		setSize(400, 400);
		setTitle(dialogTitle);

		getRootPane().registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				OkCancelDialog.this.actionPerformed(
					new ActionEvent(e.getSource(), e.getID(), CANCEL_ACTION));
			}
		},
			KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false),
			JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
	}

	public static String DEFAULT_ICON =
		"images/OkCancelDialog.gif";

	/** @return the default classpath path for the icon */
	protected String getDefaultIcon() {
		return DEFAULT_ICON;
	}

	private void init(String title, String comment) {
		String icon = getDefaultIcon();
		try {
			if (BUNDLE) {
				ResourceBundle resource = ResourceBundle.getBundle(getName());
				try {
					title = resource.getString("TITLE");
				} catch (MissingResourceException th) {
				}
				try {
					comment = resource.getString("COMMENT");
				} catch (MissingResourceException th) {
				}
				try {
					actions[0] = resource.getString("OK");
				} catch (MissingResourceException th) {
				}
				try {
					actions[1] = resource.getString("CANCEL");
				} catch (MissingResourceException th) {
				}
				try {
					icon = resource.getString("ICON");
				} catch (MissingResourceException th) {
				}
			}
		} catch (Throwable th) {
		}

		pnlOkCancel = new JPanel(); // JPF
		JButton btn;

		for (int i = 0; i < actions.length; i++) {
			btn = new JButton(actions[i]);
			if (i == 0) {
				getRootPane().setDefaultButton(btn);
			}
			btn.addActionListener(this);
			btn.setActionCommand(actions[i]);
			pnlOkCancel.add(btn);
		}

		//pnlOkCancel.setBorder(new TitledBorder(""));

		JPanel pnlHeader = new JPanel();
		pnlHeader.setBackground(Color.WHITE);
		pnlHeader.setLayout(new BoxLayout(pnlHeader, BoxLayout.Y_AXIS));
		JLabel lblTitle = new JLabel(title);
		lblTitle.setFont(new Font("dialog", Font.BOLD, 14));
		lblTitle.setHorizontalAlignment(SwingConstants.CENTER);
		lblTitle.setHorizontalTextPosition(SwingUtilities.CENTER);

		JTextArea txtComment = new JTextArea();
		txtComment.setWrapStyleWord(true);
		txtComment.setEditable(false);
		txtComment.setFont(new Font("dialog", Font.PLAIN, 10));
		txtComment.setText(comment);

		JPanel pnlTitle = new JPanel();
		pnlTitle.add(lblTitle);
		pnlTitle.setBackground(Color.WHITE);

		JPanel pnlHeaderIcon = new JPanel();
		pnlHeaderIcon.setBackground(Color.WHITE);
		pnlHeaderIcon.setLayout(new BorderLayout());
		JLabel _;

		URL url = ClassLoader.getSystemClassLoader().getResource(icon);

		if ( url != null )
		pnlHeaderIcon.add(
			_ =
				new JLabel(
					new ImageIcon( url 
						)),
			BorderLayout.EAST);
		pnlHeaderIcon.add(pnlHeader);

		pnlHeader.add(pnlTitle);
		pnlHeader.add(txtComment);

		pnlHeaderIcon.setBorder(new TitledBorder(""));

		super.getContentPane().add(pnlHeaderIcon, BorderLayout.NORTH);
		super.getContentPane().add(getContentPane(), BorderLayout.CENTER);

		JPanel _pnl = new JPanel();
		_pnl.setLayout(new BorderLayout());
				
		_pnl.add(new JSeparator( JSeparator.HORIZONTAL ), BorderLayout.NORTH);

		JPanel __pnl = new JPanel();
		__pnl.setLayout( new BorderLayout() );
		__pnl.add( pnlOkCancel, BorderLayout.EAST );
		
		if ( getAddonPanel() != null ) {
			__pnl.add( getAddonPanel(), BorderLayout.WEST );
		}

		_pnl.add( __pnl, BorderLayout.CENTER );

		super.getContentPane().add(_pnl, BorderLayout.SOUTH);
	}

	public Container getContentPane() {
		if (mainPanel == null) {
			mainPanel = new JPanel();
			mainPanel.setLayout(new BorderLayout());
		}
		Border b = createContentBorder();
		if (b != null)
			mainPanel.setBorder(b);
		return mainPanel;
	}

	/** @return a border for dialog content. By default none */
	protected Border createContentBorder() {
		return null;
	}

	protected Rectangle getLocationAndSize() {
		Dimension dim = getToolkit().getScreenSize();
		Rectangle r = 
			new Rectangle(
			( dim.width - getWidth() ) / 2,
			( dim.height - getHeight() ) / 2,
			getWidth(), 
			getHeight() );
		return r;
	}

	/** Call each time the non visible state is needed */
	protected void preNonVisible() {
	}

	public void setVisible( boolean state ) {
		if ( state ) {
			setBounds( getLocationAndSize() );
		} else
			preNonVisible();
		super.setVisible( state );
	}

	/** Show this dialog box without an auto center location */
	public void setVisibleWithoutAutoLocation(boolean state) {
		super.setVisible(state);
	}

	private String action;

	public void actionPerformed(ActionEvent evt) {
		action = evt.getActionCommand();
		if (isOk() || isCancel()) {
			setVisible(false);
			dispose();
		}
	}

	public String getAction() {
		return action;
	}

	public boolean isOk() {
		return OK_ACTION.equals(getAction());
	}

	public boolean isCancel() {
		return CANCEL_ACTION.equals(getAction());
	}

	/** @return the name of this dialog box */
	public String getName() {
		return getClass().getName();
	}

	public static void main(String[] args) {
		OkCancelDialog ok =
			new OkCancelDialog("", "Titre", "COMMENT GREAT\nOK .........");
		ok.setVisible(true);
	}

}
