package com.japisoft.xmlpad.elementview.table;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.table.*;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
class FastTableCellRenderer extends JComponent implements TableCellRenderer {

	private boolean selected;
	private Dimension d = new Dimension(100, 25);
	private int textY;
	private int fmHeight;
	FontMetrics fm;

	public FastTableCellRenderer() {
		super();
		setFont(
			UIManager.getFont( "xmlpad.tableElementView.font" ) );		
		setColorForPrefixName(
			UIManager.getColor( "xmlpad.tableElementView.prefixNameColor" ) );
		setHighlightColor( 
			UIManager.getColor( "xmlpad.tableElementView.highlightColor" ) );
		setLowlightColor( 
			UIManager.getColor( "xmlpad.tableElementView.lowlightColor" ) );
	}

	public void setFont( Font font ) {
		if ( font == null )
			font = new Font( null, 0, 14 );
		super.setFont( font );
		fm = getFontMetrics( font );
		fmHeight = fm.getHeight() + 2;
		textY = fm.getAscent();
	}

	private Color prefixNameColor;

	/** Reset the color for the prefix and name part */
	public void setColorForPrefixName( Color color ) {
		if ( color == null )
			color = new Color( 230, 230, 230 );
		this.prefixNameColor = color;
	}

	private Color highlightColor = null;

	public void setHighlightColor( Color color ) {
		if  (color == null )
			color = new Color( 255, 255, 255 );
		this.highlightColor = color;
	}

	private Color lowlightColor = null;

	public void setLowlightColor( Color color ) {
		if ( color == null )
			color = new Color( 240, 240, 240 );
		this.lowlightColor = color;
	}

	String content = null;
	int row = 0;

	public Component getTableCellRendererComponent(
		JTable table,
		Object value,
		boolean isSelected,
		boolean hasFocus,
		int row,
		int column) {
		content = ( String )value;
		this.selected = isSelected;
		this.row = row;
		
		if  ( row < 2 )
			setBackground( prefixNameColor );
		else
			setBackground( row % 2 == 0 ? highlightColor : lowlightColor );
		
		return this;
	}

	private Color textColor = Color.black;
	
	public void setTextColor( Color c ) {
		this.textColor = c;
	}

	public void paintComponent( Graphics gc ) {
		super.paintComponent( gc );

		Graphics2D g2 = ( Graphics2D )gc;

		g2.setRenderingHint( 
				RenderingHints.KEY_TEXT_ANTIALIASING, 
				RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB 
			);				

		gc.setColor( getBackground() );
		gc.fillRect( 0, 0, getWidth(), getHeight() );

		if ( content != null ) {
			gc.setColor( textColor );
			gc.drawString( content, 0, textY );
		}
	}
	
	// For optimizing rendering
	public boolean isDoubleBuffered() {
		return false;
	}
	// For optimizing rendering		
	public void invalidate() {
	}
	// For optimizing rendering		
	public void validate() {
	}
	// For optimizing rendering		
	public void repaint() {
	}

}
