package com.japisoft.xmlpad.helper.handler.schema.dtd;

import com.japisoft.dtdparser.node.ElementDTDNode;
import com.japisoft.dtdparser.node.RootDTDNode;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.helper.handler.schema.AbstractTagHandler;
import com.japisoft.xmlpad.helper.model.SchemaNodable;
import com.japisoft.xmlpad.helper.model.SchemaNode;
import com.japisoft.xmlpad.helper.model.TagDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DTDTagHandler extends AbstractTagHandler 
		implements SchemaNodable {
	private RootDTDNode root;
	private String rootElement;

	public DTDTagHandler( 
			String rootElement, 
			RootDTDNode root ) {
		this.root = root;
		this.rootElement = rootElement;
	}

	public void dispose() {
		super.dispose();
		this.root = null;
	}	

	public TagDescriptor getTag( FPNode node ) {
		if ( node == null )
			return null;
		String name = node.getContent();
		ElementDTDNode element = root.getElementDeclaration( name );
		if ( element != null )
			return new DTDTagDescriptor( element );
		return null;
	}

	protected void notifyLocation() {

		if ( root == null )
			return;

		schemaNode = null;
		
		if ( currentDocumentNode == null ) {
			// Add the rootElement

			if ( rootElement != null ) {
				ElementDTDNode node = root.getElementDeclaration( rootElement );
				if ( node != null ) {
					addTagDescriptor( new DTDTagDescriptor( node ) );
				}
			}

			return;
		}

		String name = currentDocumentNode.getContent();
		
		// Search for the DTD declaration

		ElementDTDNode node = root.getElementDeclaration( name );

		if ( node == null || 
				node.isEmptyElement() || 
					node.hasPCDATA() )
			return;

		schemaNode = ( new DTDToSchemaNode() ).getSchemaNode( node );
	}

	protected void completeContentForElementWithoutPrefix() {
		super.completeContentForElementWithoutPrefix();
		if ( rootElement != null && rootElement.indexOf( ":" ) > 0 ) {
			for ( int i = 0; i < root.getDTDNodeCount(); i++ ) {
				if ( root.getDTDNodeAt( i ).isElement() ) {
					ElementDTDNode node = (ElementDTDNode)root.getDTDNodeAt( i );
					if ( node.getName().indexOf( ":" ) == -1 )
						addTagDescriptor( new DTDTagDescriptor( node ) );
				}
			}
		}
	}
	
	private SchemaNode schemaNode = null;

	public SchemaNode getSchemaNode() {
		return schemaNode;
	}

}
