package com.japisoft.xmlpad.helper.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import com.japisoft.xmlpad.SharedProperties;
import com.japisoft.xmlpad.helper.handler.AbstractHelperHandler;
import com.japisoft.xmlpad.helper.handler.schema.AbstractTagHandler;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SchemaCache {

	private static HashMap htTagHelper = null;

	public static AbstractHelperHandler[] restoreHelperHandlers( Object source, String locationKey ) {
		if ( htTagHelper == null || 
				!SharedProperties.SCHEMA_CACHING )
			return null;
		ReferenceWrapper rw = ( ReferenceWrapper )htTagHelper.get( locationKey );
		if ( rw == null )
			return null;
		return rw.handlers;
	}

	public static void addNewReference( 
			AbstractHelperHandler[] handlers,
			String locationKey ) {

		if ( !SharedProperties.SCHEMA_CACHING )
			return;
		
		if ( locationKey != null ) {
			if ( htTagHelper == null )
				htTagHelper = new HashMap();
						
			htTagHelper.put( locationKey, 
					new ReferenceWrapper( handlers ) );
		}
	}

	public static void removeReference( Object source, AbstractHelperHandler[] handlers ) {
		
		if ( !SharedProperties.SCHEMA_CACHING )
			return;
				
		if ( htTagHelper != null && 
				handlers != null ) {
			Collection keys = htTagHelper.keySet();
			Iterator it = keys.iterator();
			while ( it.hasNext() ) {
				String loc = ( String )it.next();
				ReferenceWrapper rw = ( ReferenceWrapper )htTagHelper.get( loc );
				if ( compareArray( rw.handlers , handlers ) ) {
					if ( rw.removeSource( source ) ) {
						htTagHelper.remove( loc );
						break;
					}
				}
			}
		}
	}

	private static boolean compareArray( AbstractHelperHandler[] a1, AbstractHelperHandler[] a2 ) {
		for ( int i = 0; i < a1.length; i++ ) {
			if ( a1[ i ] != a2[ i ] )
				return false;
		}
		return true;
	}
	
	// ------------------------------------------------------------------
	
	static class ReferenceWrapper {
		public AbstractHelperHandler[] handlers = null;

		public ReferenceWrapper( AbstractHelperHandler[] handlers ) {
			this.handlers = handlers;
		}
		
		private ArrayList sources = null;
		
		public void addSource( Object source ) {
			if ( sources == null )
				sources = new ArrayList();
			sources.add( new Integer( source.hashCode() ) );
		}
		
		public boolean removeSource( Object source ) {
			if ( sources == null )
				return false;
			return sources.remove( new Integer( source.hashCode() ) );
		}
		
	}

}
 
