package com.japisoft.xmlpad.toolkit;

import java.util.ArrayList;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xpath.NodeSet;
import com.japisoft.xpath.XPath;
import com.japisoft.xpath.XPathException;
import com.japisoft.xpath.kit.FastParserKit;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XPathHandler {

	private ArrayList l;
	private int index = -1;
	private XMLContainer container;
	
	public XPathHandler( XMLContainer container, String xpathExpression, boolean relative ) throws XPathException {
		this.container = container;

		FPNode root = null;
		if ( !relative )
			root = container.getRootNode();
		else {
			root = container.getCurrentNode();
			if ( root == null )
				root = container.getRootNode();
		}

		if ( root == null )
			throw new XPathException( "No document", 0 );

		XPath xpath = new XPath( 
				new FastParserKit() );
		xpath.setXPathExpression( xpathExpression );
		xpath.setReferenceNode( root );

		res = xpath.resolve();

		if ( res instanceof NodeSet ) {
			NodeSet ns = ( NodeSet )res;
			l = new ArrayList();
			for ( int i = 0; i < ns.size(); i++ ) {
				FPNode sn = ( FPNode )ns.get( i );
				
				if ( sn.getType() == FPNode.COMMENT_NODE + 1 ) {
					if ( ns.size() == 1 ) {
						res = sn;
						return;
					}
				}
				
				l.add( sn );
			}
			container.getEditor().highlightNodes( l );
			selectNext();
		}
	}

	Object res;

	
	/** @return <code>true</code> if the result type is a nodeset */
	public boolean isResultNodesType() {
		return res instanceof NodeSet;
	}
	
	/** @return <code>true</code> if the result type is an attribute */
	public boolean isAttributeType() {
		return res instanceof FPNode;
	}

	/** @return the attribute value for an attribute type result */
	public String getAttributeValue() {
		if ( res instanceof FPNode ) {
			FPNode sn = ( FPNode )res;
			String attname = sn.getNodeContent();
			FPNode p = sn.getFPParent();
			return p.getAttribute( attname );
		}
		return null;
	}

	/** @return the xpath result. This is useful only for a non nodeset type */
	public Object getRawResult() { return res; }
	
	/**
	 * @return <code>true</code> if the previous xpath expression has result node
	 */
	public boolean hasResultNodes() {
		return ( l.size() > 0 );
	}

	/** 
	 * Select the next node
	 */
	public boolean selectNext() {
				
		if ( l.size() > 0 && 
				( index + 1 ) < l.size() ) {
			index++;
			FPNode node = ( FPNode )l.get( index );
			int line = node.getStartingLine();
			container.getEditor().highlightLine( line );
			return ( index + 1 < l.size() );
		} else
			return false;
	}

	/**
	 * Select the previous node
	 */
	public boolean selectPrevious() {
		if ( index > 0 ) {
			index--;
			FPNode node = ( FPNode )l.get( index );
			int line = node.getStartingLine();
			container.getEditor().highlightLine( line );			
			return ( index > 0 );
		} else
			return false;		
	}

}
