package com.japisoft.xpath;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FastVector {

	Object elementData[];
	int elementCount;
	int capacityIncrement;

	public FastVector(int initialCapacity, int capacityIncrement) {
		super();
		this.elementData = new Object[initialCapacity];
		this.capacityIncrement = capacityIncrement;
	}

	public FastVector(int initialCapacity) {
		this(initialCapacity, 0);
	}

	public FastVector() {
		this(5);
	}

	public final boolean remove(Object obj) {
		int i = indexOf(obj);
		if (i >= 0) {
			removeElementAt(i);
			return true;
		}
		return false;
	}

	public final void removeElementAt(int index) {
		if (index >= elementCount) {
			throw new ArrayIndexOutOfBoundsException(
				index + " >= " + elementCount);
		} else if (index < 0) {
			throw new ArrayIndexOutOfBoundsException(index);
		}
		int j = elementCount - index - 1;
		if (j > 0) {
			System.arraycopy(elementData, index + 1, elementData, index, j);
		}
		elementCount--;
		elementData[elementCount] = null;
	}

	public final void insertElementAt(Object obj, int index) {
		int newcount = elementCount + 1;
		if (index >= newcount) {
			throw new ArrayIndexOutOfBoundsException(
				index + " > " + elementCount);
		}
		if (newcount > elementData.length) {
			ensureCapacityHelper(newcount);
		}
		System.arraycopy(
			elementData,
			index,
			elementData,
			index + 1,
			elementCount - index);
		elementData[index] = obj;
		elementCount++;
	}

	public final void ensureCapacity(int minCapacity) {
		if (minCapacity > elementData.length) {
			ensureCapacityHelper(minCapacity);
		}
	}

	private void ensureCapacityHelper(int minCapacity) {
		int oldCapacity = elementData.length;
		Object oldData[] = elementData;
		int newCapacity =
			(capacityIncrement > 0)
				? (oldCapacity + capacityIncrement)
				: (oldCapacity * 2);
		if (newCapacity < minCapacity) {
			newCapacity = minCapacity;
		}
		elementData = new Object[newCapacity];
		System.arraycopy(oldData, 0, elementData, 0, elementCount);
	}

	public final int size() {
		return elementCount;
	}

	public final int indexOf(Object elem) {
		return indexOf(elem, 0);
	}

	public final int indexOf(Object elem, int index) {
		for (int i = index; i < elementCount; i++) {
			if (elem.equals(elementData[i])) {
				return i;
			}
		}
		return -1;
	}

	public final Object get(int index) {
		return elementData[index];
	}

	public final Object elementAt(int index) {
		return elementData[index];
	}

	public final boolean contains(Object obj) {
		return indexOf(obj) >= 0;
	}

	public final void addElement(Object obj) {
		int newcount = elementCount + 1;
		if (newcount > elementData.length) {
			ensureCapacityHelper(newcount);
		}
		elementData[elementCount++] = obj;
	}
}
