// (c) ALEXANDRE BRILLANT : http://www.japisoft.com
// All this work is confidential, you have rights to
// change and evolve it for your products but you
// have no rights to sell it, propose concurrent works.
// Morever any changes to bugs or evolutions should
// be send to JAPISOFT that needs to maintain a
// valid version and has all rights on the product.


package com.japisoft.xpath.function;

import com.japisoft.xpath.NodeSet;
import com.japisoft.xpath.XPathContext;
import com.japisoft.xpath.FastVector;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public abstract class AbstractFunction implements Function {
    public AbstractFunction() {
	super();
    }

    /** Evalute this function for this context and the following arguments */
    public abstract Object eval( XPathContext context, FastVector arg );

    /** Throw a RuntimeException if arg has no single NodeSet argument */
    public void checkOneNumber( FastVector arg ) {
	checkOneParameter( arg );
	if ( !( arg.elementAt( 0 ) instanceof Double ) )
	    throw new RuntimeException( "Invalid parameter in " + this );
    }

    /** Throw a RuntimeException if arg has no single NodeSet argument */
    public void checkOneNodeSet( FastVector arg ) {
	checkOneParameter( arg );
	if ( !( arg.elementAt( 0 ) instanceof NodeSet ) )
	    throw new RuntimeException( "Invalid parameter in " + this );
    }

    /** Throw a RuntimeException if arg has no single argument */
    public void checkOneParameter( FastVector arg ) {
	if ( arg.size() != 1 )
	    throw new RuntimeException( "Invalid parameter in " + this );
    }

    /** Throw a RuntimeException if arg has no 1 string argument */
    public void checkOneString( FastVector arg ) {
	if ( arg.size() != 1 )
	    throw new RuntimeException( "Invalid parameters in " + this );
	if ( !( arg.elementAt( 0 ) instanceof String ) )
	    throw new RuntimeException( "Invalid parameters in " + this );
    }

    /** Throw a RuntimeException if arg has no 2 string arguments */
    public void checkTwoStrings( FastVector arg ) {
	if ( arg.size() != 2 )
	    throw new RuntimeException( "Invalid parameters in " + this );
	if ( !( ( arg.elementAt( 0 ) instanceof String ) &&
		arg.elementAt( 1 ) instanceof String ) )
	    throw new RuntimeException( "Invalid parameters in " + this );
    }

    /** Throw a RuntimeException if arg has no 3 string arguments */
    public void checkThreeStrings( FastVector arg ) {
	if ( arg.size() != 3 )
	    throw new RuntimeException( "Invalid parameters in " + this );
	if ( !( ( arg.elementAt( 0 ) instanceof String ) &&
		( arg.elementAt( 1 ) instanceof String ) &&
		( arg.elementAt( 2 ) instanceof String ) ) )
	    throw new RuntimeException( "Invalid parameters in " + this );
    }

    /** @return only the class name without the package */
    public String toString() {
	return getClass().getName().substring( getClass().getName().lastIndexOf( "." ) );
    }

    public java.lang.String getString1( XPathContext context, FastVector arg ) {
      return context.getStringValue( arg.elementAt( 1 ) );
    }

    public java.lang.String getString2( XPathContext context, FastVector arg ) {
      return context.getStringValue( arg.elementAt( 0 ) );
    }

}

// AbstractFunction ends here
