// (c) ALEXANDRE BRILLANT : http://www.japisoft.com
// All this work is confidential, you have rights to
// change and evolve it for your products but you
// have no rights to sell it, propose concurrent works.
// Morever any changes to bugs or evolutions should
// be send to JAPISOFT that needs to maintain a
// valid version and has all rights on the product.

package com.japisoft.xpath.kit;

import com.japisoft.xpath.function.basic.FunctionLib;
import com.japisoft.xpath.function.Lib;
import com.japisoft.xpath.navigator.DOMNavigator;
import com.japisoft.xpath.AbstractKit;
import com.japisoft.xpath.NodeSet;
import com.japisoft.xpath.Navigator;

import org.w3c.dom.*;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DOMKit
    extends AbstractKit {

  /** Feature for ignoring lower/upper case, by default to false */
  public final static String IGNORE_CASE_FEATURE = "http://www.japisoft.com/jxp/dom/ignorecase";

  public DOMKit() {
    super();
    addFeature( IGNORE_CASE_FEATURE, false);
  }

	/** Replace a DOM document by its root */
  public Object getBetterReferenceNode( Object ref ) {
		if ( ref instanceof Document ) {
			return ( (Document)ref ).getDocumentElement();
		}  	
		return ref;
  }

  /** @return the library resolver. If <code>null</code> is returned then the standard library is used */
  public Lib getLibrary() {
    return new FunctionLib();
  }

  private Navigator cache;

  /** @return the tree navigator toolkit */
  public Navigator getNavigator() {
    if (cache != null) {
      return cache;
    }
    cache = new DOMNavigator();
    if ( isFeatureSupported( IGNORE_CASE_FEATURE ) )
      ( (DOMNavigator)cache ).setIgnoreCaseMode( true );
    return cache;
  }

  // Particular method for the string-value on node
  private void findSubTextNode(Node n, StringBuffer res) {
    NodeList nl = n.getChildNodes();
    for (int i = 0; i < nl.getLength(); i++) {
      Node child = nl.item(i);
      if (child.getNodeType() == Node.TEXT_NODE) {
        res.append(child.getNodeValue());
      }
    }
    for (int i = 0; i < nl.getLength(); i++) {
      Node child = nl.item(i);
      if (child.hasChildNodes() && child.getNodeType() == Node.ELEMENT_NODE) {
        findSubTextNode(child, res);
      }
    }
  }

  /** Compute the string-value for this node */
  public String getStringValue(Object node) {
    if (node instanceof Node) {
      Node n = (Node) node;
      if (n.getNodeType() == Node.ELEMENT_NODE) {
        // Particular cas for node : get all sub text node
        StringBuffer sb = new StringBuffer();
        findSubTextNode(n, sb);
        return sb.toString();
      }
      else {
        return n.getNodeValue();
      }
    }
    else
    if (node instanceof NodeSet) {
      NodeSet ns = (NodeSet) node;
      if (ns.size() > 0) {
        Node n = (Node) ns.elementAt(0);
        return getStringValue(n);
      }
    }
    return node.toString();
  }

  /** Compute the local name of the node */
  public String getLocalName(Object node) {
    Node n = (Node) node;
    String s = n.getLocalName();
    if (s == null) {
      s = n.getNodeName();
    }
    if (s == null) {
      return "";
    }
    return s;
  }

  /** Compute the namespace URI for this node */
  public String getNamespaceURI(Object node) {
    Node n = (Node) node;
    return n.getNamespaceURI();
  }

  /** Compute the qualified name for this node */
  public String getName(Object node) {
    Node n = (Node) node;
    return n.getNodeName();
  }

  /** Compute the language for this node */
  public String getLang(Object node) {
    if (node instanceof Element) {
      Element e = (Element) node;
      String l = e.getAttribute("xml:lang");
      if ("".equals(l)) {
        if (e.getParentNode() != null) {
          return getLang(e.getParentNode());
        }
        else {
          return null;
        }
      }
      else {
        return l;
      }
    }
    else {
      return null;
    }
  }

  /**
   * @param refNode a reference document element
   * @param id ID value to match
   * @return the node with the unique ID. The ID scope is theorically
   * limited to attribute defined as ID in the DTD
   */
  public Object getNodeForId(Object refNode, String id) {
    Document doc = ( (Node) refNode).getOwnerDocument();
    if (doc == null) {
      throw new RuntimeException("No owner document for node " + refNode + " ?");
    }
    return doc.getElementById(id);
  }

}
